@@ -10,68 +10,57 @@
 #ifndef __AFSD_H_ENV__
 #define __AFSD_H_ENV__ 1
 
+#define USE_BPLUS 1
+
 #include <afs/param.h>
 
-#ifndef DJGPP
 BOOL InitClass(HANDLE);
 BOOL InitInstance(HANDLE, int);
 
 LONG APIENTRY MainWndProc(HWND, unsigned int, unsigned int, long);
 BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
-#endif /* !DJGPP */
 
-#ifndef DJGPP
 #include <nb30.h>
-#else /* DJGPP */
-#include <sys/farptr.h>
-#include <go32.h>
-#include "dosdefs95.h"
-#include "largeint95.h"
-#endif /* !DJGPP */
-
-#include "afsdicon.h"
 
 #include "cm.h"
-
-#include "krb.h"
-#include "krb_prot.h"
-/*#include <crypt.h>*/
-#include <afs/prs_fs.h>
+#include "cm_nls.h"
 
 #include <osi.h>
+#include <afs/vldbint.h>
+#include <afs/afsint.h>
+#include <afs/prs_fs.h>
+
 #include "cm_config.h"
 #include "cm_user.h"
+#include "cm_scache.h"
 #include "cm_callback.h"
 #ifdef DISKCACHE95
 #include "cm_diskcache95.h"
 #endif /* DISKCACHE95 */
 #include "cm_conn.h"
 #include "cm_aclent.h"
-#include "cm_cell.h"
 #include "cm_server.h"
-#include "cm_scache.h"
+#include "cm_cell.h"
+#include "cm_volstat.h"
 #include "cm_volume.h"
 #include "cm_dcache.h"
 #include "cm_access.h"
+#include "cm_dir.h"
 #include "cm_utils.h"
 #include "cm_vnodeops.h"
-#include "cm_dir.h"
+#include "cm_btree.h"
 #include "cm_daemon.h"
 #include "cm_ioctl.h"
 #include "cm_dnlc.h"
 #include "cm_buf.h"
 #include "cm_memmap.h"
 #include "cm_freelance.h"
+#include "cm_performance.h"
 #include "smb_ioctl.h"
+#include "smb_iocons.h"
 #include "afsd_init.h"
-#ifdef DJGPP
-#include "afs/afsmsg95.h"
-#else
 #include "afsd_eventlog.h"
-#endif
 
-#include <afs/vldbint.h>
-#include <afs/afsint.h>
 
 #define AFS_DAEMON_SERVICE_NAME AFSREG_CLT_SVC_NAME
 #define AFS_DAEMON_EVENT_NAME   AFSREG_CLT_SW_NAME
@@ -95,9 +84,12 @@ extern cm_scache_t *cm_rootSCachep;
 
 extern osi_log_t *afsd_logp;
 
-extern char cm_mountRoot[];
+extern fschar_t cm_mountRoot[];
 extern DWORD cm_mountRootLen;
 
+extern clientchar_t cm_mountRootC[];
+extern DWORD cm_mountRootCLen;
+
 extern char cm_CachePath[];
 
 extern BOOL isGateway;
@@ -114,12 +106,10 @@ extern cm_localMountPoint_t* cm_localMountPoints;	// array of fake mountpoints
 extern int cm_fakeDirSize;				// size (in bytes) of fake root.afs directory
 
 extern int cm_fakeDirCallback;				// state of the fake root.afs directory. indicates
-													// if it needs to be refreshed
+                                                        // if it needs to be refreshed
 
 extern int cm_fakeGettingCallback;			// 1 if currently updating the fake root.afs directory,
-													// 0 otherwise
-
-extern int cm_fakeDirVersion;				// the version number of the root.afs directory. used 
+							// 0 otherwise
 #endif /* AFS_FREELANCE_CLIENT */
 
 extern int cm_dnsEnabled;
@@ -129,7 +119,7 @@ extern long rx_mtu;
 
 extern HANDLE WaitToTerminate;
 
-#undef  DFS_SUPPORT
+#define DFS_SUPPORT 1
 #define LOG_PACKET 1
 #undef  NOTSERVICE
 #define LOCK_TESTING 1
