@@ -213,73 +213,86 @@ void buf_Release(cm_buf_t *bp)
     }
 }
 
-/* incremental sync daemon.  Writes all dirty buffers every 5000 ms */
-void buf_IncrSyncer(long parm)
+long 
+buf_Sync(int quitOnShutdown) 
 {
     cm_buf_t **bpp, *bp, *prevbp;
-    long i;				/* counter */
     long wasDirty = 0;
     cm_req_t req;
 
+    /* go through all of the dirty buffers */
+    lock_ObtainRead(&buf_globalLock);
+    for (bpp = &cm_data.buf_dirtyListp, prevbp = NULL; bp = *bpp; ) {
+        if (quitOnShutdown && buf_ShutdownFlag)
+            break;
+
+        lock_ReleaseRead(&buf_globalLock);
+        /* all dirty buffers are held when they are added to the
+        * dirty list.  No need for an additional hold.
+        */
+        lock_ObtainMutex(&bp->mx);
+
+        if (bp->flags & CM_BUF_DIRTY) {
+            /* start cleaning the buffer; don't touch log pages since
+            * the log code counts on knowing exactly who is writing
+            * a log page at any given instant.
+            */
+            cm_InitReq(&req);
+            req.flags |= CM_REQ_NORETRY;
+            wasDirty |= buf_CleanAsyncLocked(bp, &req);
+        }
+
+        /* the buffer may or may not have been dirty
+        * and if dirty may or may not have been cleaned
+        * successfully.  check the dirty flag again.  
+        */
+        if (!(bp->flags & CM_BUF_DIRTY)) {
+            /* remove the buffer from the dirty list */
+            lock_ObtainWrite(&buf_globalLock);
+#ifdef DEBUG_REFCOUNT
+            if (bp->dirtyp == NULL && bp != cm_data.buf_dirtyListEndp) {
+                osi_Log1(afsd_logp,"buf_IncrSyncer bp 0x%p list corruption",bp);
+                afsi_log("buf_IncrSyncer bp 0x%p list corruption", bp);
+            }
+#endif
+            *bpp = bp->dirtyp;
+            bp->dirtyp = NULL;
+            bp->flags &= ~CM_BUF_INDL;
+            if (cm_data.buf_dirtyListp == NULL)
+                cm_data.buf_dirtyListEndp = NULL;
+            else if (cm_data.buf_dirtyListEndp == bp)
+                cm_data.buf_dirtyListEndp = prevbp;
+            buf_ReleaseLocked(bp, TRUE);
+            lock_ConvertWToR(&buf_globalLock);
+        } else {
+            /* advance the pointer so we don't loop forever */
+            lock_ObtainRead(&buf_globalLock);
+            bpp = &bp->dirtyp;
+            prevbp = bp;
+        }
+        lock_ReleaseMutex(&bp->mx);
+    }	/* for loop over a bunch of buffers */
+    lock_ReleaseRead(&buf_globalLock);
+
+    return wasDirty;
+}
+
+/* incremental sync daemon.  Writes all dirty buffers every 5000 ms */
+void buf_IncrSyncer(long parm)
+{
+    long wasDirty = 0;
+    long i;
+
     while (buf_ShutdownFlag == 0) {
-	if (!wasDirty) {
+
+        if (!wasDirty) {
 	    i = SleepEx(5000, 1);
-	    if (i != 0) continue;
+	    if (i != 0) 
+                continue;
 	}
 
-	wasDirty = 0;
-
-        /* go through all of the dirty buffers */
-        lock_ObtainRead(&buf_globalLock);
-        for (bpp = &cm_data.buf_dirtyListp, prevbp = NULL; bp = *bpp; ) {
-            lock_ReleaseRead(&buf_globalLock);
-	    /* all dirty buffers are held when they are added to the
-	     * dirty list.  No need for an additional hold.
-	     */
-            lock_ObtainMutex(&bp->mx);
-
-	    if (bp->flags & CM_BUF_DIRTY) {
-		/* start cleaning the buffer; don't touch log pages since
- 		 * the log code counts on knowing exactly who is writing
-		 * a log page at any given instant.
-		 */
-		cm_InitReq(&req);
-		req.flags |= CM_REQ_NORETRY;
-		wasDirty |= buf_CleanAsyncLocked(bp, &req);
-	    }
-
-	    /* the buffer may or may not have been dirty
-	     * and if dirty may or may not have been cleaned
-	     * successfully.  check the dirty flag again.  
-	     */
-            if (!(bp->flags & CM_BUF_DIRTY)) {
-                /* remove the buffer from the dirty list */
-                lock_ObtainWrite(&buf_globalLock);
-#ifdef DEBUG_REFCOUNT
-                if (bp->dirtyp == NULL && bp != cm_data.buf_dirtyListEndp) {
-                    osi_Log1(afsd_logp,"buf_IncrSyncer bp 0x%p list corruption",bp);
-                    afsi_log("buf_IncrSyncer bp 0x%p list corruption", bp);
-                }
-#endif
-                *bpp = bp->dirtyp;
-                bp->dirtyp = NULL;
-                bp->flags &= ~CM_BUF_INDL;
-                if (cm_data.buf_dirtyListp == NULL)
-                    cm_data.buf_dirtyListEndp = NULL;
-                else if (cm_data.buf_dirtyListEndp == bp)
-                    cm_data.buf_dirtyListEndp = prevbp;
-                buf_ReleaseLocked(bp, TRUE);
-                lock_ConvertWToR(&buf_globalLock);
-            } else {
-                /* advance the pointer so we don't loop forever */
-                lock_ObtainRead(&buf_globalLock);
-                bpp = &bp->dirtyp;
-                prevbp = bp;
-            }
-            lock_ReleaseMutex(&bp->mx);
-        }	/* for loop over a bunch of buffers */
-        lock_ReleaseRead(&buf_globalLock);
-    }		/* whole daemon's while loop */
+        wasDirty = buf_Sync(1);
+    } /* whole daemon's while loop */
 }
 
 long
@@ -359,8 +372,12 @@ buf_ValidateBuffers(void)
 }
 
 void buf_Shutdown(void)  
-{                        
+{  
+    /* disable the buf_IncrSyncer() threads */
     buf_ShutdownFlag = 1;
+
+    /* then force all dirty buffers to the file servers */
+    buf_Sync(0);
 }                        
 
 /* initialize the buffer package; called with no locks
@@ -389,7 +406,7 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
 
     if (osi_Once(&once)) {
         /* initialize global locks */
-        lock_InitializeRWLock(&buf_globalLock, "Global buffer lock");
+        lock_InitializeRWLock(&buf_globalLock, "Global buffer lock", LOCK_HIERARCHY_BUF_GLOBAL);
 
         if ( newFile ) {
             /* remember this for those who want to reset it */
@@ -424,7 +441,7 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
                 
                 osi_QAdd((osi_queue_t **)&cm_data.buf_freeListp, &bp->q);
                 bp->flags |= CM_BUF_INLRU;
-                lock_InitializeMutex(&bp->mx, "Buffer mutex");
+                lock_InitializeMutex(&bp->mx, "Buffer mutex", LOCK_HIERARCHY_BUFFER);
                 
                 /* grab appropriate number of bytes from aligned zone */
                 bp->datap = data;
@@ -448,7 +465,7 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
             data = cm_data.bufDataBaseAddress;
             
             for (i=0; i<cm_data.buf_nbuffers; i++) {
-                lock_InitializeMutex(&bp->mx, "Buffer mutex");
+                lock_InitializeMutex(&bp->mx, "Buffer mutex", LOCK_HIERARCHY_BUFFER);
                 bp->userp = NULL;
                 bp->waitCount = 0;
                 bp->waitRequests = 0;
@@ -516,7 +533,7 @@ long buf_AddBuffers(afs_uint64 nbuffers)
     for (i=0; i<nbuffers; i++) {
         memset(bp, 0, sizeof(*bp));
         
-        lock_InitializeMutex(&bp->mx, "cm_buf_t");
+        lock_InitializeMutex(&bp->mx, "cm_buf_t", LOCK_HIERARCHY_BUFFER);
 
         /* grab appropriate number of bytes from aligned zone */
         bp->datap = data;
@@ -722,7 +739,7 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
             bp->dirty_offset = 0;
             bp->dirty_length = 0;
 	    bp->error = code;
-	    bp->dataVersion = CM_BUF_VERSION_BAD; /* bad */
+	    bp->dataVersion = CM_BUF_VERSION_BAD;
 	    bp->dirtyCounter++;
             break;
 	}
@@ -998,24 +1015,25 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
             osi_QRemove((osi_queue_t **) &cm_data.buf_freeListp, &bp->q);
             bp->flags &= ~CM_BUF_INLRU;
 
+            /* prepare to return it.  Give it a refcount */
+            bp->refCount = 1;
+#ifdef DEBUG_REFCOUNT
+            osi_Log2(afsd_logp,"buf_GetNewLocked bp 0x%p ref %d", bp, 1);
+            afsi_log("%s:%d buf_GetNewLocked bp 0x%p, ref %d", __FILE__, __LINE__, bp, 1);
+#endif
             /* grab the mutex so that people don't use it
              * before the caller fills it with data.  Again, no one	
              * should have been able to get to this dude to lock it.
              */
 	    if (!lock_TryMutex(&bp->mx)) {
 	    	osi_Log2(afsd_logp, "buf_GetNewLocked bp 0x%p cannot be mutex locked.  refCount %d should be 0",
-			 bp, bp->refCount);
+                         bp, bp->refCount);
 		osi_panic("buf_GetNewLocked: TryMutex failed",__FILE__,__LINE__);
 	    }
 
-	    /* prepare to return it.  Give it a refcount */
-            bp->refCount = 1;
-#ifdef DEBUG_REFCOUNT
-            osi_Log2(afsd_logp,"buf_GetNewLocked bp 0x%p ref %d", bp, 1);
-            afsi_log("%s:%d buf_GetNewLocked bp 0x%p, ref %d", __FILE__, __LINE__, bp, 1);
-#endif
             lock_ReleaseWrite(&buf_globalLock);
             lock_ReleaseRead(&scp->bufCreateLock);
+
             *bufpp = bp;
 
 #ifdef TESTING
@@ -1729,13 +1747,38 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
                         cm_ReleaseUser(bp->userp);
                     bp->userp = userp;
                 }   
-                wasDirty = buf_CleanAsyncLocked(bp, reqp);
-                buf_CleanWait(scp, bp, TRUE);
-                if (bp->flags & CM_BUF_ERROR) {
-                    code = bp->error;
-                    if (code == 0) 
-                        code = -1;
+
+                switch (code) {
+                case CM_ERROR_NOSUCHFILE:
+                case CM_ERROR_BADFD:
+                case CM_ERROR_NOACCESS:
+                case CM_ERROR_QUOTA:
+                case CM_ERROR_SPACE:
+                case CM_ERROR_TOOBIG:
+                case CM_ERROR_READONLY:
+                case CM_ERROR_NOSUCHPATH:
+                    /* 
+                     * Apply the previous fatal error to this buffer.
+                     * Do not waste the time attempting to store to
+                     * the file server when we know it will fail.
+                     */
+                    bp->flags &= ~CM_BUF_DIRTY;
+                    bp->flags |= CM_BUF_ERROR;
+                    bp->dirty_offset = 0;
+                    bp->dirty_length = 0;
+                    bp->error = code;
+                    bp->dataVersion = CM_BUF_VERSION_BAD;
+                    bp->dirtyCounter++;
+                    break;
+                default:
+                    wasDirty = buf_CleanAsyncLocked(bp, reqp);
+                    if (bp->flags & CM_BUF_ERROR) {
+                        code = bp->error;
+                        if (code == 0)
+                            code = -1;
+                    }
                 }
+                buf_CleanWait(scp, bp, TRUE);
             }
             lock_ReleaseMutex(&bp->mx);
         }
