@@ -21,21 +21,29 @@ typedef struct cm_cell {
     struct cm_cell *allNextp;	        /* locked by cm_cellLock */
     struct cm_cell *nameNextp;	        /* locked by cm_cellLock */
     struct cm_cell *idNextp;	        /* locked by cm_cellLock */
+    struct cm_cell *freeNextp;
     char name[CELL_MAXNAMELEN];         /* cell name; never changes */
     cm_serverRef_t *vlServersp;         /* locked by cm_serverLock */
     osi_mutex_t mx;			/* mutex locking fields (flags) */
     long flags;			        /* locked by mx */
-    time_t timeout;                     /* if dns, time at which the server addrs expire */
+    time_t timeout;                     /* if dns, time at which the server addrs expire (mx) */
+    char linkedName[CELL_MAXNAMELEN];   /* linked cell name; cm_cellLock */
 } cm_cell_t;
 
 /* These are bit flag values */
-#define CM_CELLFLAG_SUID	       1	/* setuid flag; not yet used */
+#define CM_CELLFLAG_SUID	       1  /* setuid flag; not yet used */
 #define CM_CELLFLAG_DNS                2  /* cell servers are from DNS */
 #define CM_CELLFLAG_VLSERVER_INVALID   4  /* cell servers are invalid */
 #define CM_CELLFLAG_FREELANCE          8  /* local freelance fake cell */
 #define CM_CELLFLAG_IN_NAMEHASH        0x10
 #define CM_CELLFLAG_IN_IDHASH          0x20
 
+typedef struct cm_cell_rock {
+    cm_cell_t * cellp;
+    afs_uint32  flags;
+} cm_cell_rock_t;
+
+
 #define CM_CELL_NAME_HASH(name)  (SDBMHash(name) % cm_data.cellHashTableSize)
 
 #define CM_CELL_ID_HASH(id)   ((unsigned long) id % cm_data.cellHashTableSize)
@@ -50,7 +58,7 @@ extern cm_cell_t *cm_GetCell(char *namep, afs_uint32 flags);
 
 extern cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, afs_uint32 flags);
 
-extern cm_cell_t *cm_FindCellByID(afs_int32 cellID);
+extern cm_cell_t *cm_FindCellByID(afs_int32 cellID, afs_uint32 flags);
 
 extern void cm_ChangeRankCellVLServer(cm_server_t *tsp);
 
@@ -64,4 +72,8 @@ extern void cm_AddCellToNameHashTable(cm_cell_t * cellp);
 
 extern void cm_AddCellToIDHashTable(cm_cell_t * cellp);
 
+extern long cm_AddCellProc(void *rockp, struct sockaddr_in *addrp, char *namep);
+
+extern cm_cell_t *cm_UpdateCell(cm_cell_t * cp, afs_uint32 flags);
+
 #endif /* __CELL_H_ENV_ */
