@@ -12,6 +12,7 @@
 
 #include <errno.h>
 #include <windows.h>
+#include <shlwapi.h>
 #include <winsock2.h>
 #ifndef EWOULDBLOCK
 #define EWOULDBLOCK             WSAEWOULDBLOCK
@@ -330,7 +331,7 @@ long cm_MapVLRPCError(long error, cm_req_t *reqp)
 
     if (error < 0) 
 	error = CM_ERROR_TIMEDOUT;
-    else if (error == VL_NOENT) 
+    else if (error == VL_NOENT || error == VL_BADNAME) 
 	error = CM_ERROR_NOSUCHVOLUME;
     return error;
 }
@@ -340,7 +341,7 @@ cm_space_t *cm_GetSpace(void)
 	cm_space_t *tsp;
 
 	if (osi_Once(&cm_utilsOnce)) {
-		lock_InitializeRWLock(&cm_utilsLock, "cm_utilsLock");
+		lock_InitializeRWLock(&cm_utilsLock, "cm_utilsLock", LOCK_HIERARCHY_UTILS_GLOBAL);
 		osi_EndOnce(&cm_utilsOnce);
         }
         
@@ -398,7 +399,10 @@ int cm_Is8Dot3(clientchar_t *namep)
     int sawDot = 0;
     clientchar_t tc;
     int charCount = 0;
-        
+    
+    if (namep == NULL || !namep[0])
+        return 0;
+
     /*
      * can't have a leading dot;
      * special case for . and ..
@@ -440,7 +444,7 @@ char cm_8Dot3Mapping[42] =
 {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 
  'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 
- 'V', 'W', 'X', 'Y', 'Z', '_', '-', '$', '#', '!', '+', '='
+ 'V', 'W', 'X', 'Y', 'Z', '_', '-', '$', '#', '!', '{', '}'
 };
 int cm_8Dot3MapSize = sizeof(cm_8Dot3Mapping);
 
@@ -657,6 +661,14 @@ szWildCardMatchFileName(clientchar_t * pattern, clientchar_t * name, int casefol
                                             cm_ClientCharNext(p), FALSE))
                     return TRUE;
             } /* endfor */
+            if (*pattern == '.' && *pattern_next == '\0') {
+                for (p = name; p && *p; p = cm_ClientCharNext(p))
+                    if (*p == '.')
+                        break;
+                if (p && *p)
+                    return FALSE;
+                return TRUE;
+            }
             return FALSE;
 
         default:
@@ -681,8 +693,9 @@ szWildCardMatchFileName(clientchar_t * pattern, clientchar_t * name, int casefol
  */
 int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags) 
 {
-    clientchar_t * newmask;
-    int    i, j, star, qmark, casefold, retval;
+    clientchar_t *newmask, lastchar = _C('\0');
+    int    i, j, casefold, retval;
+    int  star = 0, qmark = 0, dot = 0;
 
     /* make sure we only match 8.3 names, if requested */
     if ((flags & CM_FLAG_8DOT3) && !cm_Is8Dot3(namep)) 
@@ -695,8 +708,9 @@ int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags)
      * for example  the sequence "*?*?*?*"
      * must be turned into the form "*"
      */
-    newmask = (clientchar_t *)malloc((cm_ClientStrLen(maskp)+1)*sizeof(clientchar_t));
+    newmask = (clientchar_t *)malloc((cm_ClientStrLen(maskp)+2)*sizeof(clientchar_t));
     for ( i=0, j=0, star=0, qmark=0; maskp[i]; i++) {
+        lastchar = maskp[i];
         switch ( maskp[i] ) {
         case '?':
         case '>':
@@ -706,6 +720,9 @@ int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags)
         case '*':
             star++;
             break;
+        case '.':
+            dot++;
+            /* fallthrough */
         default:
             if ( star ) {
                 newmask[j++] = '*';
@@ -724,6 +741,8 @@ int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags)
         while ( qmark-- )
             newmask[j++] = '?';
     }
+    if (dot == 0 && lastchar == '<')
+        newmask[j++] = '.';
     newmask[j++] = '\0';
 
     retval = szWildCardMatchFileName(newmask, namep, casefold) ? 1:0;
@@ -732,3 +751,36 @@ int cm_MatchMask(clientchar_t *namep, clientchar_t *maskp, int flags)
     return retval;
 }
 
+BOOL
+cm_TargetPerceivedAsDirectory(const fschar_t *target)
+{
+    char        * ext;
+
+    ext = PathFindExtension(target);
+    if (!ext[0])
+        return TRUE;
+
+    return FALSE;
+}
+
+HANDLE 
+cm_LoadAfsdHookLib(void)
+{
+    char dllname[260];
+    char *p;
+    HANDLE hLib;
+
+    if (!GetModuleFileName(NULL, dllname, sizeof(dllname)))
+        return NULL;
+
+    p = strrchr(dllname, '\\');
+    if (p) {
+        p++;
+        strcpy(p, AFSD_HOOK_DLL);
+        hLib = LoadLibrary(dllname);
+    } else {
+        hLib = LoadLibrary(AFSD_HOOK_DLL);
+    }
+
+    return hLib;
+}
