@@ -83,7 +83,7 @@ void cm_InitVolume(int newFile, long maxVols)
     static osi_once_t once;
 
     if (osi_Once(&once)) {
-        lock_InitializeRWLock(&cm_volumeLock, "cm global volume lock");
+        lock_InitializeRWLock(&cm_volumeLock, "cm global volume lock", LOCK_HIERARCHY_VOLUME_GLOBAL);
 
         if ( newFile ) {
             cm_data.allVolumesp = NULL;
@@ -100,14 +100,14 @@ void cm_InitVolume(int newFile, long maxVols)
             for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp) {
                 afs_uint32 volType;
 
-                lock_InitializeRWLock(&volp->rw, "cm_volume_t rwlock");
+                lock_InitializeRWLock(&volp->rw, "cm_volume_t rwlock", LOCK_HIERARCHY_VOLUME);
                 volp->flags |= CM_VOLUMEFLAG_RESET;
                 volp->flags &= ~CM_VOLUMEFLAG_UPDATING_VL;
                 for (volType = RWVOL; volType < NUM_VOL_TYPES; volType++) {
                     volp->vol[volType].state = vl_unknown;
                     volp->vol[volType].serversp = NULL;
                     if (volp->vol[volType].ID)
-                        cm_VolumeStatusNotification(volp, volp->vol[volType].ID, vl_alldown, volp->vol[volType].state);
+                        cm_VolumeStatusNotification(volp, volp->vol[volType].ID, vl_unknown, volp->vol[volType].state);
                 }
                 volp->cbExpiresRO = 0;
             }
@@ -185,6 +185,8 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
 #endif
     afs_uint32 volType;
 
+    lock_AssertWrite(&volp->rw);
+
 #ifdef AFS_FREELANCE_CLIENT
     if ( cellp->cellID == AFS_FAKE_ROOT_CELL_ID && volp->vol[RWVOL].ID == AFS_FAKE_ROOT_VOL_ID ) 
     {
@@ -225,7 +227,9 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             cm_UpdateCell(cellp, 0);
 
         /* now we have volume structure locked and held; make RPC to fill it */
-	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", volp->cellp->name, volp->namep);
+	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", 
+                  osi_LogSaveString(afsd_logp,volp->cellp->name), 
+                  osi_LogSaveString(afsd_logp,volp->namep));
         do {
             struct rx_connection * rxconnp;
 
@@ -252,10 +256,12 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         code = cm_MapVLRPCError(code, reqp);
 	if ( code )
 	    osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x", 
-		      volp->cellp->name, volp->namep, code);
+		      osi_LogSaveString(afsd_logp,volp->cellp->name), 
+                      osi_LogSaveString(afsd_logp,volp->namep), code);
 	else
 	    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS", 
-		      volp->cellp->name, volp->namep);
+		      osi_LogSaveString(afsd_logp,volp->cellp->name), 
+                      osi_LogSaveString(afsd_logp,volp->namep));
     }
 
     /* We can end up here with code == CM_ERROR_NOSUCHVOLUME if the base volume name
@@ -269,7 +275,8 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         snprintf(name, VL_MAXNAMELEN, "%s.readonly", volp->namep);
                 
         /* now we have volume structure locked and held; make RPC to fill it */
-	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", volp->cellp->name, 
+	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", 
+                 osi_LogSaveString(afsd_logp,volp->cellp->name),
                  osi_LogSaveString(afsd_logp,name));
         do {
             struct rx_connection * rxconnp;
@@ -297,10 +304,12 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         code = cm_MapVLRPCError(code, reqp);
 	if ( code )
 	    osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x", 
-		      volp->cellp->name, osi_LogSaveString(afsd_logp,name), code);
+		     osi_LogSaveString(afsd_logp,volp->cellp->name), 
+                     osi_LogSaveString(afsd_logp,name), code);
 	else
 	    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS", 
-		      volp->cellp->name, osi_LogSaveString(afsd_logp,name));
+		     osi_LogSaveString(afsd_logp,volp->cellp->name), 
+                     osi_LogSaveString(afsd_logp,name));
     }
     
     lock_ObtainWrite(&volp->rw);
@@ -425,7 +434,8 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                 name[len - 9] = '\0';
             }
             
-            osi_Log2(afsd_logp, "cm_UpdateVolume name %s -> %s", volp->namep, name);
+            osi_Log2(afsd_logp, "cm_UpdateVolume name %s -> %s", 
+                     osi_LogSaveString(afsd_logp,volp->namep), osi_LogSaveString(afsd_logp,name));
 
             if (volp->flags & CM_VOLUMEFLAG_IN_HASH)
                 cm_RemoveVolumeFromNameHashTable(volp);
@@ -556,10 +566,11 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             cm_RandomizeServer(&volp->vol[ROVOL].serversp);
         }
 
+
         rwNewstate = rwServers_alldown ? vl_alldown : vl_online;
         roNewstate = roServers_alldown ? vl_alldown : vl_online;
         bkNewstate = bkServers_alldown ? vl_alldown : vl_online;
-    } else if (code == CM_ERROR_NOSUCHVOLUME || code == VL_NOENT) {
+    } else if (code == CM_ERROR_NOSUCHVOLUME || code == VL_NOENT || code == VL_BADNAME) {
         /* this volume does not exist - we should discard it */
         if (volp->flags & CM_VOLUMEFLAG_IN_HASH)
             cm_RemoveVolumeFromNameHashTable(volp);
@@ -602,7 +613,8 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
 
     volp->flags &= ~CM_VOLUMEFLAG_UPDATING_VL;
     osi_Log4(afsd_logp, "cm_UpdateVolumeLocation done, waking others name %s:%s flags 0x%x code 0x%x", 
-             volp->cellp->name, volp->namep, volp->flags, code);
+             osi_LogSaveString(afsd_logp,volp->cellp->name), 
+             osi_LogSaveString(afsd_logp,volp->namep), volp->flags, code);
     osi_Wakeup((LONG_PTR) &volp->flags);
 
     return code;
@@ -738,7 +750,17 @@ long cm_FindVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
     /* otherwise, we didn't find it so consult the VLDB */
     sprintf(volNameString, "%u", volumeID);
     code = cm_FindVolumeByName(cellp, volNameString, userp, reqp,
-			      flags, outVolpp);
+			      flags | CM_GETVOL_FLAG_IGNORE_LINKED_CELL, outVolpp);
+
+    if (code == CM_ERROR_NOSUCHVOLUME && cellp->linkedName[0] && 
+        !(flags & CM_GETVOL_FLAG_IGNORE_LINKED_CELL)) {
+        cm_cell_t *linkedCellp = cm_GetCell(cellp->linkedName, flags);
+
+        if (linkedCellp)
+            code = cm_FindVolumeByID(linkedCellp, volumeID, userp, reqp, 
+                                     flags | CM_GETVOL_FLAG_IGNORE_LINKED_CELL, 
+                                     outVolpp);
+    }
     return code;
 }
 
@@ -795,6 +817,13 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         afs_uint32 volType;
         /* otherwise, get from VLDB */
 
+        /* 
+         * Change to a write lock so that we have exclusive use of
+         * the first cm_volume_t with a refCount of 0 so that we 
+         * have time to increment it.
+         */
+        lock_ConvertRToW(&cm_volumeLock);
+
 	if ( cm_data.currentVolumes >= cm_data.maxVolumes ) {
 #ifdef RECYCLE_FROM_ALL_VOLUMES_LIST
 	    for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp) {
@@ -817,7 +846,8 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
 	    if (!volp)
 		osi_panic("Exceeded Max Volumes", __FILE__, __LINE__);
 
-            lock_ReleaseRead(&cm_volumeLock);
+            InterlockedIncrement(&volp->refCount);
+            lock_ReleaseWrite(&cm_volumeLock);
             lock_ObtainWrite(&volp->rw);
             lock_ObtainWrite(&cm_volumeLock);
 
@@ -836,6 +866,9 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
                     cm_VolumeStatusNotification(volp, volp->vol[volType].ID, volp->vol[volType].state, vl_unknown);
                 volp->vol[volType].ID = 0;
                 cm_SetFid(&volp->vol[volType].dotdotFid, 0, 0, 0, 0);
+                lock_ReleaseWrite(&cm_volumeLock);
+                cm_FreeServerList(&volp->vol[volType].serversp, CM_FREESERVERLIST_DELETE);
+                lock_ObtainWrite(&cm_volumeLock);
             }
 	} else {
 	    volp = &cm_data.volumeBaseAddress[cm_data.currentVolumes++];
@@ -843,15 +876,15 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
 	    volp->magic = CM_VOLUME_MAGIC;
 	    volp->allNextp = cm_data.allVolumesp;
 	    cm_data.allVolumesp = volp;
-	    lock_InitializeRWLock(&volp->rw, "cm_volume_t rwlock");
-            lock_ReleaseRead(&cm_volumeLock);
+	    lock_InitializeRWLock(&volp->rw, "cm_volume_t rwlock", LOCK_HIERARCHY_VOLUME);
+            lock_ReleaseWrite(&cm_volumeLock);
             lock_ObtainWrite(&volp->rw);
             lock_ObtainWrite(&cm_volumeLock);
+            volp->refCount = 1;	/* starts off held */
         }
 	volp->cellp = cellp;
 	strncpy(volp->namep, name, VL_MAXNAMELEN);
 	volp->namep[VL_MAXNAMELEN-1] = '\0';
-        volp->refCount = 1;	/* starts off held */
 	volp->flags = CM_VOLUMEFLAG_RESET;
     
         for ( volType = RWVOL; volType < NUM_VOL_TYPES; volType++) {
@@ -897,10 +930,25 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         cm_PutVolume(volp);
         lock_ReleaseRead(&cm_volumeLock);
     }
+
+    if (code == CM_ERROR_NOSUCHVOLUME && cellp->linkedName[0] && 
+        !(flags & CM_GETVOL_FLAG_IGNORE_LINKED_CELL)) {
+        cm_cell_t *linkedCellp = cm_GetCell(cellp->linkedName, flags);
+
+        if (linkedCellp)
+            code = cm_FindVolumeByName(linkedCellp, volumeNamep, userp, reqp, 
+                                       flags | CM_GETVOL_FLAG_IGNORE_LINKED_CELL, 
+                                       outVolpp);
+    }
     return code;
 }	
 
-void cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
+/* 
+ * Only call this function in response to a VNOVOL or VMOVED error
+ * from a file server.  Do not call it in response to CM_ERROR_NOSUCHVOLUME
+ * as that can lead to recursive calls.
+ */
+long cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
 {
     cm_cell_t *cellp;
     cm_volume_t *volp;
@@ -908,11 +956,14 @@ void cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
     cm_volume_t *volp2;
 #endif
     afs_uint32  hash;
+    long code;
 
-    if (!fidp) return;
+    if (!fidp) 
+        return CM_ERROR_INVAL;
 
     cellp = cm_FindCellByID(fidp->cell, 0);
-    if (!cellp) return;
+    if (!cellp) 
+        return CM_ERROR_NOSUCHCELL;
 
     /* search for the volume */
     lock_ObtainRead(&cm_volumeLock);
@@ -960,38 +1011,31 @@ void cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
     lock_ReleaseRead(&cm_volumeLock);
 
     if (!volp)
-        return;
+        return CM_ERROR_NOSUCHVOLUME;
 
     /* update it */
     cm_data.mountRootGen = time(NULL);
     lock_ObtainWrite(&volp->rw);
     volp->flags |= CM_VOLUMEFLAG_RESET;
-#ifdef COMMENT
-    /* Mark the volume to be updated but don't update it now.
-     * This function is called only from within cm_Analyze
-     * when cm_ConnByMServers has failed with all servers down
-     * The problem is that cm_UpdateVolume is going to call
-     * cm_ConnByMServers which may cause a recursive chain
-     * of calls each returning a retry on failure.
-     * Instead, set the flag so the next time the volume is
-     * accessed by Name or ID the UpdateVolume call will
-     * occur.
-     */
+
     code = cm_UpdateVolumeLocation(cellp, userp, reqp, volp);
-#endif
     lock_ReleaseWrite(&volp->rw);
 
     lock_ObtainRead(&cm_volumeLock);
     cm_PutVolume(volp);
     lock_ReleaseRead(&cm_volumeLock);
+
+    return code;
 }
 
 /* find the appropriate servers from a volume */
-cm_serverRef_t **cm_GetVolServers(cm_volume_t *volp, afs_uint32 volume)
+cm_serverRef_t **cm_GetVolServers(cm_volume_t *volp, afs_uint32 volume, cm_user_t *userp, cm_req_t *reqp)
 {
     cm_serverRef_t **serverspp;
-    cm_serverRef_t *current;;
+    cm_serverRef_t *current;
+    int firstTry = 1;
 
+  start:
     lock_ObtainWrite(&cm_serverLock);
 
     if (volume == volp->vol[RWVOL].ID)
@@ -1000,10 +1044,26 @@ cm_serverRef_t **cm_GetVolServers(cm_volume_t *volp, afs_uint32 volume)
         serverspp = &volp->vol[ROVOL].serversp;
     else if (volume == volp->vol[BACKVOL].ID)
         serverspp = &volp->vol[BACKVOL].serversp;
-    else 
-        osi_panic("bad volume ID in cm_GetVolServers", __FILE__, __LINE__);
-        
-    for (current = *serverspp; current; current = current->next)
+    else {
+        lock_ReleaseWrite(&cm_serverLock);
+        if (firstTry) {
+            afs_int32 code;
+            firstTry = 0;
+            lock_ObtainWrite(&volp->rw);
+            volp->flags |= CM_VOLUMEFLAG_RESET;
+            code = cm_UpdateVolumeLocation(volp->cellp, userp, reqp, volp);
+            lock_ReleaseWrite(&volp->rw);
+            if (code == 0)
+                goto start;
+        }
+        return NULL;
+    }
+
+    /* 
+     * Increment the refCount on deleted items as well.
+     * They will be freed by cm_FreeServerList when they get to zero 
+     */
+    for (current = *serverspp; current; current = current->next) 
         current->refCount++;
 
     lock_ReleaseWrite(&cm_serverLock);
@@ -1078,12 +1138,9 @@ void cm_RefreshVolumes(void)
 
 }
 
-
-/* The return code is 0 if the volume is not online and 
- * 1 if the volume is online
- */
-long
-cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
+void
+cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32 volID,
+                           afs_uint32 *onlinep, afs_uint32 *volumeUpdatedp)
 {
     cm_conn_t *connp;
     long code;
@@ -1096,139 +1153,102 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
     char volName[32];
     char offLineMsg[256];
     char motd[256];
-    long online = 0;
+    long alldown, alldeleted;
     cm_serverRef_t *serversp;
 
     Name = volName;
     OfflineMsg = offLineMsg;
     MOTD = motd;
 
+    if (statep->ID != 0 && (!volID || volID == statep->ID)) {
+        if (!statep->serversp && !(*volumeUpdatedp)) {
+            cm_InitReq(&req);
+            code = cm_UpdateVolumeLocation(volp->cellp, cm_rootUserp, &req, volp);
+            *volumeUpdatedp = 1;
+        }
+
+        if (statep->serversp) {
+            alldown = 1;
+            alldeleted = 1;
+            for (serversp = statep->serversp; serversp; serversp = serversp->next) {
+                if (serversp->status == srv_deleted)
+                    continue;
+
+                alldeleted = 0;
+                *onlinep = 1;
+                alldown = 0;
+                
+                if (serversp->status == srv_busy || serversp->status == srv_offline)
+                    serversp->status = srv_not_busy;
+            }
+
+            if (alldeleted && !(*volumeUpdatedp)) {
+                cm_InitReq(&req);
+                code = cm_UpdateVolumeLocation(volp->cellp, cm_rootUserp, &req, volp);
+                *volumeUpdatedp = 1;
+            }
+
+            if (statep->state == vl_busy || statep->state == vl_offline || statep->state == vl_unknown ||
+                (!alldown && statep->state == vl_alldown)) {
+                cm_InitReq(&req);
+
+                lock_ReleaseWrite(&volp->rw);
+                do {
+                    code = cm_ConnFromVolume(volp, statep->ID, cm_rootUserp, &req, &connp);
+                    if (code) 
+                        continue;
+
+                    rxconnp = cm_GetRxConn(connp);
+                    code = RXAFS_GetVolumeStatus(rxconnp, statep->ID,
+                                                 &volStat, &Name, &OfflineMsg, &MOTD);
+                    rx_PutConnection(rxconnp);            
+
+                } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
+                code = cm_MapRPCError(code, &req);
+
+                lock_ObtainWrite(&volp->rw);
+                if (code == 0 && volStat.Online) {
+                    cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_online);
+                    statep->state = vl_online;
+                    *onlinep = 1;
+                } else if (code == CM_ERROR_NOACCESS) {
+                    cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_unknown);
+                    statep->state = vl_unknown;
+                    *onlinep = 1;
+                }
+            } else if (alldown && statep->state != vl_alldown) {
+                cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_alldown);
+                statep->state = vl_alldown;
+            }
+        } else if (statep->state != vl_alldown) {
+            cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_alldown);
+            statep->state = vl_alldown;
+        }
+    }
+}
+
+/* The return code is 0 if the volume is not online and 
+ * 1 if the volume is online
+ */
+long
+cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
+{
+    long code;
+    cm_req_t req;
+    afs_uint32 online = 0;
+    afs_uint32 volumeUpdated = 0;
+
     lock_ObtainWrite(&volp->rw);
 
     if (volp->flags & CM_VOLUMEFLAG_RESET) {
         cm_InitReq(&req);
         code = cm_UpdateVolumeLocation(volp->cellp, cm_rootUserp, &req, volp);
+        volumeUpdated = 1;
     }
 
-    if (volp->vol[RWVOL].ID != 0 && (!volID || volID == volp->vol[RWVOL].ID) &&
-         volp->vol[RWVOL].serversp) {
-       
-        for (serversp = volp->vol[RWVOL].serversp; serversp; serversp = serversp->next) {
-            if (serversp->status == srv_busy || serversp->status == srv_offline) {
-                serversp->status = srv_not_busy;
-                online = 1;
-            }
-        }
-
-        if (volp->vol[RWVOL].state == vl_busy || volp->vol[RWVOL].state == vl_offline || volp->vol[RWVOL].state == vl_unknown) {
-            cm_InitReq(&req);
-
-            lock_ReleaseWrite(&volp->rw);
-            do {
-                code = cm_ConnFromVolume(volp, volp->vol[RWVOL].ID, cm_rootUserp, &req, &connp);
-                if (code) 
-                    continue;
-
-                rxconnp = cm_GetRxConn(connp);
-                code = RXAFS_GetVolumeStatus(rxconnp, volp->vol[RWVOL].ID,
-                                             &volStat, &Name, &OfflineMsg, &MOTD);
-                rx_PutConnection(rxconnp);            
-
-            } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
-            code = cm_MapRPCError(code, &req);
-
-            lock_ObtainWrite(&volp->rw);
-            if (code == 0 && volStat.Online) {
-                cm_VolumeStatusNotification(volp, volp->vol[RWVOL].ID, volp->vol[RWVOL].state, vl_online);
-                volp->vol[RWVOL].state = vl_online;
-                online = 1;
-            } else if (code == CM_ERROR_NOACCESS) {
-                cm_VolumeStatusNotification(volp, volp->vol[RWVOL].ID, volp->vol[RWVOL].state, vl_unknown);
-                volp->vol[RWVOL].state = vl_unknown;
-                online = 1;
-            }
-        }
-    }
-
-    if (volp->vol[ROVOL].ID != 0 && (!volID || volID == volp->vol[ROVOL].ID) &&
-         volp->vol[ROVOL].serversp) {
-
-        for (serversp = volp->vol[ROVOL].serversp; serversp; serversp = serversp->next) {
-            if (serversp->status == srv_busy || serversp->status == srv_offline) {
-                serversp->status = srv_not_busy;
-                online = 1;
-            }
-        }
-
-        if (volp->vol[ROVOL].state == vl_busy || volp->vol[ROVOL].state == vl_offline || volp->vol[ROVOL].state == vl_unknown) {
-            cm_InitReq(&req);
-
-            lock_ReleaseWrite(&volp->rw);
-            do {
-                code = cm_ConnFromVolume(volp, volp->vol[ROVOL].ID, cm_rootUserp, &req, &connp);
-                if (code) 
-                    continue;
-
-                rxconnp = cm_GetRxConn(connp);
-                code = RXAFS_GetVolumeStatus(rxconnp, volp->vol[ROVOL].ID,
-                                              &volStat, &Name, &OfflineMsg, &MOTD);
-                rx_PutConnection(rxconnp);        
-
-            } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
-            code = cm_MapRPCError(code, &req);
-
-            lock_ObtainWrite(&volp->rw);
-            if (code == 0 && volStat.Online) {
-                cm_VolumeStatusNotification(volp, volp->vol[ROVOL].ID, volp->vol[ROVOL].state, vl_online);
-                volp->vol[ROVOL].state = vl_online;
-                online = 1;
-            } else if (code == CM_ERROR_NOACCESS) {
-                cm_VolumeStatusNotification(volp, volp->vol[ROVOL].ID, volp->vol[ROVOL].state, vl_unknown);
-                volp->vol[ROVOL].state = vl_unknown;
-                online = 1;
-            }
-        }
-    }
-
-    if (volp->vol[BACKVOL].ID != 0 && (!volID || volID == volp->vol[BACKVOL].ID) &&
-         volp->vol[BACKVOL].serversp) {
-        
-        for (serversp = volp->vol[BACKVOL].serversp; serversp; serversp = serversp->next) {
-            if (serversp->status == srv_busy || serversp->status == srv_offline) {
-                serversp->status = srv_not_busy;
-                online = 1;
-            }
-        }
-
-        if (volp->vol[BACKVOL].state == vl_busy || volp->vol[BACKVOL].state == vl_offline || volp->vol[BACKVOL].state == vl_unknown) {
-            cm_InitReq(&req);
-
-            lock_ReleaseWrite(&volp->rw);
-            do {
-                code = cm_ConnFromVolume(volp, volp->vol[BACKVOL].ID, cm_rootUserp, &req, &connp);
-                if (code) 
-                    continue;
-
-                rxconnp = cm_GetRxConn(connp);
-                code = RXAFS_GetVolumeStatus(rxconnp, volp->vol[BACKVOL].ID,
-                                              &volStat, &Name, &OfflineMsg, &MOTD);
-                rx_PutConnection(rxconnp);        
-
-            } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
-            code = cm_MapRPCError(code, &req);
-
-            lock_ObtainWrite(&volp->rw);
-            if (code == 0 && volStat.Online) {
-                cm_VolumeStatusNotification(volp, volp->vol[BACKVOL].ID, volp->vol[BACKVOL].state, vl_online);
-                volp->vol[BACKVOL].state = vl_online;
-                online = 1;
-            } else if (code == CM_ERROR_NOACCESS) {
-                cm_VolumeStatusNotification(volp, volp->vol[BACKVOL].ID, volp->vol[BACKVOL].state, vl_unknown);
-                volp->vol[BACKVOL].state = vl_unknown;
-                online = 1;
-            }
-        }
-    }
+    cm_CheckOfflineVolumeState(volp, &volp->vol[RWVOL], volID, &online, &volumeUpdated);
+    cm_CheckOfflineVolumeState(volp, &volp->vol[ROVOL], volID, &online, &volumeUpdated);
+    cm_CheckOfflineVolumeState(volp, &volp->vol[BACKVOL], volID, &online, &volumeUpdated);
 
     lock_ReleaseWrite(&volp->rw);
     return online;
@@ -1273,22 +1293,26 @@ cm_UpdateVolumeStatusInt(cm_volume_t *volp, struct cm_vol_state *statep)
 
     lock_ObtainWrite(&cm_serverLock);
     for (tsrp = statep->serversp; tsrp; tsrp=tsrp->next) {
+        if (tsrp->status == srv_deleted)
+            continue;
         tsp = tsrp->server;
-        cm_GetServerNoLock(tsp);
-        if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
-	    allDown = 0;
-            if (tsrp->status == srv_busy) {
-		allOffline = 0;
-                someBusy = 1;
-            } else if (tsrp->status == srv_offline) {
-		allBusy = 0;
-		someOffline = 1;
-            } else {
-		allOffline = 0;
-                allBusy = 0;
+        if (tsp) {
+            cm_GetServerNoLock(tsp);
+            if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
+                allDown = 0;
+                if (tsrp->status == srv_busy) {
+                    allOffline = 0;
+                    someBusy = 1;
+                } else if (tsrp->status == srv_offline) {
+                    allBusy = 0;
+                    someOffline = 1;
+                } else {
+                    allOffline = 0;
+                    allBusy = 0;
+                }
             }
+            cm_PutServerNoLock(tsp);
         }
-        cm_PutServerNoLock(tsp);
     }   
     lock_ReleaseWrite(&cm_serverLock);
 
