@@ -16,6 +16,9 @@ extern "C" {
 
 #include "afscreds.h"
 
+#define SECURITY_WIN32 1
+#include <security.h>
+
 /*
  * DEFINITIONS ________________________________________________________________
  *
@@ -171,11 +174,11 @@ BOOL CALLBACK Main_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
                      }
                   lock_ReleaseMutex(&g.credsLock);
                   CheckMenuItem (hm, M_REMIND, MF_BYCOMMAND | ((fRemind) ? MF_CHECKED : MF_UNCHECKED));
-
+		  SetForegroundWindow(hDlg);
                   TrackPopupMenu (GetSubMenu (hmDummy, 0),
                                   TPM_RIGHTALIGN | TPM_RIGHTBUTTON,
                                   pt.x, pt.y, NULL, hDlg, NULL);
-
+		  PostMessage(hDlg, WM_NULL, 0, 0);
                   DestroyMenu (hmDummy);
                   }
                break;
@@ -207,20 +210,20 @@ BOOL CALLBACK Main_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
                   {
                       if (StartService (hService, 0, 0))
                           TestAndDoMapShare(SERVICE_START_PENDING);
-		                  if ( KFW_is_available() && KFW_AFS_wait_for_service_start() ) {
+		      if ( KFW_is_available() && KFW_AFS_wait_for_service_start() ) {
 #ifdef USE_MS2MIT
-                              KFW_import_windows_lsa();
+			  KFW_import_windows_lsa();
 #endif /* USE_MS2MIT */
-			                  KFW_AFS_renew_tokens_for_all_cells();
-						  }
+			  KFW_AFS_renew_tokens_for_all_cells();
+		      }
 
                       CloseServiceHandle (hService);
                   }
 
                   CloseServiceHandle (hManager);
               }
-              KFW_AFS_wait_for_service_start();
-              ObtainTokensFromUserIfNeeded(g.hMain);
+              if (KFW_AFS_wait_for_service_start())
+		  ObtainTokensFromUserIfNeeded(g.hMain);
           }
           break;
       }
@@ -246,74 +249,47 @@ void Main_Show (BOOL fShow)
 
 void Main_OnInitDialog (HWND hDlg)
 {
-   if (!g.fIsWinNT)
-      {
-      TCHAR szTitle[256];
-      GetString (szTitle, IDS_TITLE_95);
-      SetWindowText (hDlg, szTitle);
-      }
-
-   TCHAR szVersion[256];
-   DWORD dwPatch = 0;
-   TCHAR szUser[256];
-   GetString (szVersion, IDS_UNKNOWN);
-   GetString (szUser, IDS_UNKNOWN);
-
-   HKEY hk;
-   if (RegOpenKey (HKEY_LOCAL_MACHINE, REGSTR_PATH_AFS, &hk) == 0)
-      {
-      DWORD dwSize = sizeof(szVersion);
-      DWORD dwType = REG_SZ;
-      RegQueryValueEx (hk, REGVAL_AFS_VERSION, NULL, &dwType, (PBYTE)szVersion, &dwSize);
-
-      dwSize = sizeof(dwPatch);
-      dwType = REG_DWORD;
-      RegQueryValueEx (hk, REGVAL_AFS_PATCH, NULL, &dwType, (PBYTE)&dwPatch, &dwSize);
-      RegCloseKey (hk);
-      }
-
-   BOOL fFoundUserName = FALSE;
-    if (RegOpenKey (HKEY_LOCAL_MACHINE, TEXT("Software\\Microsoft\\Windows NT\\CurrentVersion\\Explorer"), &hk) == 0)
+    if (!g.fIsWinNT)
     {
-        DWORD dwSize = sizeof(szUser);
+        TCHAR szTitle[256];
+        GetString (szTitle, IDS_TITLE_95);
+        SetWindowText (hDlg, szTitle);
+    }
+
+    TCHAR szVersion[256];
+    DWORD dwPatch = 0;
+    TCHAR szUser[1024];
+    GetString (szVersion, IDS_UNKNOWN);
+    GetString (szUser, IDS_UNKNOWN);
+
+    HKEY hk;
+    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, AFSREG_CLT_SW_VERSION_SUBKEY, 0,
+                       (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
+    {
+        DWORD dwSize = sizeof(szVersion);
         DWORD dwType = REG_SZ;
-        if (RegQueryValueEx (hk, TEXT("Logon User Name"), NULL, &dwType, (PBYTE)szUser, &dwSize) == 0)
-            fFoundUserName = TRUE;
+        RegQueryValueEx (hk, REGVAL_AFS_VERSION, NULL, &dwType, (PBYTE)szVersion, &dwSize);
+
+        dwSize = sizeof(dwPatch);
+        dwType = REG_DWORD;
+        RegQueryValueEx (hk, REGVAL_AFS_PATCH, NULL, &dwType, (PBYTE)&dwPatch, &dwSize);
         RegCloseKey (hk);
     }
-    if (!fFoundUserName ) 
-    {
-        if (RegOpenKey (HKEY_LOCAL_MACHINE, TEXT("Software\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"), &hk) == 0)
-        {
-            DWORD dwSize = sizeof(szUser);
-            DWORD dwType = REG_SZ;
-            if (RegQueryValueEx (hk, TEXT("DefaultUserName"), NULL, &dwType, (PBYTE)szUser, &dwSize) == 0)
-                fFoundUserName = TRUE;
-            RegCloseKey (hk);
-        }
-    }
-   if (!fFoundUserName)
-   {
-       if (RegOpenKey (HKEY_LOCAL_MACHINE, TEXT("Network\\Logon"), &hk) == 0)
-       {
-           DWORD dwSize = sizeof(szUser);
-           DWORD dwType = REG_SZ;
-           if (RegQueryValueEx (hk, TEXT("UserName"), NULL, &dwType, (PBYTE)szUser, &dwSize) == 0)
-               fFoundUserName = TRUE;
-           RegCloseKey (hk);
-       }
-   }
 
-   TCHAR szSource[ cchRESOURCE ];
-   TCHAR szTarget[ cchRESOURCE ];
+    DWORD dwUserLen =  sizeof(szUser)/sizeof(TCHAR);
+    if (!GetUserNameEx(NameUserPrincipal, szUser, &dwUserLen))
+        GetUserNameEx(NameSamCompatible, szUser, &dwUserLen);
 
-   GetString (szSource, (dwPatch) ? IDS_TITLE_VERSION : IDS_TITLE_VERSION_NOPATCH);
-   wsprintf (szTarget, szSource, szVersion, dwPatch);
-   SetDlgItemText (hDlg, IDC_TITLE_VERSION, szTarget);
+    TCHAR szSource[ cchRESOURCE ];
+    TCHAR szTarget[ cchRESOURCE ];
 
-   GetDlgItemText (hDlg, IDC_TITLE_NT, szSource, cchRESOURCE);
-   wsprintf (szTarget, szSource, szUser);
-   SetDlgItemText (hDlg, IDC_TITLE_NT, szTarget);
+    GetString (szSource, (dwPatch) ? IDS_TITLE_VERSION : IDS_TITLE_VERSION_NOPATCH);
+    wsprintf (szTarget, szSource, szVersion, dwPatch);
+    SetDlgItemText (hDlg, IDC_TITLE_VERSION, szTarget);
+
+    GetDlgItemText (hDlg, IDC_TITLE_NT, szSource, cchRESOURCE);
+    wsprintf (szTarget, szSource, szUser);
+    SetDlgItemText (hDlg, IDC_TITLE_NT, szTarget);
 }
 
 
@@ -321,7 +297,8 @@ void Main_OnCheckMenuRemind (void)
 {
    BOOL fRemind = FALSE;
    lock_ObtainMutex(&g.credsLock);
-   for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
+   size_t iCreds;
+   for (iCreds = 0; iCreds < g.cCreds; ++iCreds)
       {
       if (g.aCreds[ iCreds ].fRemind)
          fRemind = TRUE;
@@ -363,13 +340,58 @@ void Main_OnRemindTimer (void)
    // dialog. Make sure we never display a warning more than once.
    //
    size_t iExpired;
-   if ((iExpired = Main_FindExpiredCreds()) != -1)
-      {
-      if (InterlockedIncrement (&g.fShowingMessage) != 1)
-         InterlockedDecrement (&g.fShowingMessage);
-      else
-         ShowObtainCreds (TRUE, g.aCreds[ iExpired ].szCell);
-      }
+   if ((iExpired = Main_FindExpiredCreds()) != -1) {
+       if (InterlockedIncrement (&g.fShowingMessage) != 1) {
+           InterlockedDecrement (&g.fShowingMessage);
+       } else { 
+           char * rootcell = NULL;
+           char   password[PROBE_PASSWORD_LEN+1];
+           struct afsconf_cell cellconfig;
+           BOOL   serverReachable = FALSE;
+           DWORD  code;
+
+           rootcell = (char *)GlobalAlloc(GPTR,MAXCELLCHARS+1);
+           if (!rootcell) 
+               goto cleanup;
+
+           code = KFW_AFS_get_cellconfig(g.aCreds[ iExpired ].szCell, 
+                                         (afsconf_cell*)&cellconfig, rootcell);
+           if (code) 
+               goto cleanup;
+
+           if (KFW_is_available()) {
+               // If we can't use the FSProbe interface we can attempt to forge
+               // a kinit and if we can back an invalid user error we know the
+               // kdc is at least reachable
+               serverReachable = KFW_probe_kdc(&cellconfig);
+           } else {
+               int i;
+
+               for ( i=0 ; i<PROBE_PASSWORD_LEN ; i++ )
+                   password[i] = 'x';
+
+               code = ObtainNewCredentials(rootcell, PROBE_USERNAME, password, TRUE);
+               switch ( code ) {
+               case INTK_BADPW:
+               case KERB_ERR_PRINCIPAL_UNKNOWN:
+               case KERB_ERR_SERVICE_EXP:
+               case RD_AP_TIME:
+                   serverReachable = TRUE;
+                   break;
+               default:
+                   serverReachable = FALSE;
+               }
+           }
+         cleanup:
+           if (rootcell)
+               GlobalFree(rootcell);
+
+           if (serverReachable)
+               ShowObtainCreds (TRUE, g.aCreds[ iExpired ].szCell);
+           else
+               InterlockedDecrement (&g.fShowingMessage);
+       }
+   }
 }
 
 
@@ -402,14 +424,16 @@ void Main_OnCheckTerminate (void)
 {
     HKEY hk;
 
-    if (RegOpenKey (HKEY_CURRENT_USER, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    if (RegOpenKeyEx (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, 0,
+                       (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
     {
         DWORD dwSize = sizeof(g.fStartup);
         DWORD dwType = REG_DWORD;
         RegQueryValueEx (hk, TEXT("ShowTrayIcon"), NULL, &dwType, (PBYTE)&g.fStartup, &dwSize);
         RegCloseKey (hk);
     }
-    else if (RegOpenKey (HKEY_LOCAL_MACHINE, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    else if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY, 0,
+                           (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
     {
       DWORD dwSize = sizeof(g.fStartup);
       DWORD dwType = REG_DWORD;
@@ -486,7 +510,8 @@ void Main_RepopulateTabs (BOOL fDestroyInvalid)
 
       size_t nCreds = 0;
       lock_ObtainMutex(&g.credsLock);
-      for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
+      size_t iCreds;
+      for (iCreds = 0; iCreds < g.cCreds; ++iCreds)
          {
          if (g.aCreds[ iCreds ].szCell[0])
             ++nCreds;
@@ -519,7 +544,8 @@ void Main_RepopulateTabs (BOOL fDestroyInvalid)
          {
          if (g.aCreds[ iCreds ].szCell[0])
             {
-            for (size_t ii = 0; ii < iTabOut; ++ii)
+            size_t ii;
+            for (ii = 0; ii < iTabOut; ++ii)
                {
                if (!ISCELLTAB (aTabs[ii]))
                   continue;
@@ -619,9 +645,18 @@ void Main_EnableRemindTimer (BOOL fEnable)
 size_t Main_FindExpiredCreds (void)
 {
    size_t retval = (size_t) -1;
+   static bool expirationCheck = false;
    lock_ObtainMutex(&g.expirationCheckLock);
+   if (expirationCheck) {
+       lock_ReleaseMutex(&g.expirationCheckLock);
+       return -1;
+   }
+   expirationCheck = true;
+   lock_ReleaseMutex(&g.expirationCheckLock);
+
    if ( KFW_is_available() )
        KFW_AFS_renew_expiring_tokens();
+   
    lock_ObtainMutex(&g.credsLock);
    for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
       {
@@ -656,6 +691,9 @@ size_t Main_FindExpiredCreds (void)
       }
    
    lock_ReleaseMutex(&g.credsLock);
+
+   lock_ObtainMutex(&g.expirationCheckLock);
+   expirationCheck = false;
    lock_ReleaseMutex(&g.expirationCheckLock);
 
    return retval;
@@ -728,7 +766,8 @@ void Terminate_OnOK (HWND hDlg)
    g.fStartup = IsDlgButtonChecked (hDlg, IDC_STARTUP);
 
     HKEY hk;
-    if (RegCreateKey (HKEY_CURRENT_USER, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    if (RegCreateKeyEx (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, 0, NULL, 0,
+                         (IsWow64()?KEY_WOW64_64KEY:0)|KEY_WRITE, NULL, &hk, NULL) == 0)
     {
         DWORD dwSize = sizeof(g.fStartup);
         DWORD dwType = REG_DWORD;
