@@ -22,7 +22,7 @@
  * SOFTWARE.
  */
 
-/* $Id: afsfuncs.c,v 1.1.2.18 2008/03/30 03:36:57 jaltman Exp $ */
+/* $Id: afsfuncs.c,v 1.1.2.21 2008/11/07 18:19:51 jaltman Exp $ */
 
 /* Disable the 'macro redefinition' warning which is getting
    triggerred by a redefinition of the ENCRYPT and DECRYPT macros. */
@@ -728,14 +728,15 @@ afs_klog(khm_handle identity,
          char *realm,
          int LifeTime,
          afs_tk_method method,
-         time_t * tok_expiration) {
+         time_t * tok_expiration,
+         char *linkedCell) {
 
     long	rc;
     CREDENTIALS	creds;
     struct ktc_principal	aserver;
     struct ktc_principal	aclient;
-    char	realm_of_user[REALM_SZ]; /* Kerberos realm of user */
-    char	realm_of_cell[REALM_SZ]; /* Kerberos realm of cell */
+    char	realm_of_user[MAXKTCREALMLEN]; /* Kerberos realm of user */
+    char	realm_of_cell[MAXKTCREALMLEN]; /* Kerberos realm of cell */
     char	local_cell[MAXCELLCHARS+1];
     char	Dmycell[MAXCELLCHARS+1];
     struct ktc_token	atoken;
@@ -763,6 +764,7 @@ afs_klog(khm_handle identity,
     if ( !cell )    cell = "";
     if ( !service ) service = "";
 
+    memset(&ak_cellconfig, 0, sizeof(ak_cellconfig));
     memset(RealmName, '\0', sizeof(RealmName));
     memset(CellName, '\0', sizeof(CellName));
     memset(ServiceName, '\0', sizeof(ServiceName));
@@ -784,6 +786,10 @@ afs_klog(khm_handle identity,
         return(rc);
     }
 
+    if (linkedCell && ak_cellconfig.linkedCell)
+        StringCbCopyA(linkedCell, MAXCELLCHARS, 
+                      ak_cellconfig.linkedCell);
+
     StringCbCopyA(realm_of_cell, sizeof(realm_of_cell), 
                   afs_realm_of_cell(&ak_cellconfig, FALSE));
 
@@ -831,8 +837,8 @@ afs_klog(khm_handle identity,
 
             pkrb5_cc_get_principal(context, k5cc, &client_principal);
             i = krb5_princ_realm(context, client_principal)->length;
-            if (i > REALM_SZ-1) 
-                i = REALM_SZ-1;
+            if (i > MAXKTCREALMLEN-1) 
+                i = MAXKTCREALMLEN-1;
             StringCchCopyNA(realm_of_user, ARRAYLENGTH(realm_of_user),
                             krb5_princ_realm(context, client_principal)->data,
                             i);
@@ -1061,7 +1067,8 @@ afs_klog(khm_handle identity,
 
             _reportf(L"Same token already exists");
             
-            return 0;
+            rc = 0;
+            goto cleanup;
         }
 
         // * Reset the "aclient" structure before we call ktc_SetToken.
@@ -1105,7 +1112,7 @@ afs_klog(khm_handle identity,
             if (context)
                 pkrb5_free_context(context);
             
-            return 0;
+            goto cleanup;
         }
 
         _reportf(L"SetToken returns code %d", rc);
@@ -1151,6 +1158,7 @@ afs_klog(khm_handle identity,
         _reportf(L"Kerberos 4 not configured");
 
     if (!bGotCreds && supports_krb4 && 
+        strlen(RealmName) < REALM_SZ && 
         (method == AFS_TOKEN_AUTO ||
          method == AFS_TOKEN_KRB4)) {
 
@@ -1238,7 +1246,8 @@ afs_klog(khm_handle identity,
             !memcmp(atoken.ticket, btoken.ticket, atoken.ticketLen)) {
 
             /* success! */
-            return(0);
+            rc = 0;
+            goto cleanup;
         }
 
         // Reset the "aclient" structure before we call ktc_SetToken.
@@ -1263,7 +1272,7 @@ afs_klog(khm_handle identity,
 
         if (rc = ktc_SetToken(&aserver, &atoken, &aclient, 0)) {
             afs_report_error(rc, "ktc_SetToken()");
-            return(rc);
+            goto cleanup;
         }
     } else if (method == AFS_TOKEN_AUTO ||
                method >= AFS_TOKEN_USER) {
@@ -1293,6 +1302,10 @@ afs_klog(khm_handle identity,
         }
     }
 
+  cleanup:
+    if (ak_cellconfig.linkedCell)
+        free(ak_cellconfig.linkedCell);
+
     return rc;
 }
 
@@ -1303,7 +1316,7 @@ static char *
 afs_realm_of_cell(afs_conf_cell *cellconfig, BOOL referral_fallback)
 {
     char krbhst[MAX_HSTNM]="";
-    static char krbrlm[REALM_SZ+1]="";
+    static char krbrlm[MAXKTCREALMLEN+1]="";
     krb5_context  ctx = 0;
     char ** realmlist=NULL;
     krb5_error_code r = 0;
@@ -1384,7 +1397,7 @@ afs_get_cellconfig(char *cell, afs_conf_cell *cellconfig, char *local_cell)
     if (strlen(cell) == 0)
         StringCbCopyA(cell, (MAXCELLCHARS+1) * sizeof(char), local_cell);
 
-    /* WIN32: cm_SearchCellFile(cell, pcallback, pdata) */
+    /* WIN32: cm_SearchCellFile(cell, newcell, pcallback, pdata) */
     StringCbCopyA(cellconfig->name, (MAXCELLCHARS+1) * sizeof(char), cell);
 
     rc = cm_SearchCellFile(cell, NULL, afs_get_cellconfig_callback, 
@@ -1592,12 +1605,15 @@ afs_check_for_cell_realm_match(khm_handle identity, char * cell) {
     int rc;
 
     ZeroMemory(local_cell, sizeof(local_cell));
+    ZeroMemory(&cellconfig, sizeof(cellconfig));
 
     rc = afs_get_cellconfig(cell, &cellconfig, local_cell);
     if (rc)
         return FALSE;
 
     realm = afs_realm_of_cell(&cellconfig, FALSE);
+    if (cellconfig.linkedCell)
+        free(cellconfig.linkedCell);
     if (realm == NULL)
         return FALSE;
 
