@@ -22,7 +22,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_symlink.c,v 1.19.2.7 2008/06/09 03:30:59 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_symlink.c,v 1.24.4.7 2008/11/08 16:34:42 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -76,6 +76,9 @@ int afs_symlink
 	goto done2;
 
     afs_InitFakeStat(&fakestate);
+
+    AFS_DISCON_LOCK();
+    
     code = afs_EvalFakeStat(&adp, &fakestate, &treq);
     if (code)
 	goto done;
@@ -89,6 +92,10 @@ int afs_symlink
 	code = afs_DynrootVOPSymlink(adp, acred, aname, atargetName);
 	goto done;
     }
+    if (afs_IsDynrootMount(adp)) {
+	code = EROFS;
+	goto done;
+    }
 
     code = afs_VerifyVCache(adp, &treq);
     if (code) {
@@ -104,6 +111,11 @@ int afs_symlink
 	goto done;
     }
 
+    if (AFS_IS_DISCONNECTED && !AFS_IS_LOGGING) {
+        code = ENETDOWN;
+        goto done;
+    }
+    
     InStatus.Mask = AFS_SETMODTIME | AFS_SETMODE;
     InStatus.ClientModTime = osi_Time();
     alen = strlen(atargetName);	/* we want it to include the null */
@@ -231,6 +243,7 @@ int afs_symlink
     afs_PutFakeStat(&fakestate);
     if (volp)
 	afs_PutVolume(volp, READ_LOCK);
+    AFS_DISCON_UNLOCK();
     code = afs_CheckCode(code, &treq, 31);
   done2:
     return code;
@@ -306,7 +319,10 @@ afs_UFSHandleLink(register struct vcache *avc, struct vrequest *areq)
 		   ICL_TYPE_POINTER, tdc, ICL_TYPE_OFFSET,
 		   ICL_HANDLE_OFFSET(avc->m.Length));
 	if (!tdc) {
-	    return EIO;
+	    if (AFS_IS_DISCONNECTED)
+	        return ENETDOWN;
+	    else
+	        return EIO;
 	}
 	/* otherwise we have the data loaded, go for it */
 	if (len > 1024) {
@@ -320,7 +336,11 @@ afs_UFSHandleLink(register struct vcache *avc, struct vrequest *areq)
 	rbuf = (char *)osi_AllocLargeSpace(AFS_LRALLOCSIZ);
 	tlen = len;
 	ObtainReadLock(&tdc->lock);
+#if defined(LINUX_USE_FH)
+	tfile = osi_UFSOpen_fh(&tdc->f.fh, tdc->f.fh_type);
+#else
 	tfile = osi_UFSOpen(tdc->f.inode);
+#endif
 	code = afs_osi_Read(tfile, -1, rbuf, tlen);
 	osi_UFSClose(tfile);
 	ReleaseReadLock(&tdc->lock);
@@ -356,6 +376,9 @@ afs_readlink(OSI_VC_ARG(avc), auio, acred)
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
     afs_InitFakeStat(&fakestat);
+
+    AFS_DISCON_LOCK();
+    
     code = afs_EvalFakeStat(&avc, &fakestat, &treq);
     if (code)
 	goto done;
@@ -380,6 +403,7 @@ afs_readlink(OSI_VC_ARG(avc), auio, acred)
     ReleaseWriteLock(&avc->lock);
   done:
     afs_PutFakeStat(&fakestat);
+    AFS_DISCON_UNLOCK();
     code = afs_CheckCode(code, &treq, 32);
     return code;
 }
