@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_buffer.c,v 1.22.4.1 2007/12/13 19:18:29 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_buffer.c,v 1.22.4.3 2008/11/08 16:34:42 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -232,7 +232,11 @@ DRead(register struct dcache *adc, register int page)
 	MReleaseWriteLock(&tb->lock);
 	return NULL;
     }
+#if defined(LINUX_USE_FH)
+    tfile = afs_CFileOpen(&adc->f.fh, adc->f.fh_type);
+#else
     tfile = afs_CFileOpen(adc->f.inode);
+#endif
     code =
 	afs_CFileRead(tfile, tb->page * AFS_BUFFER_PAGESIZE, tb->data,
 		      AFS_BUFFER_PAGESIZE);
@@ -344,7 +348,11 @@ afs_newslot(struct dcache *adc, afs_int32 apage, register struct buffer *lp)
 
     if (lp->dirty) {
 	/* see DFlush for rationale for not getting and locking the dcache */
-	tfile = afs_CFileOpen(lp->inode);
+#if defined(LINUX_USE_FH)
+        tfile = afs_CFileOpen(&lp->fh, lp->fh_type);
+#else
+        tfile = afs_CFileOpen(lp->inode);
+#endif
 	afs_CFileWrite(tfile, lp->page * AFS_BUFFER_PAGESIZE, lp->data,
 		       AFS_BUFFER_PAGESIZE);
 	lp->dirty = 0;
@@ -354,7 +362,12 @@ afs_newslot(struct dcache *adc, afs_int32 apage, register struct buffer *lp)
 
     /* Now fill in the header. */
     lp->fid = adc->index;
+#if defined(LINUX_USE_FH)
+    memcpy(&lp->fh, &adc->f.fh, sizeof(struct fid));
+    lp->fh_type = adc->f.fh_type;
+#else
     lp->inode = adc->f.inode;
+#endif
     lp->page = apage;
     lp->accesstime = timecounter++;
     FixupBucket(lp);		/* move to the right hash bucket */
@@ -430,11 +443,16 @@ DVOffset(register void *ap)
     return AFS_BUFFER_PAGESIZE * bp->page + (int)(((char *)ap) - bp->data);
 }
 
-/* 1/1/91 - I've modified the hash function to take the page as well
+/*! 
+ * Zap one dcache entry: destroy one FID's buffers.
+ *
+ * 1/1/91 - I've modified the hash function to take the page as well
  * as the *fid, so that lookup will be a bit faster.  That presents some
  * difficulties for Zap, which now has to have some knowledge of the nature
  * of the hash function.  Oh well.  This should use the list traversal 
  * method of DRead...
+ *
+ * \param adc The dcache entry to be zapped.
  */
 void
 DZap(struct dcache *adc)
@@ -486,7 +504,11 @@ DFlush(void)
 		 * we cannot lock afs_xdcache). In addition, we cannot obtain
 		 * a dcache lock while holding the tb->lock of the same file
 		 * since that can deadlock with DRead/DNew */
+#if defined(LINUX_USE_FH)
+		tfile = afs_CFileOpen(&tb->fh, tb->fh_type);
+#else
 		tfile = afs_CFileOpen(tb->inode);
+#endif
 		afs_CFileWrite(tfile, tb->page * AFS_BUFFER_PAGESIZE,
 			       tb->data, AFS_BUFFER_PAGESIZE);
 		tb->dirty = 0;	/* Clear the dirty flag */
