@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_conn.c,v 1.14.8.3 2008/05/23 14:25:15 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_conn.c,v 1.14.8.7 2008/10/20 16:40:48 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -52,6 +52,17 @@ afs_int32 cryptall = 0;		/* encrypt all communications */
 
 
 unsigned int VNOSERVERS = 0;
+
+/**
+ * Try setting up a connection to the server containing the specified fid.
+ * Gets the volume, checks if it's up and does the connection by server address.
+ *
+ * @param afid 
+ * @param areq Request filled in by the caller.
+ * @param locktype Type of lock that will be used.
+ *
+ * @return The conn struct, or NULL.
+ */
 struct conn *
 afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
 	 afs_int32 locktype)
@@ -66,6 +77,7 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
     struct srvAddr *sa1p;
 
     AFS_STATCNT(afs_Conn);
+    /* Get fid's volume. */
     tv = afs_GetVolume(afid, areq, READ_LOCK);
     if (!tv) {
 	if (areq) {
@@ -132,6 +144,19 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
 }				/*afs_Conn */
 
 
+/**
+ * Connects to a server by it's server address.
+ *
+ * @param sap Server address.
+ * @param aport Server port.
+ * @param acell
+ * @param tu Connect as this user.
+ * @param force_if_down
+ * @param create
+ * @param locktype Specifies type of lock to be used for this function.
+ *
+ * @return The new connection.
+ */
 struct conn *
 afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	     struct unixuser *tu, int force_if_down, afs_int32 create,
@@ -148,6 +173,7 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
     }
 
     ObtainSharedLock(&afs_xconn, 15);
+    /* Get conn by port and user. */
     for (tc = sap->conns; tc; tc = tc->next) {
 	if (tc->user == tu && tc->port == aport) {
 	    break;
@@ -155,11 +181,12 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
     }
 
     if (!tc && !create) {
+	/* Not found and can't create a new one. */
 	ReleaseSharedLock(&afs_xconn);
 	return NULL;
     }
     
-    if (AFS_IS_DISCONNECTED) {
+    if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
         afs_warnuser("afs_ConnBySA: disconnected\n");
         ReleaseSharedLock(&afs_xconn);
         return NULL;
@@ -187,7 +214,7 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	afs_ActivateServer(sap);
 
 	ConvertWToSLock(&afs_xconn);
-    }
+    } /* end of if (!tc) */
     tc->refCount++;
 
     if (tu->states & UTokensBad) {
@@ -251,20 +278,27 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	if (csec)
 	    rxs_Release(csec);
 	ConvertWToSLock(&afs_xconn);
-    }
+    } /* end of if (tc->forceConnectFS)*/
 
     ReleaseSharedLock(&afs_xconn);
     return tc;
 }
 
-/*
- * afs_ConnByHost
- *
+/**
  * forceConnectFS is set whenever we must recompute the connection. UTokensBad
  * is true only if we know that the tokens are bad.  We thus clear this flag
  * when we get a new set of tokens..
  * Having force... true and UTokensBad true simultaneously means that the tokens
  * went bad and we're supposed to create a new, unauthenticated, connection.
+ *
+ * @param aserver Server to connect to.
+ * @param aport Connection port.
+ * @param acell The cell where all of this happens.
+ * @param areq The request.
+ * @param aforce Force connection?
+ * @param locktype Type of lock to be used.
+ *
+ * @return The established connection.
  */
 struct conn *
 afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
@@ -276,7 +310,7 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
 
     AFS_STATCNT(afs_ConnByHost);
 
-    if (AFS_IS_DISCONNECTED) {
+    if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
         afs_warnuser("afs_ConnByHost: disconnected\n");
         return NULL;
     }
@@ -313,6 +347,18 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
 }				/*afs_ConnByHost */
 
 
+/**
+ * Connect by multiple hosts.
+ * Try to connect to one of the hosts from the ahosts array.
+ *
+ * @param ahosts Multiple hosts to connect to.
+ * @param aport Connection port.
+ * @param acell The cell where all of this happens.
+ * @param areq The request.
+ * @param locktype Type of lock to be used.
+ *
+ * @return The established connection or NULL.
+ */
 struct conn *
 afs_ConnByMHosts(struct server *ahosts[], unsigned short aport,
 		 afs_int32 acell, register struct vrequest *areq,
@@ -337,6 +383,11 @@ afs_ConnByMHosts(struct server *ahosts[], unsigned short aport,
 }				/*afs_ConnByMHosts */
 
 
+/**
+ * Decrement reference count to this connection.
+ * @param ac
+ * @param locktype
+ */
 void
 afs_PutConn(register struct conn *ac, afs_int32 locktype)
 {
@@ -345,10 +396,13 @@ afs_PutConn(register struct conn *ac, afs_int32 locktype)
 }				/*afs_PutConn */
 
 
-/* for multi homed clients, an RPC may timeout because of a
-client network interface going down. We need to reopen new 
-connections in this case
-*/
+/** 
+ * For multi homed clients, a RPC may timeout because of a 
+ * client network interface going down. We need to reopen new 
+ * connections in this case.
+ *
+ * @param sap Server address.
+ */
 void
 ForceNewConnections(struct srvAddr *sap)
 {
