@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_osi_alloc.c,v 1.11.6.3 2006/07/31 21:27:38 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_osi_alloc.c,v 1.11.6.7 2008/08/26 14:01:31 shadow Exp $");
 
 
 
@@ -19,7 +19,7 @@ RCSID
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 
-#ifndef AFS_FBSD_ENV
+
 
 #ifdef AFS_AIX41_ENV
 #include "sys/lockl.h"
@@ -207,7 +207,11 @@ osi_AllocSmallSpace(size_t size)
     if (!freeSmallList) {
 	afs_stats_cmperf.SmallBlocksAlloced++;
 	afs_stats_cmperf.SmallBlocksActive++;
-	return afs_osi_Alloc(AFS_SMALLOCSIZ);
+	tp = afs_osi_Alloc(AFS_SMALLOCSIZ);
+#ifdef KERNEL_HAVE_PIN
+        pin((char *)tp, AFS_SMALLOCSIZ);
+#endif
+        return (char *)tp;
     }
     afs_stats_cmperf.SmallBlocksActive++;
     MObtainWriteLock(&osi_fsplock, 327);
@@ -223,8 +227,6 @@ osi_AllocSmallSpace(size_t size)
 void
 shutdown_osinet(void)
 {
-    extern int afs_cold_shutdown;
-
     AFS_STATCNT(shutdown_osinet);
     if (afs_cold_shutdown) {
 	struct osi_packet *tp;
@@ -247,5 +249,12 @@ shutdown_osinet(void)
 	LOCK_INIT(&osi_fsplock, "osi_fsplock");
 	LOCK_INIT(&osi_flplock, "osi_flplock");
     }
+    if (afs_stats_cmperf.LargeBlocksActive || 
+	afs_stats_cmperf.SmallBlocksActive) 
+    {
+	afs_warn("WARNING: not all blocks freed: large %d small %d\n", 
+		 afs_stats_cmperf.LargeBlocksActive, 
+		 afs_stats_cmperf.SmallBlocksActive);
+    }
 }
-#endif
+
