@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_osidnlc.c,v 1.13 2006/02/18 04:08:04 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_osidnlc.c,v 1.13.2.3 2008/10/10 23:24:28 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -25,7 +25,7 @@ RCSID
  *    look into interactions of dnlc and readdir.
  *    cache larger names, perhaps by using a better,longer key (SHA) and discarding
  *    the actual name itself.
- *    precompute a key and stuff for @sys, and combine the HandleAtName function with
+ *    precompute a key and stuff for \sys, and combine the HandleAtName function with
  *    this, since we're looking at the name anyway.  
  */
 
@@ -63,6 +63,14 @@ int dnlct;
 
 #define dnlcHash(ts, hval) for (hval=0; *ts; ts++) { hval *= 173;  hval  += *ts;   }
 
+#if defined(AFS_FBSD80_ENV) && !defined(UKERNEL)
+#define ma_critical_enter critical_enter
+#define ma_critical_exit critical_exit
+#else
+#define ma_critical_enter() {}
+#define ma_critical_exit() {}
+#endif
+
 static struct nc *
 GetMeAnEntry(void)
 {
@@ -201,12 +209,17 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
     vnode_t tvp;
 #endif
 
-    if (!afs_usednlc)
-	return 0;
+    ma_critical_enter();
+
+    if (!afs_usednlc) {
+      ma_critical_exit();
+      return 0;
+    }
 
     dnlcHash(ts, key);		/* leaves ts pointing at the NULL */
     if (ts - aname >= AFSNCNAMESIZE) {
-	return 0;
+      ma_critical_exit();
+      return 0;
     }
     skey = key & (NHSIZE - 1);
 
@@ -231,6 +244,7 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
 	    ReleaseReadLock(&afs_xdnlc);
 	    ReleaseReadLock(&afs_xvcache);
 	    osi_dnlc_purge();
+	    ma_critical_exit();
 	    return (0);
 	}
     }
@@ -242,10 +256,16 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
 	ReleaseReadLock(&afs_xvcache);
 	dnlcstats.misses++;
     } else {
-	if (tvc->states & CVInit) {
+	if ((tvc->states & CVInit)
+#ifdef  AFS_DARWIN80_ENV
+	    ||(tvc->states & CDeadVnode)
+#endif
+	    )      
+	{
 	    ReleaseReadLock(&afs_xvcache);
 	    dnlcstats.misses++;
 	    osi_dnlc_remove(adp, aname, tvc);
+	    ma_critical_exit();
 	    return 0;
 	}
 #ifdef	AFS_OSF_ENV
@@ -257,6 +277,7 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
 	    ReleaseReadLock(&afs_xvcache);
 	    dnlcstats.misses++;
 	    osi_dnlc_remove(adp, aname, tvc);
+	    ma_critical_exit();
 	    return 0;
 	}
 	if (vnode_ref(tvp)) {
@@ -266,10 +287,18 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
 	    AFS_GLOCK();
 	    dnlcstats.misses++;
 	    osi_dnlc_remove(adp, aname, tvc);
+	    ma_critical_exit();
 	    return 0;
 	}
 #else
+#ifdef AFS_FBSD50_ENV
+	/* can't sleep in a critical section */
+	ma_critical_exit();
 	osi_vnhold(tvc, 0);
+	ma_critical_enter();
+#else
+	osi_vnhold(tvc, 0);
+#endif /* AFS_FBSD80_ENV */
 #endif
 #endif
 	ReleaseReadLock(&afs_xvcache);
@@ -308,6 +337,7 @@ osi_dnlc_lookup(struct vcache *adp, char *aname, int locktype)
 #endif
     }
 
+    ma_critical_exit();
     return tvc;
 }
 
@@ -382,10 +412,15 @@ osi_dnlc_remove(struct vcache *adp, char *aname, struct vcache *avc)
     return 0;
 }
 
-/* remove anything pertaining to this directory.  I can invalidate
+/*! 
+ * Remove anything pertaining to this directory.  I can invalidate
  * things without the lock, since I am just looking through the array,
  * but to move things off the lists or into the freelist, I need the
- * write lock */
+ * write lock 
+ *
+ * \param adp vcache entry for the directory to be purged.
+ * \return 0
+ */
 int
 osi_dnlc_purgedp(struct vcache *adp)
 {
@@ -424,7 +459,12 @@ osi_dnlc_purgedp(struct vcache *adp)
     return 0;
 }
 
-/* remove anything pertaining to this file */
+/*! 
+ * Remove anything pertaining to this file 
+ *
+ * \param File vcache entry.
+ * \return 0
+ */
 int
 osi_dnlc_purgevp(struct vcache *avc)
 {
