@@ -19,6 +19,7 @@
  * afs_FlushActiveVcaches
  * afs_VerifyVCache2
  * afs_WriteVCache
+ * afs_WriteVCacheDiscon
  * afs_SimpleVStat
  * afs_ProcessFS
  * TellALittleWhiteLie
@@ -26,6 +27,7 @@
  * afs_GetVCache
  * afs_LookupVCache
  * afs_GetRootVCache
+ * afs_UpdateStatus
  * afs_FetchStatus
  * afs_StuffVcache
  * afs_PutVCache
@@ -39,7 +41,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.114.2.9 2008/05/23 14:25:16 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.114.2.14 2008/10/12 18:07:01 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -81,10 +83,10 @@ int afs_norefpanic = 0;
 /* Forward declarations */
 static afs_int32 afs_QueueVCB(struct vcache *avc);
 
-/*
- * afs_HashCBRFid
- *
+/*!
  * Generate an index into the hash table for a given Fid.
+ * \param fid 
+ * \return The hash value.
  */
 static int
 afs_HashCBRFid(struct AFSFid *fid)
@@ -92,11 +94,11 @@ afs_HashCBRFid(struct AFSFid *fid)
     return (fid->Volume + fid->Vnode + fid->Unique) % CBRSIZE;
 }
 
-/*
- * afs_InsertHashCBR
- *
+/*!
  * Insert a CBR entry into the hash table.
  * Must be called with afs_xvcb held.
+ * \param cbr
+ * \return
  */
 static void
 afs_InsertHashCBR(struct afs_cbr *cbr)
@@ -111,15 +113,9 @@ afs_InsertHashCBR(struct afs_cbr *cbr)
     afs_cbrHashT[slot] = cbr;
 }
 
-/*
- * afs_FlushVCache
+/*!
  *
- * Description:
- *	Flush the given vcache entry.
- *
- * Parameters:
- *	avc : Pointer to vcache entry to flush.
- *	slept : Pointer to int to set 1 if we sleep/drop locks, 0 if we don't.
+ * Flush the given vcache entry.
  *
  * Environment:
  *	afs_xvcache lock must be held for writing upon entry to
@@ -128,8 +124,11 @@ afs_InsertHashCBR(struct afs_cbr *cbr)
  * LOCK: afs_FlushVCache afs_xvcache W
  * REFCNT: vcache ref count must be zero on entry except for osf1
  * RACE: lock is dropped and reobtained, permitting race in caller
+ *
+ * \param avc Pointer to vcache entry to flush.
+ * \param slept Pointer to int to set 1 if we sleep/drop locks, 0 if we don't.
+ *
  */
-
 int
 afs_FlushVCache(struct vcache *avc, int *slept)
 {				/*afs_FlushVCache */
@@ -277,10 +276,11 @@ afs_FlushVCache(struct vcache *avc, int *slept)
 }				/*afs_FlushVCache */
 
 #ifndef AFS_SGI_ENV
-/*
- * afs_InactiveVCache
+/*!
+ *  The core of the inactive vnode op for all but IRIX.
  *
- * The core of the inactive vnode op for all but IRIX.
+ * \param avc 
+ * \param acred
  */
 void
 afs_InactiveVCache(struct vcache *avc, struct AFS_UCRED *acred)
@@ -303,14 +303,14 @@ afs_InactiveVCache(struct vcache *avc, struct AFS_UCRED *acred)
 }
 #endif
 
-/*
- * afs_AllocCBR
- *
- * Description: allocate a callback return structure from the
+/*!
+ *   Allocate a callback return structure from the
  * free list and return it.
  *
- * Env: The alloc and free routines are both called with the afs_xvcb lock
+ * Environment: The alloc and free routines are both called with the afs_xvcb lock
  * held, so we don't have to worry about blocking in osi_Alloc.
+ *
+ * \return The allocated afs_cbr.
  */
 static struct afs_cbr *afs_cbrSpace = 0;
 struct afs_cbr *
@@ -342,15 +342,14 @@ afs_AllocCBR(void)
     return tsp;
 }
 
-/*
- * afs_FreeCBR
- *
- * Description: free a callback return structure, removing it from all lists.
- *
- * Parameters:
- *	asp -- the address of the structure to free.
+/*!
+ * Free a callback return structure, removing it from all lists.
  *
  * Environment: the xvcb lock is held over these calls.
+ *
+ * \param asp The address of the structure to free.
+ *
+ * \rerurn 0
  */
 int
 afs_FreeCBR(register struct afs_cbr *asp)
@@ -368,15 +367,13 @@ afs_FreeCBR(register struct afs_cbr *asp)
     return 0;
 }
 
-/*
- * afs_FlushVCBs
- *
- * Description: flush all queued callbacks to all servers.
- *
- * Parameters: none.
+/*!
+ *   Flush all queued callbacks to all servers.
  *
  * Environment: holds xvcb lock over RPC to guard against race conditions
  *	when a new callback is granted for the same file later on.
+ *
+ * \return 0 for success.
  */
 afs_int32
 afs_FlushVCBs(afs_int32 lockit)
@@ -490,18 +487,15 @@ afs_FlushVCBs(afs_int32 lockit)
     return 0;
 }
 
-/*
- * afs_QueueVCB
- *
- * Description:
- *	Queue a callback on the given fid.
- *
- * Parameters:
- *	avc: vcache entry
+/*!
+ *  Queue a callback on the given fid.
  *
  * Environment:
  *	Locks the xvcb lock.
  *	Called when the xvcache lock is already held.
+ *
+ * \param avc vcache entry
+ * \return 0 for success < 0 otherwise.
  */
 
 static afs_int32
@@ -536,19 +530,16 @@ afs_QueueVCB(struct vcache *avc)
 }
 
 
-/*
- * afs_RemoveVCB
- *
- * Description:
- *	Remove a queued callback for a given Fid.
- *
- * Parameters:
- *	afid: The fid we want cleansed of queued callbacks.
+/*!
+ *   Remove a queued callback for a given Fid.
  *
  * Environment:
  *	Locks xvcb and xserver locks.
  *	Typically called with xdcache, xvcache and/or individual vcache
  *	entries locked.
+ *
+ * \param afid The fid we want cleansed of queued callbacks.
+ *
  */
 
 void
@@ -621,21 +612,19 @@ afs_FlushReclaimedVcaches(void)
 #endif
 }
 
-/*
- * afs_NewVCache
+/*!
+ *   This routine is responsible for allocating a new cache entry
+ * from the free list.  It formats the cache entry and inserts it
+ * into the appropriate hash tables.  It must be called with
+ * afs_xvcache write-locked so as to prevent several processes from
+ * trying to create a new cache entry simultaneously.
  *
- * Description:
- *	This routine is responsible for allocating a new cache entry
- *	from the free list.  It formats the cache entry and inserts it
- *	into the appropriate hash tables.  It must be called with
- *	afs_xvcache write-locked so as to prevent several processes from
- *	trying to create a new cache entry simultaneously.
+ * LOCK: afs_NewVCache  afs_xvcache W
  *
- * Parameters:
- *	afid  : The file id of the file whose cache entry is being
- *		created.
+ * \param afid The file id of the file whose cache entry is being created.
+ *
+ * \return The new vcache struct.
  */
-/* LOCK: afs_NewVCache  afs_xvcache W */
 struct vcache *
 afs_NewVCache(struct VenusFid *afid, struct server *serverp)
 {
@@ -752,8 +741,9 @@ restart:
 		return NULL;
 	    }
 	}
-    }
+    } /* finished freeing up space */
 
+/* Alloc new vnode. */
 #if defined(AFS_LINUX22_ENV)
 {
     struct inode *ip;
@@ -868,7 +858,8 @@ restart:
 	    if (tq == uq)
 		break;
 	}
-    }
+    } /* end of if (!freeVCList) */
+
     if (!freeVCList) {
 	/* none free, making one is better than a panic */
 	afs_stats_cmperf.vcacheXAllocs++;	/* count in case we have a leak */
@@ -901,7 +892,8 @@ restart:
 	tvc = freeVCList;	/* take from free list */
 	freeVCList = tvc->nextfree;
 	tvc->nextfree = NULL;
-    }
+    } /* end of if (!freeVCList) */
+
 #endif /* AFS_OSF_ENV */
 
 #if defined(AFS_XBSD_ENV) || defined(AFS_DARWIN_ENV)
@@ -942,6 +934,10 @@ restart:
     tvc->Access = NULL;
     tvc->callback = serverp;    /* to minimize chance that clear
 				 * request is lost */
+#if defined(AFS_DISCON_ENV)
+    tvc->ddirty_next = NULL;
+    tvc->ddirty_flags = 0;
+#endif
 
     i = VCHash(afid);
     j = VCHashV(afid);
@@ -1033,6 +1029,12 @@ restart:
     tvc->vmh = tvc->segid = NULL;
     tvc->credp = NULL;
 #endif
+
+#if defined(AFS_CACHE_BYPASS)
+    tvc->cachingStates = 0;
+    tvc->cachingTransitions = 0;
+#endif
+
 #ifdef AFS_BOZONLOCK_ENV
 #if	defined(AFS_SUN5_ENV)
     rw_init(&tvc->rwlock, "vcache rwlock", RW_DEFAULT, NULL);
@@ -1122,6 +1124,11 @@ restart:
     insmntque(tvc, afs_globalVFS);
 #endif /* AFS_OSF_ENV */
 #endif /* AFS_DUX40_ENV */
+#ifdef AFS_FBSD70_ENV
+#ifndef AFS_FBSD80_ENV /* yup.  they put it back. */
+    insmntque(AFSTOV(tvc), afs_globalVFS);
+#endif
+#endif
 #if defined(AFS_SGI_ENV)
     VN_SET_DPAGES(&(tvc->v), (struct pfdat *)NULL);
     osi_Assert((tvc->v.v_flag & VINACT) == 0);
@@ -1153,16 +1160,13 @@ restart:
 }				/*afs_NewVCache */
 
 
-/*
- * afs_FlushActiveVcaches
+/*!
+ * ???
  *
- * Description:
- *	???
+ * LOCK: afs_FlushActiveVcaches afs_xvcache N
  *
- * Parameters:
- *	doflocks : Do we handle flocks?
+ * \param doflocks : Do we handle flocks?
  */
-/* LOCK: afs_FlushActiveVcaches afs_xvcache N */
 void
 afs_FlushActiveVcaches(register afs_int32 doflocks)
 {
@@ -1310,11 +1314,8 @@ afs_FlushActiveVcaches(register afs_int32 doflocks)
 
 
 
-/*
- * afs_VerifyVCache
- *
- * Description:
- *	Make sure a cache entry is up-to-date status-wise.
+/*!
+ *   Make sure a cache entry is up-to-date status-wise.
  *
  * NOTE: everywhere that calls this can potentially be sped up
  *       by checking CStatd first, and avoiding doing the InitReq
@@ -1323,11 +1324,26 @@ afs_FlushActiveVcaches(register afs_int32 doflocks)
  *  Anymore, the only places that call this KNOW already that the
  *  vcache is not up-to-date, so we don't screw around.
  *
- * Parameters:
- *	avc  : Ptr to vcache entry to verify.
- *	areq : ???
+ * \param avc  : Ptr to vcache entry to verify.
+ * \param areq : ???
  */
 
+/*!
+ * 
+ *   Make sure a cache entry is up-to-date status-wise.
+ *   
+ *   NOTE: everywhere that calls this can potentially be sped up
+ *       by checking CStatd first, and avoiding doing the InitReq
+ *       if this is up-to-date.
+ *
+ *   Anymore, the only places that call this KNOW already that the
+ * vcache is not up-to-date, so we don't screw around.
+ *
+ * \param avc Pointer to vcache entry to verify.
+ * \param areq
+ *
+ * \return 0 for success or other error codes.
+ */
 int
 afs_VerifyVCache2(struct vcache *avc, struct vrequest *areq)
 {
@@ -1377,20 +1393,14 @@ afs_VerifyVCache2(struct vcache *avc, struct vrequest *areq)
 }				/*afs_VerifyVCache */
 
 
-/*
- * afs_SimpleVStat
+/*!
+ * Simple copy of stat info into cache.
  *
- * Description:
- *	Simple copy of stat info into cache.
+ * Callers:as of 1992-04-29, only called by WriteVCache
  *
- * Parameters:
- *	avc   : Ptr to vcache entry involved.
- *	astat : Ptr to stat info to copy.
+ * \param avc   Ptr to vcache entry involved.
+ * \param astat Ptr to stat info to copy.
  *
- * Environment:
- *	Nothing interesting.
- *
- * Callers:  as of 1992-04-29, only called by WriteVCache
  */
 static void
 afs_SimpleVStat(register struct vcache *avc,
@@ -1473,20 +1483,17 @@ afs_SimpleVStat(register struct vcache *avc,
 }				/*afs_SimpleVStat */
 
 
-/*
- * afs_WriteVCache
+/*!
+ * Store the status info *only* back to the server for a
+ * fid/vrequest.
  *
- * Description:
- *	Store the status info *only* back to the server for a
- *	fid/vrequest.
+ * Environment: Must be called with a shared lock held on the vnode.
  *
- * Parameters:
- *	avc	: Ptr to the vcache entry.
- *	astatus	: Ptr to the status info to store.
- *	areq	: Ptr to the associated vrequest.
+ * \param avc Ptr to the vcache entry.
+ * \param astatus Ptr to the status info to store.
+ * \param areq Ptr to the associated vrequest.
  *
- * Environment:
- *	Must be called with a shared lock held on the vnode.
+ * \return Operation status.
  */
 
 int
@@ -1502,7 +1509,6 @@ afs_WriteVCache(register struct vcache *avc,
     AFS_STATCNT(afs_WriteVCache);
     afs_Trace2(afs_iclSetp, CM_TRACE_WVCACHE, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
-
     do {
 	tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
 	if (tc) {
@@ -1542,24 +1548,111 @@ afs_WriteVCache(register struct vcache *avc,
     return code;
 
 }				/*afs_WriteVCache */
+#if defined(AFS_DISCON_ENV)
 
-/*
- * afs_ProcessFS
+/*!
+ * Store status info only locally, set the proper disconnection flags
+ * and add to dirty list.
  *
- * Description:
- *	Copy astat block into vcache info
+ * \param avc The vcache to be written locally.
+ * \param astatus Get attr fields from local store.
+ * \param attrs This one is only of the vs_size.
  *
- * Parameters:
- *	avc   : Ptr to vcache entry.
- *	astat : Ptr to stat block to copy in.
- *	areq  : Ptr to associated request.
+ * \note Must be called with a shared lock on the vnode
+ */
+int afs_WriteVCacheDiscon(register struct vcache *avc,
+				register struct AFSStoreStatus *astatus,
+				struct vattr *attrs)
+{
+    afs_int32 code = 0;
+    afs_int32 flags = 0;
+
+    UpgradeSToWLock(&avc->lock, 700);
+
+    if (!astatus->Mask) {
+
+	return code;
+
+    } else {
+
+    	/* Set attributes. */
+    	if (astatus->Mask & AFS_SETMODTIME) {
+		avc->m.Date = astatus->ClientModTime;
+		flags |= VDisconSetTime;
+	}
+
+	if (astatus->Mask & AFS_SETOWNER) {
+		printf("Not allowed yet. \n");
+		//avc->m.Owner = astatus->Owner;
+	}
+
+	if (astatus->Mask & AFS_SETGROUP) {
+		printf("Not allowed yet. \n");
+		//avc->m.Group =  astatus->Group;
+	}
+
+	if (astatus->Mask & AFS_SETMODE) {
+		avc->m.Mode = astatus->UnixModeBits;
+
+#if 0 	/* XXX: Leaving this out, so it doesn't mess up the file type flag.*/
+
+		if (vType(avc) == VREG) {
+			avc->m.Mode |= S_IFREG;
+		} else if (vType(avc) == VDIR) {
+			avc->m.Mode |= S_IFDIR;
+		} else if (vType(avc) == VLNK) {
+			avc->m.Mode |= S_IFLNK;
+			if ((avc->m.Mode & 0111) == 0)
+				avc->mvstat = 1;
+		}
+#endif
+		flags |= VDisconSetMode;
+	 } 		/* if(astatus.Mask & AFS_SETMODE) */
+
+     } 			/* if (!astatus->Mask) */
+
+     if (attrs->va_size > 0) {
+     	/* XXX: Do I need more checks? */
+     	/* Truncation operation. */
+     	flags |= VDisconTrunc;
+     }
+
+    ObtainWriteLock(&afs_DDirtyVCListLock, 701);
+
+    if (flags) {
+    	/* Add to disconnected dirty list and set dirty flag.*/
+	if (!avc->ddirty_flags ||
+		(avc->ddirty_flags == VDisconShadowed)) {
+		/* Not in dirty list. */
+		AFS_DISCON_ADD_DIRTY(avc);
+	}
+
+	avc->ddirty_flags |= flags;
+    }
+
+    ReleaseWriteLock(&afs_DDirtyVCListLock);
+
+    /* XXX: How about the rest of the fields? */
+
+    ConvertWToSLock(&avc->lock);
+
+    return code;
+}
+
+#endif
+
+/*!
+ * Copy astat block into vcache info
  *
- * Environment:
- *	Must be called under a write lock
+ * \note This code may get dataversion and length out of sync if the file has
+ * been modified.  This is less than ideal.  I haven't thought about it sufficiently 
+ * to be certain that it is adequate.
  *
- * Note: this code may get dataversion and length out of sync if the file has
- *       been modified.  This is less than ideal.  I haven't thought about
- *       it sufficiently to be certain that it is adequate.
+ * \note Environment: Must be called under a write lock
+ *
+ * \param avc  Ptr to vcache entry.
+ * \param astat Ptr to stat block to copy in.
+ * \param areq Ptr to associated request.
  */
 void
 afs_ProcessFS(register struct vcache *avc,
@@ -1645,6 +1738,19 @@ afs_ProcessFS(register struct vcache *avc,
 }				/*afs_ProcessFS */
 
 
+/*!
+ * Get fid from server.
+ *
+ * \param afid 
+ * \param areq Request to be passed on.
+ * \param name Name of ?? to lookup.
+ * \param OutStatus Fetch status.
+ * \param CallBackp 
+ * \param serverp
+ * \param tsyncp
+ *
+ * \return Success status of operation.
+ */
 int
 afs_RemoteLookup(register struct VenusFid *afid, struct vrequest *areq,
 		 char *name, struct VenusFid *nfid,
@@ -1683,21 +1789,19 @@ afs_RemoteLookup(register struct VenusFid *afid, struct vrequest *areq,
 }
 
 
-/*
+/*!
  * afs_GetVCache
  *
- * Description:
- *	Given a file id and a vrequest structure, fetch the status
- *	information associated with the file.
+ * Given a file id and a vrequest structure, fetch the status
+ * information associated with the file.
  *
- * Parameters:
- *	afid : File ID.
- *	areq : Ptr to associated vrequest structure, specifying the
- *		user whose authentication tokens will be used.
- *      avc  : caller may already have a vcache for this file, which is
- *             already held.
+ * \param afid File ID.
+ * \param areq Ptr to associated vrequest structure, specifying the
+ *  user whose authentication tokens will be used.
+ * \param avc Caller may already have a vcache for this file, which is
+ *  already held.
  *
- * Environment:
+ * \note Environment:
  *	The cache entry is returned with an increased vrefCount field.
  *	The entry must be discarded by calling afs_PutVCache when you
  *	are through using the pointer to the cache entry.
@@ -1713,11 +1817,12 @@ afs_RemoteLookup(register struct VenusFid *afid, struct vrequest *areq,
  *	of a parent dir cache entry, given a file (to check its access
  *	control list).  It also allows renames to be handled easily by
  *	locking directories in a constant order.
- * NB.  NewVCache -> FlushVCache presently (4/10/95) drops the xvcache lock.
+ * 
+ * \note NB.  NewVCache -> FlushVCache presently (4/10/95) drops the xvcache lock.
+ *
+ * \note Might have a vcache structure already, which must
+ *  already be held by the caller 
  */
-   /* might have a vcache structure already, which must
-    * already be held by the caller */
-
 struct vcache *
 afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	      afs_int32 * cached, struct vcache *avc)
@@ -1822,6 +1927,20 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	  ObtainWriteLock(&tvc->lock, 954);
 	if (!iheldthelock)
 	    VOP_UNLOCK(vp, LK_EXCLUSIVE, current_proc());
+#elif defined(AFS_FBSD80_ENV)
+	iheldthelock = VOP_ISLOCKED(vp);
+	if (!iheldthelock) {
+	    /* nosleep/sleep lock order reversal */
+	    int glocked = ISAFS_GLOCK();
+	    if (glocked)
+		AFS_GUNLOCK();
+	    vn_lock(vp, LK_EXCLUSIVE | LK_RETRY);
+	    if (glocked)
+		AFS_GLOCK();
+	}
+	vinvalbuf(vp, V_SAVE, curthread, PINOD, 0);
+	if (!iheldthelock)
+	    VOP_UNLOCK(vp, 0);
 #elif defined(AFS_FBSD60_ENV)
 	iheldthelock = VOP_ISLOCKED(vp, curthread);
 	if (!iheldthelock)
@@ -1899,12 +2018,18 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	    tvc->parentUnique = OutStatus.ParentUnique;
 	    code = 0;
 	} else {
-	    /* If we've got here and we're disconnected, then we can go
-	     * no further
-	     */
+
 	    if (AFS_IS_DISCONNECTED) {
-		code = ENETDOWN;
-		/*printf("Network is down in afs_GetCache");*/
+		if (AFS_IS_DISCON_RW) {
+		    if (vType(tvc) == VDIR)
+		    	OutStatus.FileType = Directory;
+
+		    code = tvc?0:ENOENT;
+		} else {
+		    /* Nothing to do otherwise...*/
+		    code = ENETDOWN;
+		    printf("Network is down in afs_GetCache");
+		}
 	    } else
 	        code = afs_FetchStatus(tvc, afid, areq, &OutStatus);
 
@@ -1938,6 +2063,19 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 
 
 
+/*!
+ * Lookup a vcache by fid. Look inside the cache first, if not
+ * there, lookup the file on the server, and then get it's fresh
+ * cache entry.
+ * 
+ * \param afid
+ * \param areq 
+ * \param cached Is element cached? If NULL, don't answer.
+ * \param adp
+ * \param aname
+ *
+ * \return The found element or NULL.
+ */
 struct vcache *
 afs_LookupVCache(struct VenusFid *afid, struct vrequest *areq,
 		 afs_int32 * cached, struct vcache *adp, char *aname)
@@ -1996,7 +2134,7 @@ afs_LookupVCache(struct VenusFid *afid, struct vrequest *areq,
     origCBs = afs_allCBs;	/* if anything changes, we don't have a cb */
     
     if (AFS_IS_DISCONNECTED) {
-	/*printf("Network is down in afs_LookupVcache\n");*/
+	printf("Network is down in afs_LookupVcache\n");
         code = ENETDOWN;
     } else 
         code =
@@ -2357,9 +2495,70 @@ afs_GetRootVCache(struct VenusFid *afid, struct vrequest *areq,
 }
 
 
+/*!
+ * Update callback status and (sometimes) attributes of a vnode.
+ * Called after doing a fetch status RPC. Whilst disconnected, attributes
+ * shouldn't be written to the vcache here.
+ *
+ * \param avc
+ * \param afid
+ * \param areq
+ * \param Outsp Server status after rpc call.
+ * \param acb Callback for this vnode.
+ *
+ * \note The vcache must be write locked.
+ */
+void
+afs_UpdateStatus(struct vcache *avc,
+			struct VenusFid *afid,
+			struct vrequest *areq,
+			struct AFSFetchStatus *Outsp,
+			struct AFSCallBack *acb,
+			afs_uint32 start)
+{
+    struct volume *volp;
 
-/*
- * must be called with avc write-locked
+    if (!AFS_IN_SYNC)
+	/* Dont write status in vcache if resyncing after a disconnection. */
+	afs_ProcessFS(avc, Outsp, areq);
+
+    volp = afs_GetVolume(afid, areq, READ_LOCK);
+    ObtainWriteLock(&afs_xcbhash, 469);
+    avc->states |= CTruth;
+    if (avc->callback /* check for race */ ) {
+	if (acb->ExpirationTime != 0) {
+	    avc->cbExpires = acb->ExpirationTime + start;
+	    avc->states |= CStatd;
+	    avc->states &= ~CBulkFetching;
+	    afs_QueueCallback(avc, CBHash(acb->ExpirationTime), volp);
+    	} else if (avc->states & CRO) {
+	    /* ordinary callback on a read-only volume -- AFS 3.2 style */
+	    avc->cbExpires = 3600 + start;
+	    avc->states |= CStatd;
+	    avc->states &= ~CBulkFetching;
+	    afs_QueueCallback(avc, CBHash(3600), volp);
+    	} else {
+	    afs_DequeueCallback(avc);
+	    avc->callback = NULL;
+	    avc->states &= ~(CStatd | CUnique);
+	    if ((avc->states & CForeign) || (avc->fid.Fid.Vnode & 1))
+	    	osi_dnlc_purgedp(avc);	/* if it (could be) a directory */
+    	}
+    } else {
+    	afs_DequeueCallback(avc);
+    	avc->callback = NULL;
+    	avc->states &= ~(CStatd | CUnique);
+    	if ((avc->states & CForeign) || (avc->fid.Fid.Vnode & 1))
+	    osi_dnlc_purgedp(avc);	/* if it (could be) a directory */
+    }
+    ReleaseWriteLock(&afs_xcbhash);
+    if (volp)
+    	afs_PutVolume(volp, READ_LOCK);
+
+}
+
+/*!
+ * Must be called with avc write-locked
  * don't absolutely have to invalidate the hint unless the dv has
  * changed, but be sure to get it right else there will be consistency bugs.
  */
@@ -2372,7 +2571,6 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
     register struct conn *tc;
     struct AFSCallBack CallBack;
     struct AFSVolSync tsync;
-    struct volume *volp;
     XSTATS_DECLS;
     do {
 	tc = afs_Conn(afid, areq, SHARED_LOCK);
@@ -2396,38 +2594,7 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 	      SHARED_LOCK, NULL));
 
     if (!code) {
-	afs_ProcessFS(avc, Outsp, areq);
-	volp = afs_GetVolume(afid, areq, READ_LOCK);
-	ObtainWriteLock(&afs_xcbhash, 469);
-	avc->states |= CTruth;
-	if (avc->callback /* check for race */ ) {
-	    if (CallBack.ExpirationTime != 0) {
-		avc->cbExpires = CallBack.ExpirationTime + start;
-		avc->states |= CStatd;
-		avc->states &= ~CBulkFetching;
-		afs_QueueCallback(avc, CBHash(CallBack.ExpirationTime), volp);
-	    } else if (avc->states & CRO) {	/* ordinary callback on a read-only volume -- AFS 3.2 style */
-		avc->cbExpires = 3600 + start;
-		avc->states |= CStatd;
-		avc->states &= ~CBulkFetching;
-		afs_QueueCallback(avc, CBHash(3600), volp);
-	    } else {
-		afs_DequeueCallback(avc);
-		avc->callback = NULL;
-		avc->states &= ~(CStatd | CUnique);
-		if ((avc->states & CForeign) || (avc->fid.Fid.Vnode & 1))
-		    osi_dnlc_purgedp(avc);	/* if it (could be) a directory */
-	    }
-	} else {
-	    afs_DequeueCallback(avc);
-	    avc->callback = NULL;
-	    avc->states &= ~(CStatd | CUnique);
-	    if ((avc->states & CForeign) || (avc->fid.Fid.Vnode & 1))
-		osi_dnlc_purgedp(avc);	/* if it (could be) a directory */
-	}
-	ReleaseWriteLock(&afs_xcbhash);
-	if (volp)
-	    afs_PutVolume(volp, READ_LOCK);
+	afs_UpdateStatus(avc, afid, areq, Outsp, &CallBack, start);
     } else {
 	/* used to undo the local callback, but that's too extreme.
 	 * There are plenty of good reasons that fetchstatus might return
@@ -2590,17 +2757,12 @@ afs_StuffVcache(register struct VenusFid *afid,
 }				/*afs_StuffVcache */
 #endif
 
-/*
- * afs_PutVCache
+/*!
+ * Decrements the reference count on a cache entry.
  *
- * Description:
- *	Decrements the reference count on a cache entry.
+ * \param avc Pointer to the cache entry to decrement.
  *
- * Parameters:
- *	avc : Pointer to the cache entry to decrement.
- *
- * Environment:
- *	Nothing interesting.
+ * \note Environment: Nothing interesting.
  */
 void
 afs_PutVCache(register struct vcache *avc)
@@ -2620,6 +2782,15 @@ afs_PutVCache(register struct vcache *avc)
 }				/*afs_PutVCache */
 
 
+/*!
+ * Sleepa when searching for a vcache. Releases all the pending locks,
+ * sleeps then obtains the previously released locks.
+ *
+ * \param vcache Enter sleep state.
+ * \param flag Determines what locks to use.
+ *
+ * \return 
+ */
 static void findvc_sleep(struct vcache *avc, int flag) {
     if (flag & IS_SLOCK) {
 	    ReleaseSharedLock(&afs_xvcache);
@@ -2641,23 +2812,18 @@ static void findvc_sleep(struct vcache *avc, int flag) {
 	}
     }
 }
-/*
- * afs_FindVCache
+/*!
+ * Find a vcache entry given a fid.
  *
- * Description:
- *	Find a vcache entry given a fid.
+ * \param afid Pointer to the fid whose cache entry we desire.
+ * \param retry (SGI-specific) tell the caller to drop the lock on xvcache,
+ *  unlock the vnode, and try again.
+ * \param flag Bit 1 to specify whether to compute hit statistics.  Not
+ *  set if FindVCache is called as part of internal bookkeeping.
  *
- * Parameters:
- *	afid : Pointer to the fid whose cache entry we desire.
- *      retry: (SGI-specific) tell the caller to drop the lock on xvcache,
- *             unlock the vnode, and try again.
- *      flags: bit 1 to specify whether to compute hit statistics.  Not
- *             set if FindVCache is called as part of internal bookkeeping.
- *
- * Environment:
- *	Must be called with the afs_xvcache lock at least held at
- *	the read level.  In order to do the VLRU adjustment, the xvcache lock
- *      must be shared-- we upgrade it here.
+ * \note Environment: Must be called with the afs_xvcache lock at least held at
+ * the read level.  In order to do the VLRU adjustment, the xvcache lock
+ * must be shared-- we upgrade it here.
  */
 
 struct vcache *
@@ -2773,28 +2939,22 @@ afs_FindVCache(struct VenusFid *afid, afs_int32 * retry, afs_int32 flag)
     return tvc;
 }				/*afs_FindVCache */
 
-/*
- * afs_NFSFindVCache
+/*!
+ * Find a vcache entry given a fid. Does a wildcard match on what we
+ * have for the fid. If more than one entry, don't return anything.
  *
- * Description:
- *	Find a vcache entry given a fid. Does a wildcard match on what we
- *	have for the fid. If more than one entry, don't return anything.
- *
- * Parameters:
- *	avcp : Fill in pointer if we found one and only one.
- *	afid : Pointer to the fid whose cache entry we desire.
- *      retry: (SGI-specific) tell the caller to drop the lock on xvcache,
+ * \param avcp Fill in pointer if we found one and only one.
+ * \param afid Pointer to the fid whose cache entry we desire.
+ * \param retry (SGI-specific) tell the caller to drop the lock on xvcache,
  *             unlock the vnode, and try again.
- *      flags: bit 1 to specify whether to compute hit statistics.  Not
+ * \param flags bit 1 to specify whether to compute hit statistics.  Not
  *             set if FindVCache is called as part of internal bookkeeping.
  *
- * Environment:
- *	Must be called with the afs_xvcache lock at least held at
- *	the read level.  In order to do the VLRU adjustment, the xvcache lock
- *      must be shared-- we upgrade it here.
+ * \note Environment: Must be called with the afs_xvcache lock at least held at
+ *  the read level.  In order to do the VLRU adjustment, the xvcache lock
+ *  must be shared-- we upgrade it here.
  *
- * Return value:
- *	number of matches found.
+ * \return Number of matches found.
  */
 
 int afs_duplicate_nfs_fids = 0;
@@ -2947,10 +3107,10 @@ afs_NFSFindVCache(struct vcache **avcp, struct VenusFid *afid)
 
 
 
-/*
- * afs_vcacheInit
- *
+/*!
  * Initialize vcache related variables
+ *
+ * \param astatSize
  */
 void
 afs_vcacheInit(int astatSize)
@@ -3015,9 +3175,8 @@ afs_vcacheInit(int astatSize)
 	QInit(&afs_vhashTV[i]);
 }
 
-/*
- * shutdown_vcache
- *
+/*!
+ * Shutdown vcache.
  */
 void
 shutdown_vcache(void)
