@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bozo/fsbnodeops.c,v 1.12 2003/07/15 23:14:43 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bozo/fsbnodeops.c,v 1.15.2.2 2007/10/31 04:21:14 shadow Exp $");
 
 #include <sys/types.h>
 #include <lwp.h>
@@ -26,13 +26,7 @@ RCSID
 #else
 #include <sys/file.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <stdlib.h>
 
 #endif /* AFS_NT40_ENV */
@@ -40,15 +34,7 @@ RCSID
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <afs/afsutil.h>
 #include "bnode.h"
-
-static int fs_timeout(), fs_getstat(), fs_setstat(), fs_delete();
-static int fs_procexit(), fs_getstring(), fs_getparm(), fs_restartp();
-static int fs_hascore();
-struct bnode *fs_create();
-struct bnode *fsmr_create();
-
-static SetNeedsClock();
-static NudgeProcs();
+#include "bosprototypes.h"
 
 static int emergency = 0;
 
@@ -77,6 +63,77 @@ static int emergency = 0;
     The needsSalvage flag is cleared when the salvager exits.
 */
 
+struct fsbnode {
+    struct bnode b;
+    afs_int32 timeSDStarted;	/* time shutdown operation started */
+    char *filecmd;		/* command to start primary file server */
+    char *volcmd;		/* command to start secondary vol server */
+    char *salsrvcmd;            /* command to start salvageserver (demand attach fs) */
+    char *salcmd;		/* command to start salvager */
+    char *scancmd;		/* command to start scanner (MR-AFS) */
+    struct bnode_proc *fileProc;	/* process for file server */
+    struct bnode_proc *volProc;	/* process for vol server */
+    struct bnode_proc *salsrvProc;	/* process for salvageserver (demand attach fs) */
+    struct bnode_proc *salProc;	/* process for salvager */
+    struct bnode_proc *scanProc;	/* process for scanner (MR-AFS) */
+    afs_int32 lastFileStart;	/* last start for file */
+    afs_int32 lastVolStart;	/* last start for vol */
+    afs_int32 lastSalsrvStart;	/* last start for salvageserver (demand attach fs) */
+    afs_int32 lastScanStart;	/* last start for scanner (MR-AFS) */
+    char fileRunning;		/* file process is running */
+    char volRunning;		/* volser is running */
+    char salsrvRunning;		/* salvageserver is running (demand attach fs) */
+    char salRunning;		/* salvager is running */
+    char scanRunning;		/* scanner is running (MR_AFS) */
+    char fileSDW;		/* file shutdown wait */
+    char volSDW;		/* vol shutdown wait */
+    char salsrvSDW;		/* salvageserver shutdown wait (demand attach fs) */
+    char salSDW;		/* waiting for the salvager to shutdown */
+    char scanSDW;		/* scanner shutdown wait (MR_AFS) */
+    char fileKillSent;		/* kill signal has been sent */
+    char volKillSent;
+    char salsrvKillSent;        /* kill signal has been sent (demand attach fs) */
+    char salKillSent;
+    char scanKillSent;		/* kill signal has been sent (MR_AFS) */
+    char needsSalvage;		/* salvage before running */
+    char needsClock;		/* do we need clock ticks */
+};
+
+
+
+struct bnode * fs_create(char *ainstance, char *afilecmd, char *avolcmd, 
+			 char *asalcmd, char *ascancmd);
+struct bnode * dafs_create(char *ainstance, char *afilecmd, char *avolcmd, 
+			   char * asalsrvcmd, char *asalcmd, char *ascancmd);
+
+static int fs_hascore(register struct ezbnode *abnode);
+static int fs_restartp(register struct fsbnode *abnode);
+static int SetSalFlag(register struct fsbnode *abnode, register int aflag);
+static int RestoreSalFlag(register struct fsbnode *abnode);
+static int fs_delete(struct fsbnode *abnode);
+static int fs_timeout(struct fsbnode *abnode);
+static int fs_getstat(struct fsbnode *abnode, afs_int32 * astatus);
+static int fs_setstat(register struct fsbnode *abnode, afs_int32 astatus);
+static int fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc);
+static int fs_getstring(struct fsbnode *abnode, char *abuffer, afs_int32 alen);
+
+
+static int fs_getparm(struct fsbnode *abnode, afs_int32 aindex, 
+		      char *abuffer, afs_int32 alen);
+static int dafs_getparm(struct fsbnode *abnode, afs_int32 aindex, 
+			char *abuffer, afs_int32 alen);
+
+#ifdef AFS_NT40_ENV
+static void AppendExecutableExtension(char *cmd);
+#else
+#define AppendExecutableExtension(x)
+#endif
+
+static void SetNeedsClock(register struct fsbnode *ab);
+static int NudgeProcs(register struct fsbnode *abnode);
+
+
+
 struct bnode_ops fsbnode_ops = {
     fs_create,
     fs_timeout,
@@ -90,36 +147,21 @@ struct bnode_ops fsbnode_ops = {
     fs_hascore,
 };
 
-struct fsbnode {
-    struct bnode b;
-    afs_int32 timeSDStarted;	/* time shutdown operation started */
-    char *filecmd;		/* command to start primary file server */
-    char *volcmd;		/* command to start secondary vol server */
-    char *salcmd;		/* command to start salvager */
-    char *scancmd;		/* command to start scanner (MR-AFS) */
-    struct bnode_proc *fileProc;	/* process for file server */
-    struct bnode_proc *volProc;	/* process for vol server */
-    struct bnode_proc *salProc;	/* process for salvager */
-    struct bnode_proc *scanProc;	/* process for scanner (MR-AFS) */
-    afs_int32 lastFileStart;	/* last start for file */
-    afs_int32 lastVolStart;	/* last start for vol */
-    afs_int32 lastScanStart;	/* last start for scanner (MR-AFS) */
-    char fileRunning;		/* file process is running */
-    char volRunning;		/* volser is running */
-    char salRunning;		/* salvager is running */
-    char scanRunning;		/* scanner is running (MR_AFS) */
-    char fileSDW;		/* file shutdown wait */
-    char volSDW;		/* vol shutdown wait */
-    char salSDW;		/* waiting for the salvager to shutdown */
-    char scanSDW;		/* scanner shutdown wait (MR_AFS) */
-    char fileKillSent;		/* kill signal has been sent */
-    char volKillSent;
-    char salKillSent;
-    char scanKillSent;		/* kill signal has been sent (MR_AFS) */
-    char needsSalvage;		/* salvage before running */
-    char needsClock;		/* do we need clock ticks */
+/* demand attach fs bnode ops */
+struct bnode_ops dafsbnode_ops = {
+    dafs_create,
+    fs_timeout,
+    fs_getstat,
+    fs_setstat,
+    fs_delete,
+    fs_procexit,
+    fs_getstring,
+    dafs_getparm,
+    fs_restartp,
+    fs_hascore,
 };
 
+
 /* Function to tell whether this bnode has a core file or not.  You might
  * think that this could be in bnode.c, and decide what core files to check
  * for based on the bnode's coreName property, but that doesn't work because
@@ -141,6 +183,11 @@ fs_hascore(register struct ezbnode *abnode)
     if (access(tbuffer, 0) == 0)
 	return 1;
 
+    /* see if salvageserver left a core file */
+    bnode_CoreName(abnode, "salsrv", tbuffer);
+    if (access(tbuffer, 0) == 0)
+	return 1;
+
     /* see if salvager left a core file */
     bnode_CoreName(abnode, "salv", tbuffer);
     if (access(tbuffer, 0) == 0)
@@ -199,6 +246,25 @@ fs_restartp(register struct fsbnode *abnode)
     if (code)
 	return code;
 
+    if (abnode->salsrvcmd) {    /* only in demand attach fs */
+	/* now do same for salsrvcmd (demand attach fs) */
+	code = bnode_ParseLine(abnode->salsrvcmd, &tt);
+	if (code)
+	    return 0;
+	if (!tt)
+	    return 0;
+	code = stat(tt->key, &tstat);
+	if (code) {
+	    bnode_FreeTokens(tt);
+	    return 0;
+	}
+	if (tstat.st_ctime > abnode->lastScanStart)
+	    code = 1;
+	else
+	    code = 0;
+	bnode_FreeTokens(tt);
+    }
+
     if (abnode->scancmd) {	/* Only in MR-AFS */
 	/* now do same for scancmd (MR-AFS) */
 	code = bnode_ParseLine(abnode->scancmd, &tt);
@@ -229,14 +295,17 @@ SetSalFlag(register struct fsbnode *abnode, register int aflag)
     char tbuffer[AFSDIR_PATH_MAX];
     int fd;
 
-    abnode->needsSalvage = aflag;
-    strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
-	       SALFILE, abnode->b.name, NULL);
-    if (aflag) {
-	fd = open(tbuffer, O_CREAT | O_TRUNC | O_RDWR, 0666);
-	close(fd);
-    } else {
-	unlink(tbuffer);
+    /* don't use the salvage flag for demand attach fs */
+    if (abnode->salsrvcmd == NULL) {
+	abnode->needsSalvage = aflag;
+	strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
+		   SALFILE, abnode->b.name, NULL);
+	if (aflag) {
+	    fd = open(tbuffer, O_CREAT | O_TRUNC | O_RDWR, 0666);
+	    close(fd);
+	} else {
+	    unlink(tbuffer);
+	}
     }
     return 0;
 }
@@ -247,13 +316,18 @@ RestoreSalFlag(register struct fsbnode *abnode)
 {
     char tbuffer[AFSDIR_PATH_MAX];
 
-    strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
-	       SALFILE, abnode->b.name, NULL);
-    if (access(tbuffer, 0) == 0) {
-	/* file exists, so need to salvage */
-	abnode->needsSalvage = 1;
-    } else {
+    /* never set needs salvage flag for demand attach fs */
+    if (abnode->salsrvcmd != NULL) {
 	abnode->needsSalvage = 0;
+    } else {
+	strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
+		   SALFILE, abnode->b.name, NULL);
+	if (access(tbuffer, 0) == 0) {
+	    /* file exists, so need to salvage */
+	    abnode->needsSalvage = 1;
+	} else {
+	    abnode->needsSalvage = 0;
+	}
     }
     return 0;
 }
@@ -273,6 +347,8 @@ fs_delete(struct fsbnode *abnode)
     free(abnode->filecmd);
     free(abnode->volcmd);
     free(abnode->salcmd);
+    if (abnode->salsrvcmd)
+	free(abnode->salsrvcmd);
     if (abnode->scancmd)
 	free(abnode->scancmd);
     free(abnode);
@@ -305,95 +381,235 @@ fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
     char *fileCmdpath, *volCmdpath, *salCmdpath, *scanCmdpath;
     int bailout = 0;
 
-    fileCmdpath = volCmdpath = salCmdpath = NULL;
+    te = fileCmdpath = volCmdpath = salCmdpath = scanCmdpath = NULL;
 
     /* construct local paths from canonical (wire-format) paths */
     if (ConstructLocalBinPath(afilecmd, &fileCmdpath)) {
 	bozo_Log("BNODE: command path invalid '%s'\n", afilecmd);
 	bailout = 1;
+	goto done;
     }
     if (ConstructLocalBinPath(avolcmd, &volCmdpath)) {
 	bozo_Log("BNODE: command path invalid '%s'\n", avolcmd);
 	bailout = 1;
+	goto done;
     }
     if (ConstructLocalBinPath(asalcmd, &salCmdpath)) {
 	bozo_Log("BNODE: command path invalid '%s'\n", asalcmd);
 	bailout = 1;
+	goto done;
     }
 
     if (ascancmd && strlen(ascancmd)) {
 	if (ConstructLocalBinPath(ascancmd, &scanCmdpath)) {
 	    bozo_Log("BNODE: command path invalid '%s'\n", ascancmd);
 	    bailout = 1;
+	    goto done;
 	}
     }
 
     if (!bailout) {
 	sscanf(fileCmdpath, "%s", cmdname);
-#ifdef AFS_NT40_ENV
 	AppendExecutableExtension(cmdname);
-#endif
 	if (stat(cmdname, &tstat)) {
 	    bozo_Log("BNODE: file server binary '%s' not found\n", cmdname);
 	    bailout = 1;
+	    goto done;
 	}
 
 	sscanf(volCmdpath, "%s", cmdname);
-#ifdef AFS_NT40_ENV
 	AppendExecutableExtension(cmdname);
-#endif
 	if (stat(cmdname, &tstat)) {
 	    bozo_Log("BNODE: volume server binary '%s' not found\n", cmdname);
 	    bailout = 1;
+	    goto done;
 	}
 
 	sscanf(salCmdpath, "%s", cmdname);
-#ifdef AFS_NT40_ENV
 	AppendExecutableExtension(cmdname);
-#endif
 	if (stat(cmdname, &tstat)) {
 	    bozo_Log("BNODE: salvager binary '%s' not found\n", cmdname);
 	    bailout = 1;
+	    goto done;
 	}
 
 	if (ascancmd && strlen(ascancmd)) {
 	    sscanf(scanCmdpath, "%s", cmdname);
-#ifdef AFS_NT40_ENV
 	    AppendExecutableExtension(cmdname);
-#endif
 	    if (stat(cmdname, &tstat)) {
 		bozo_Log("BNODE: scanner binary '%s' not found\n", cmdname);
 		bailout = 1;
+		goto done;
 	    }
 	}
     }
 
-    if (bailout) {
-	free(fileCmdpath);
-	free(volCmdpath);
-	free(salCmdpath);
-	return NULL;
-    }
-
     te = (struct fsbnode *)malloc(sizeof(struct fsbnode));
+    if (te == NULL) {
+	bailout = 1;
+	goto done;
+    }
     memset(te, 0, sizeof(struct fsbnode));
     te->filecmd = fileCmdpath;
     te->volcmd = volCmdpath;
+    te->salsrvcmd = NULL;
     te->salcmd = salCmdpath;
     if (ascancmd && strlen(ascancmd))
 	te->scancmd = scanCmdpath;
     else
 	te->scancmd = NULL;
     if (bnode_InitBnode(te, &fsbnode_ops, ainstance) != 0) {
-	free(te);
-	free(fileCmdpath);
-	free(volCmdpath);
-	free(salCmdpath);
-	return NULL;
+	bailout = 1;
+	goto done;
     }
     bnode_SetTimeout(te, POLLTIME);	/* ask for timeout activations every 10 seconds */
     RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
     SetNeedsClock(te);		/* compute needsClock field */
+
+ done:
+    if (bailout) {
+	if (te)
+	    free(te);
+	if (fileCmdpath)
+	    free(fileCmdpath);
+	if (volCmdpath)
+	    free(volCmdpath);
+	if (salCmdpath)
+	    free(salCmdpath);
+	if (scanCmdpath)
+	    free(scanCmdpath);
+	return NULL;
+    }
+
+    return (struct bnode *)te;
+}
+
+/* create a demand attach fs bnode */
+struct bnode *
+dafs_create(char *ainstance, char *afilecmd, char *avolcmd, 
+	    char * asalsrvcmd, char *asalcmd, char *ascancmd)
+{
+    struct stat tstat;
+    register struct fsbnode *te;
+    char cmdname[AFSDIR_PATH_MAX];
+    char *fileCmdpath, *volCmdpath, *salsrvCmdpath, *salCmdpath, *scanCmdpath;
+    int bailout = 0;
+
+    te = fileCmdpath = volCmdpath = salsrvCmdpath = salCmdpath = scanCmdpath = NULL;
+
+    /* construct local paths from canonical (wire-format) paths */
+    if (ConstructLocalBinPath(afilecmd, &fileCmdpath)) {
+	bozo_Log("BNODE: command path invalid '%s'\n", afilecmd);
+	bailout = 1;
+	goto done;
+    }
+    if (ConstructLocalBinPath(avolcmd, &volCmdpath)) {
+	bozo_Log("BNODE: command path invalid '%s'\n", avolcmd);
+	bailout = 1;
+	goto done;
+    }
+    if (ConstructLocalBinPath(asalsrvcmd, &salsrvCmdpath)) {
+	bozo_Log("BNODE: command path invalid '%s'\n", asalsrvcmd);
+	bailout = 1;
+	goto done;
+    }
+    if (ConstructLocalBinPath(asalcmd, &salCmdpath)) {
+	bozo_Log("BNODE: command path invalid '%s'\n", asalcmd);
+	bailout = 1;
+	goto done;
+    }
+
+    if (ascancmd && strlen(ascancmd)) {
+	if (ConstructLocalBinPath(ascancmd, &scanCmdpath)) {
+	    bozo_Log("BNODE: command path invalid '%s'\n", ascancmd);
+	    bailout = 1;
+	    goto done;
+	}
+    }
+
+    if (!bailout) {
+	sscanf(fileCmdpath, "%s", cmdname);
+	AppendExecutableExtension(cmdname);
+	if (stat(cmdname, &tstat)) {
+	    bozo_Log("BNODE: file server binary '%s' not found\n", cmdname);
+	    bailout = 1;
+	    goto done;
+	}
+
+	sscanf(volCmdpath, "%s", cmdname);
+	AppendExecutableExtension(cmdname);
+	if (stat(cmdname, &tstat)) {
+	    bozo_Log("BNODE: volume server binary '%s' not found\n", cmdname);
+	    bailout = 1;
+	    goto done;
+	}
+
+	sscanf(salsrvCmdpath, "%s", cmdname);
+	AppendExecutableExtension(cmdname);
+	if (stat(cmdname, &tstat)) {
+	    bozo_Log("BNODE: salvageserver binary '%s' not found\n", cmdname);
+	    bailout = 1;
+	    goto done;
+	}
+
+	sscanf(salCmdpath, "%s", cmdname);
+	AppendExecutableExtension(cmdname);
+	if (stat(cmdname, &tstat)) {
+	    bozo_Log("BNODE: salvager binary '%s' not found\n", cmdname);
+	    bailout = 1;
+	    goto done;
+	}
+
+	if (ascancmd && strlen(ascancmd)) {
+	    sscanf(scanCmdpath, "%s", cmdname);
+	    AppendExecutableExtension(cmdname);
+	    if (stat(cmdname, &tstat)) {
+		bozo_Log("BNODE: scanner binary '%s' not found\n", cmdname);
+		bailout = 1;
+		goto done;
+	    }
+	}
+    }
+
+    te = (struct fsbnode *)malloc(sizeof(struct fsbnode));
+    if (te == NULL) {
+	bailout = 1;
+	goto done;
+    }
+    memset(te, 0, sizeof(struct fsbnode));
+    te->filecmd = fileCmdpath;
+    te->volcmd = volCmdpath;
+    te->salsrvcmd = salsrvCmdpath;
+    te->salcmd = salCmdpath;
+    if (ascancmd && strlen(ascancmd))
+	te->scancmd = scanCmdpath;
+    else
+	te->scancmd = NULL;
+    if (bnode_InitBnode(te, &dafsbnode_ops, ainstance) != 0) {
+	bailout = 1;
+	goto done;
+    }
+    bnode_SetTimeout(te, POLLTIME);	/* ask for timeout activations every 10 seconds */
+    RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
+    SetNeedsClock(te);		/* compute needsClock field */
+
+ done:
+    if (bailout) {
+	if (te)
+	    free(te);
+	if (fileCmdpath)
+	    free(fileCmdpath);
+	if (volCmdpath)
+	    free(volCmdpath);
+	if (salsrvCmdpath)
+	    free(salsrvCmdpath);
+	if (salCmdpath)
+	    free(salCmdpath);
+	if (scanCmdpath)
+	    free(scanCmdpath);
+	return NULL;
+    }
+
     return (struct bnode *)te;
 }
 
@@ -432,6 +648,15 @@ fs_timeout(struct fsbnode *abnode)
 		 FSSDTIME);
 	}
     }
+    if (abnode->salsrvSDW) {
+	if (!abnode->salsrvKillSent && now - abnode->timeSDStarted > SDTIME) {
+	    bnode_StopProc(abnode->salsrvProc, SIGKILL);
+	    abnode->salsrvKillSent = 1;
+	    bozo_Log
+		("bos shutdown: salvageserver failed to shutdown within %d seconds\n",
+		 SDTIME);
+	}
+    }
     if (abnode->scanSDW) {
 	if (!abnode->scanKillSent && now - abnode->timeSDStarted > SDTIME) {
 	    bnode_StopProc(abnode->scanProc, SIGKILL);
@@ -450,15 +675,17 @@ fs_getstat(struct fsbnode *abnode, afs_int32 * astatus)
 {
     register afs_int32 temp;
     if (abnode->volSDW || abnode->fileSDW || abnode->salSDW
-	|| abnode->scanSDW)
+	|| abnode->scanSDW || abnode->salsrvSDW)
 	temp = BSTAT_SHUTTINGDOWN;
     else if (abnode->salRunning)
 	temp = BSTAT_NORMAL;
     else if (abnode->volRunning && abnode->fileRunning
-	     && (!abnode->scancmd || abnode->scanRunning))
+	     && (!abnode->scancmd || abnode->scanRunning)
+	     && (!abnode->salsrvcmd || abnode->salsrvRunning))
 	temp = BSTAT_NORMAL;
     else if (!abnode->salRunning && !abnode->volRunning
-	     && !abnode->fileRunning && !abnode->scanRunning)
+	     && !abnode->fileRunning && !abnode->scanRunning
+	     && !abnode->salsrvRunning)
 	temp = BSTAT_SHUTDOWN;
     else
 	temp = BSTAT_STARTINGUP;
@@ -509,6 +736,11 @@ fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc)
 	abnode->scanRunning = 0;
 	abnode->scanSDW = 0;
 	abnode->scanKillSent = 0;
+    } else if (aproc == abnode->salsrvProc) {
+	abnode->salsrvProc = 0;
+	abnode->salsrvRunning = 0;
+	abnode->salsrvSDW = 0;
+	abnode->salsrvKillSent = 0;
     }
 
     /* now restart anyone who needs to restart */
@@ -516,14 +748,15 @@ fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc)
 }
 
 /* make sure we're periodically checking the state if we need to */
-static int
+static void
 SetNeedsClock(register struct fsbnode *ab)
 {
     if (ab->b.goal == 1 && ab->fileRunning && ab->volRunning
-	&& (!ab->scancmd || ab->scanRunning))
+	&& (!ab->scancmd || ab->scanRunning)
+	&& (!ab->salsrvcmd || ab->salsrvRunning))
 	ab->needsClock = 0;	/* running normally */
     else if (ab->b.goal == 0 && !ab->fileRunning && !ab->volRunning
-	     && !ab->salRunning && !ab->scanRunning)
+	     && !ab->salRunning && !ab->scanRunning && !ab->salsrvRunning)
 	ab->needsClock = 0;	/* halted normally */
     else
 	ab->needsClock = 1;	/* other */
@@ -563,6 +796,18 @@ NudgeProcs(register struct fsbnode *abnode)
 		    abnode->volRunning = 1;
 		}
 	    }
+	    if (abnode->salsrvcmd) {
+		if (!abnode->salsrvRunning) {
+		    abnode->lastSalsrvStart = FT_ApproxTime();
+		    code =
+			bnode_NewProc(abnode, abnode->salsrvcmd, "salsrv",
+				      &tp);
+		    if (code == 0) {
+			abnode->salsrvProc = tp;
+			abnode->salsrvRunning = 1;
+		    }
+		}
+	    }
 	    if (abnode->scancmd) {
 		if (!abnode->scanRunning) {
 		    abnode->lastScanStart = FT_ApproxTime();
@@ -577,7 +822,8 @@ NudgeProcs(register struct fsbnode *abnode)
 	    }
 	} else {		/* file is not running */
 	    /* see how to start */
-	    if (!abnode->needsSalvage) {
+	    /* for demand attach fs, needsSalvage flag is ignored */
+	    if (!abnode->needsSalvage || abnode->salsrvcmd) {
 		/* no crash apparent, just start up normally */
 		if (!abnode->fileRunning) {
 		    abnode->lastFileStart = FT_ApproxTime();
@@ -597,6 +843,16 @@ NudgeProcs(register struct fsbnode *abnode)
 			abnode->volRunning = 1;
 		    }
 		}
+		if (abnode->salsrvcmd && !abnode->salsrvRunning) {
+		    abnode->lastSalsrvStart = FT_ApproxTime();
+		    code =
+			bnode_NewProc(abnode, abnode->salsrvcmd, "salsrv",
+				      &tp);
+		    if (code == 0) {
+			abnode->salsrvProc = tp;
+			abnode->salsrvRunning = 1;
+		    }
+		}
 		if (abnode->scancmd && !abnode->scanRunning) {
 		    abnode->lastScanStart = FT_ApproxTime();
 		    code =
@@ -657,6 +913,11 @@ NudgeProcs(register struct fsbnode *abnode)
 	    abnode->volSDW = 1;
 	    abnode->timeSDStarted = now;
 	}
+	if (abnode->salsrvRunning && !abnode->salsrvSDW) {
+	    bnode_StopProc(abnode->salsrvProc, SIGTERM);
+	    abnode->salsrvSDW = 1;
+	    abnode->timeSDStarted = now;
+	}
 	if (abnode->scanRunning && !abnode->scanSDW) {
 	    bnode_StopProc(abnode->scanProc, SIGTERM);
 	    abnode->scanSDW = 1;
@@ -725,3 +986,22 @@ fs_getparm(struct fsbnode *abnode, afs_int32 aindex, char *abuffer,
 	return BZDOM;
     return 0;
 }
+
+static int
+dafs_getparm(struct fsbnode *abnode, afs_int32 aindex, char *abuffer,
+	     afs_int32 alen)
+{
+    if (aindex == 0)
+	strcpy(abuffer, abnode->filecmd);
+    else if (aindex == 1)
+	strcpy(abuffer, abnode->volcmd);
+    else if (aindex == 2)
+	strcpy(abuffer, abnode->salsrvcmd);
+    else if (aindex == 3)
+	strcpy(abuffer, abnode->salcmd);
+    else if (aindex == 4 && abnode->scancmd)
+	strcpy(abuffer, abnode->scancmd);
+    else
+	return BZDOM;
+    return 0;
+}
