@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/vol_sets.c,v 1.11.6.2 2007/01/05 03:32:24 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/vol_sets.c,v 1.11.6.5 2007/11/26 21:08:41 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -41,6 +41,7 @@ extern char *whoami;
 extern struct bc_volumeSet *bc_FindVolumeSet(struct bc_config *cf, char *name);
 extern void FreeVolumeSet(struct bc_volumeSet *avset);
 
+static ListVolSet();
 
 /* ------------------------------------
  * command level routines
@@ -57,11 +58,10 @@ extern void FreeVolumeSet(struct bc_volumeSet *avset);
  *	parm 3 is volume regexp
  */
 
-bc_AddVolEntryCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_AddVolEntryCmd(struct cmd_syndesc *as, void *arock)
 {
-    register afs_int32 code;
+    register int code;
     char *volSetName, *serverName, *partitionName, *volRegExp;
     udbClientTextP ctPtr;
     struct bc_volumeSet *tset;
@@ -73,7 +73,7 @@ bc_AddVolEntryCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -81,7 +81,7 @@ bc_AddVolEntryCmd(as, arock)
 
     tset = bc_FindVolumeSet(bc_globalConfig, volSetName);
     if (!tset) {
-	com_err(whoami, code, "; Volume entry not added");
+	afs_com_err(whoami, code, "; Volume entry not added");
 	ERROR(code);
     }
 
@@ -93,7 +93,7 @@ bc_AddVolEntryCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -101,15 +101,15 @@ bc_AddVolEntryCmd(as, arock)
 	bc_AddVolumeItem(bc_globalConfig, volSetName, serverName,
 			 partitionName, volRegExp);
     if (code) {
-	com_err(whoami, code, "; Volume entry not added");
+	afs_com_err(whoami, code, "; Volume entry not added");
 	ERROR(code);
     }
 
     if (!(tset->flags & VSFLAG_TEMPORARY)) {
 	code = bc_SaveVolumeSet();
 	if (code) {
-	    com_err(whoami, code, "Cannot save volume set file");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, code, "Cannot save volume set file");
+	    afs_com_err(whoami, 0,
 		    "Changes are temporary - for this session only");
 	}
     }
@@ -129,13 +129,11 @@ bc_AddVolEntryCmd(as, arock)
  *	name of new volume set
  */
 
-afs_int32
-bc_AddVolSetCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_AddVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is vol set name */
-    register afs_int32 code;
+    register int code;
     register struct cmd_item *ti;
     udbClientTextP ctPtr;
     afs_int32 flags;
@@ -154,7 +152,7 @@ bc_AddVolSetCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -164,14 +162,14 @@ bc_AddVolSetCmd(as, arock)
 			   flags);
     if (code) {
 	if (code == -1)
-	    com_err(whoami, 0, "Volume set '%s' already exists", ti->data);
+	    afs_com_err(whoami, 0, "Volume set '%s' already exists", ti->data);
 	else
-	    com_err(whoami, 0, "Unknown problem");
+	    afs_com_err(whoami, 0, "Unknown problem");
     } else if (!(flags & VSFLAG_TEMPORARY)) {
 	code = bc_SaveVolumeSet();
 	if (code) {
-	    com_err(whoami, code, "Cannot save new volume set file");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, code, "Cannot save new volume set file");
+	    afs_com_err(whoami, 0,
 		    "Changes are temporary - for this session only");
 	}
     }
@@ -190,12 +188,10 @@ bc_AddVolSetCmd(as, arock)
  *	parm 1 is entry # (integer, 1 based)
  */
 
-afs_int32
-bc_DeleteVolEntryCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_DeleteVolEntryCmd(struct cmd_syndesc *as, void *arock)
 {
-    register afs_int32 code;
+    register int code;
     afs_int32 entry;
     char *vsname;
     udbClientTextP ctPtr;
@@ -205,7 +201,7 @@ bc_DeleteVolEntryCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -214,7 +210,7 @@ bc_DeleteVolEntryCmd(as, arock)
 
     tset = bc_FindVolumeSet(bc_globalConfig, vsname);
     if (!tset) {
-	com_err(whoami, 0, "No such volume set as '%s'", vsname);
+	afs_com_err(whoami, 0, "No such volume set as '%s'", vsname);
 	ERROR(code);
     }
 
@@ -226,13 +222,13 @@ bc_DeleteVolEntryCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
     entry = bc_SafeATOI(as->parms[1].items->data);
     if (entry < 0) {
-	com_err(whoami, 0, "Can't parse entry number '%s' as decimal integer",
+	afs_com_err(whoami, 0, "Can't parse entry number '%s' as decimal integer",
 		as->parms[1].items->data);
 	ERROR(BC_BADARG);
     }
@@ -240,12 +236,12 @@ bc_DeleteVolEntryCmd(as, arock)
     code = bc_DeleteVolumeItem(bc_globalConfig, vsname, entry);
     if (code) {
 	if (code == -1) {
-	    com_err(whoami, 0, "No such volume set as '%s'", vsname);
+	    afs_com_err(whoami, 0, "No such volume set as '%s'", vsname);
 	} else if (code == -2) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "There aren't %d volume items for this volume set",
 		    entry);
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "Use the 'listvolsets' command to examine the volume set");
 	}
 	ERROR(code);
@@ -257,8 +253,8 @@ bc_DeleteVolEntryCmd(as, arock)
 	    printf("backup: deleted volume entry %d from volume set %s\n",
 		   entry, vsname);
 	} else {
-	    com_err(whoami, code, "Cannot save volume set file");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, code, "Cannot save volume set file");
+	    afs_com_err(whoami, 0,
 		    "Deletion is temporary - for this session only");
 	}
     }
@@ -279,13 +275,11 @@ bc_DeleteVolEntryCmd(as, arock)
  *	name of volumeset to delete
  */
 
-afs_int32
-bc_DeleteVolSetCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_DeleteVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is vol set name */
-    register afs_int32 code;
+    register int code;
     register struct cmd_item *ti;
     udbClientTextP ctPtr;
     afs_int32 c;
@@ -300,7 +294,7 @@ bc_DeleteVolSetCmd(as, arock)
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -308,9 +302,9 @@ bc_DeleteVolSetCmd(as, arock)
 	code = bc_DeleteVolumeSet(bc_globalConfig, ti->data, &flags);
 	if (code) {
 	    if (code == -1)
-		com_err(whoami, 0, "Can't find volume set '%s'", ti->data);
+		afs_com_err(whoami, 0, "Can't find volume set '%s'", ti->data);
 	    else
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"; Unknown problem deleting volume set '%s'",
 			ti->data);
 	} else {
@@ -326,8 +320,8 @@ bc_DeleteVolSetCmd(as, arock)
 	if (c) {
 	    if (!code)
 		code = c;
-	    com_err(whoami, c, "Cannot save updated volume set file");
-	    com_err(whoami, 0, "Deletion effective for this session only");
+	    afs_com_err(whoami, c, "Cannot save updated volume set file");
+	    afs_com_err(whoami, 0, "Deletion effective for this session only");
 	}
 
     }
@@ -364,17 +358,17 @@ ListVolSet(struct bc_volumeSet *aset)
   *     optional parameter specifies a volumeset name
   */
 
-afs_int32
-bc_ListVolSetCmd(struct cmd_syndesc *as, char *arock)
+int
+bc_ListVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is optional volume set to display */
     register struct bc_volumeSet *tset;
     register struct cmd_item *ti;
-    afs_int32 code = 0;
+    int code = 0;
 
     code = bc_UpdateVolumeSet();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve volume sets");
+	afs_com_err(whoami, code, "; Can't retrieve volume sets");
 	return (code);
     }
 
@@ -387,7 +381,7 @@ bc_ListVolSetCmd(struct cmd_syndesc *as, char *arock)
 		ListVolSet(tset);
 		printf("\n");
 	    } else {
-		com_err(whoami, 0, "Can't find volume set '%s'", ti->data);
+		afs_com_err(whoami, 0, "Can't find volume set '%s'", ti->data);
 		code = 1;
 	    }
 	}
@@ -495,7 +489,7 @@ bc_ParseVolumeSet()
 	    if ((code != 2)
 		|| (strcmp(serverName, "volumeset") != 0)
 		) {
-		com_err(whoami, 0, "Bad volume header line: '%s'", tbuffer);
+		afs_com_err(whoami, 0, "Bad volume header line: '%s'", tbuffer);
 		return (-1);
 	    }
 
@@ -534,13 +528,13 @@ bc_ParseVolumeSet()
 	    tve = (struct bc_volumeEntry *)
 		malloc(sizeof(struct bc_volumeEntry));
 	    if (!tve) {
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec record!");
 		return (-1);
 	    }
 	    memset(tve, 0, sizeof(*tve));
 	    if (bc_ParseHost(serverName, &(tve->server)))
-		com_err(whoami, 0, "Can't get required info on host '%s'",
+		afs_com_err(whoami, 0, "Can't get required info on host '%s'",
 			serverName);
 
 	    /* The above code has filled in the server sockaddr, now fill in
@@ -548,26 +542,26 @@ bc_ParseVolumeSet()
 	     */
 	    tve->serverName = (char *)malloc(strlen(serverName) + 1);
 	    if (!tve->serverName) {
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec server name field!");
 		return (-1);
 	    }
 	    strcpy(tve->serverName, serverName);
 	    tve->partname = (char *)malloc(strlen(partName) + 1);
 	    if (!tve->partname) {
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec partition pattern field!");
 		return (-1);
 	    }
 	    strcpy(tve->partname, partName);
 	    code = bc_GetPartitionID(partName, &tve->partition);
 	    if (code) {
-		com_err(whoami, 0, "Can't parse partition '%s'", partName);
+		afs_com_err(whoami, 0, "Can't parse partition '%s'", partName);
 		return -1;
 	    }
 	    tp = (char *)malloc(strlen(vsname) + 1);
 	    if (!tp) {
-		com_err(whoami, 0,
+		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec volume pattern field!");
 		return (-1);
 	    }
