@@ -16,7 +16,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/budb/db_dump.c,v 1.7.14.3 2008/04/02 19:51:55 shadow Exp $");
+    ("$Header: /cvs/openafs/src/budb/db_dump.c,v 1.7.14.4 2008/07/19 06:15:01 rra Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -73,7 +73,7 @@ canWrite(fid)
     while (dumpSyncPtr->ds_bytes > 0) {
 	if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	    dumpSyncPtr->ds_readerStatus = 0;
-#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+#ifdef AFS_PTHREAD_ENV
 	    assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
 #else
 	    code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
@@ -83,7 +83,7 @@ canWrite(fid)
 	}
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
-#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+#ifdef AFS_PTHREAD_ENV
 	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
 	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
 	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
@@ -113,7 +113,7 @@ haveWritten(nbytes)
     dumpSyncPtr->ds_bytes += nbytes;
     if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	dumpSyncPtr->ds_readerStatus = 0;
-#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+#ifdef AFS_PTHREAD_ENV
 	assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
 #else
 	code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
@@ -141,7 +141,7 @@ doneWriting(error)
 	LogDebug(4, "doneWriting: waiting for Reader\n");
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
-#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+#ifdef AFS_PTHREAD_ENV
 	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
 	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
 	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
@@ -159,7 +159,7 @@ doneWriting(error)
     else
 	dumpSyncPtr->ds_writerStatus = DS_DONE;
     dumpSyncPtr->ds_readerStatus = 0;
-#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+#ifdef AFS_PTHREAD_ENV
     assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
 #else
     code = LWP_NoYieldSignal(&dumpSyncPtr->ds_readerStatus);
