@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/budb/db_text.c,v 1.12.4.1 2007/02/09 01:00:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/budb/db_text.c,v 1.12.4.3 2007/11/26 21:08:41 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -22,13 +22,7 @@ RCSID
 #include <sys/file.h>
 #include <sys/param.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <sys/types.h>
 #include <ubik.h>
 #include <afs/bubasics.h>
@@ -489,7 +483,7 @@ SaveText(call, lockHandle, textType, offset, flags, charListPtr)
 }
 
 /* debug support */
-
+void
 saveTextToFile(ut, tbPtr)
      struct ubik_trans *ut;
      struct textBlock *tbPtr;
