@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/butm/file_tm.c,v 1.12 2005/04/03 18:09:27 shadow Exp $");
+    ("$Header: /cvs/openafs/src/butm/file_tm.c,v 1.12.6.3 2007/11/26 21:08:42 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -24,6 +24,7 @@ RCSID
 #include <limits.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
 #include <lwp.h>
 #include <afs/com_err.h>
@@ -626,6 +627,7 @@ ShutdownAccess(usd_handle_t fid)
  *	dataSize - bytes used on the tape
  */
 
+void
 incSize(info, dataSize)
      struct butm_tapeInfo *info;
      afs_uint32 dataSize;
@@ -647,6 +649,7 @@ incSize(info, dataSize)
  *	dataSize - bytes used on the tape
  */
 
+void
 incPosition(info, fid, dataSize)
      struct butm_tapeInfo *info;
      usd_handle_t fid;
@@ -1752,7 +1755,7 @@ file_Configure(file)
      struct tapeConfig *file;
 {
     if (!file) {
-	com_err(whoami, BUTM_BADCONFIG, "device not specified");
+	afs_com_err(whoami, BUTM_BADCONFIG, "device not specified");
 	return BUTM_BADCONFIG;
     }
 
@@ -1763,13 +1766,13 @@ file_Configure(file)
 
     /* Tape must be large enough to at least fit a label */
     if (config.tapeSize <= 0) {
-	com_err(whoami, BUTM_BADCONFIG, "Tape size bogus: %d Kbytes",
+	afs_com_err(whoami, BUTM_BADCONFIG, "Tape size bogus: %d Kbytes",
 		config.tapeSize);
 	return BUTM_BADCONFIG;
     }
 
     if (strlen(config.tapedir) == 0) {
-	com_err(whoami, BUTM_BADCONFIG, "no tape device specified");
+	afs_com_err(whoami, BUTM_BADCONFIG, "no tape device specified");
 	return BUTM_BADCONFIG;
     }
 
