@@ -12,7 +12,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/db_verify.c,v 1.16.14.3 2007/08/11 23:41:37 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/db_verify.c,v 1.16.14.6 2008/10/27 23:53:46 shadow Exp $");
 
 /*
  *                      (3) Define a structure, idused, instead of an
@@ -44,13 +44,7 @@ RCSID
 #include <sys/file.h>
 #endif
 #include <stdio.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <errno.h>
 #include <fcntl.h>
 #include <afs/cellconfig.h>
@@ -401,8 +395,9 @@ WalkNextChain(char map[],		/* one byte per db entry */
     struct prentry c;		/* continuation entry */
     afs_int32 na;		/* next thread */
     int ni;
-    afs_int32 eid;
-    int count;			/* number of members */
+    afs_int32 eid = 0;
+    int count = 0;		/* number of members, set to > 9999 if */
+    				/* list ends early */
     int i;
     int noErrors = 1;
     int length;			/* length of chain */
@@ -416,7 +411,6 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	head = ntohl(e->next);
 	eid = ntohl(e->id);
 	bit = MAP_CONT;
-	count = 0;		/* set to >9999 if list ends early */
 #if defined(SUPERGROUPS)
 	sgcount = 0;
 	sghead = ntohl(g->next);
@@ -719,7 +713,7 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
     struct prentry c;		/* continuation entry */
     afs_int32 na;		/* next thread */
     int ni;
-    afs_int32 eid;
+    afs_int32 eid = 0;
     int length;			/* length of chain */
 
     if (e) {
@@ -1433,7 +1427,7 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
 #include "AFS_component_version_number.c"
 
 int
-WorkerBee(struct cmd_syndesc *as, char *arock)
+WorkerBee(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     char *recreateFile;
