@@ -1,4 +1,4 @@
-/* $Id: pt_util.c,v 1.11.4.1 2007/04/10 18:39:54 shadow Exp $ */
+/* $Id: pt_util.c,v 1.11.4.7 2008/10/27 23:53:46 shadow Exp $ */
 
 /*
  *
@@ -23,9 +23,10 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11.4.1 2007/04/10 18:39:54 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11.4.7 2008/10/27 23:53:46 shadow Exp $");
 
 #include <afs/cmd.h>		/*Command line parsing */
+#include <afs/afsutil.h>
 #include <errno.h>
 #include <lock.h>
 #include <netinet/in.h>
@@ -33,6 +34,7 @@ RCSID
 #include <ubik.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
+#include <afs/com_err.h>
 #include "ptint.h"
 #include "ptserver.h"
 #include "pterror.h"
@@ -53,7 +55,7 @@ void fix_pre();
 char *checkin();
 char *check_core();
 char *id_to_name();
-int CommandProc(struct cmd_syndesc *);
+int CommandProc(struct cmd_syndesc *, void *);
 
 struct hash_entry {
     char h_name[PR_MAXNAMELEN];
@@ -105,7 +107,7 @@ main(int argc, char **argv)
     register struct cmd_syndesc *cs;	/*Command line syntax descriptor */
     register afs_int32 code;	/*Return code */
 
-    cs = cmd_CreateSyntax((char *)0, CommandProc, 0,
+    cs = cmd_CreateSyntax(NULL, CommandProc, NULL,
 			  "access protection database");
     cmd_AddParm(cs, "-w", CMD_FLAG, CMD_OPTIONAL,
 		"update prdb with contents of data file");
@@ -128,15 +130,18 @@ main(int argc, char **argv)
 }
 
 int
-CommandProc(register struct cmd_syndesc *a_as)
+CommandProc(register struct cmd_syndesc *a_as, void *arock)
 {
     register int i;
-    register long code;
-    long cc, upos, gpos;
+    long code = 0;
+    long cc, upos;
+    long gpos = 0;
     struct prentry uentry, gentry;
     struct ubik_hdr *uh;
     char *dfile = 0;
-    char *pfile = "/usr/afs/db/prdb.DB0";
+    char *pbase = AFSDIR_SERVER_PRDB_FILEPATH;
+    char *pfile = NULL;
+    char pbuffer[1028];
     struct cmd_parmdesc *tparm;
 
     tparm = a_as->parms;
@@ -169,6 +174,10 @@ CommandProc(register struct cmd_syndesc *a_as)
 	dfile = tparm[8].items->data;
     }
 
+    if (pfile == NULL) {
+        afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0", pbase);
+        pfile = pbuffer;
+    }
     if ((dbase_fd = open(pfile, (wflag ? O_RDWR : O_RDONLY) | O_CREAT, 0600))
 	< 0) {
 	fprintf(stderr, "pt_util: cannot open %s: %s\n", pfile,
@@ -195,8 +204,9 @@ CommandProc(register struct cmd_syndesc *a_as)
 	fprintf(stderr, "pt_util: %s: Bad UBIK_MAGIC. Is %x should be %x\n",
 		pfile, ntohl(uh->magic), UBIK_MAGIC);
     memcpy(&uv, &uh->version, sizeof(struct ubik_version));
-    if (wflag && uv.epoch == 0 && uv.counter == 0) {
-	uv.epoch = 2;		/* a ubik version of 0 or 1 has special meaning */
+
+    if (wflag && ntohl(uv.epoch) == 0 && ntohl(uv.counter) == 0) {
+	uv.epoch = htonl(2); /* a ubik version of 0 or 1 has special meaning */
 	memcpy(&uh->version, &uv, sizeof(struct ubik_version));
 	lseek(dbase_fd, 0, SEEK_SET);
 	if (write(dbase_fd, buffer, HDRSIZE) < 0) {
@@ -205,6 +215,11 @@ CommandProc(register struct cmd_syndesc *a_as)
 	    exit(1);
 	}
     }
+
+    /* Now that any writeback is done, swap these */
+    uv.epoch = ntohl(uv.epoch);
+    uv.counter = ntohl(uv.counter);
+
     fprintf(stderr, "Ubik Version is: %d.%d\n", uv.epoch, uv.counter);
     if (read(dbase_fd, &prh, sizeof(struct prheader)) < 0) {
 	fprintf(stderr, "pt_util: error reading %s: %s\n", pfile,
@@ -314,8 +329,16 @@ CommandProc(register struct cmd_syndesc *a_as)
     } else {
 	for (i = 0; i < HASHSIZE; i++) {
 	    upos = nflag ? ntohl(prh.nameHash[i]) : ntohl(prh.idHash[i]);
-	    while (upos)
-		upos = display_entry(upos);
+	    while (upos) {
+		long newpos;
+		newpos = display_entry(upos);
+		if (newpos == upos) {
+		    fprintf(stderr, "pt_util: hash error in %s chain %d\n", 
+			    nflag ? "name":"id", i);
+		    exit(1);
+		} else
+		    upos = newpos;
+	    }
 	}
 	if (flags & DO_GRP)
 	    display_groups();
@@ -328,6 +351,10 @@ CommandProc(register struct cmd_syndesc *a_as)
 	exit(1);
     }
     uh = (struct ubik_hdr *)buffer;
+
+    uh->version.epoch = ntohl(uh->version.epoch);
+    uh->version.counter = ntohl(uh->version.counter);
+
     if ((uh->version.epoch != uv.epoch)
 	|| (uh->version.counter != uv.counter)) {
 	fprintf(stderr,
