@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_getaddr.c,v 1.23 2005/09/21 00:33:42 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_getaddr.c,v 1.23.4.6 2008/09/24 21:36:53 shadow Exp $");
 
 #ifndef AFS_DJGPP_ENV
 #ifndef KERNEL
@@ -22,6 +22,7 @@ RCSID
 #include <net/if.h>
 #include <netinet/in.h>
 #include <sys/ioctl.h>
+#include <string.h>
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 #include <sys/sysctl.h>
 #include <net/route.h>
@@ -48,17 +49,17 @@ RCSID
 #ifdef KERNEL
 /* only used for generating random noise */
 
-afs_int32 rxi_tempAddr = 0;	/* default attempt */
+afs_uint32 rxi_tempAddr = 0;	/* default attempt */
 
 /* set the advisory noise */
 void
-rxi_setaddr(afs_int32 x)
+rxi_setaddr(afs_uint32 x)
 {
     rxi_tempAddr = x;
 }
 
 /* get approx to net addr */
-afs_int32
+afs_uint32
 rxi_getaddr(void)
 {
     return rxi_tempAddr;
@@ -70,7 +71,7 @@ rxi_getaddr(void)
 
 /* to satisfy those who call setaddr */
 void
-rxi_setaddr(afs_int32 x)
+rxi_setaddr(afs_uint32 x)
 {
 }
 
@@ -87,10 +88,10 @@ rxi_setaddr(afs_int32 x)
 /* Return our internet address as a long in network byte order.  Returns zero
  * if it can't find one.
  */
-afs_int32
+afs_uint32
 rxi_getaddr(void)
 {
-    afs_int32 buffer[1024];
+    afs_uint32 buffer[1024];
     int count;
 
     count = rx_getAllAddr(buffer, 1024);
@@ -140,7 +141,7 @@ rt_xaddrs(caddr_t cp, caddr_t cplim, struct rt_addrinfo *rtinfo)
 */
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 int
-rx_getAllAddr(afs_int32 buffer[], int maxSize)
+rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 {
     size_t needed;
     int mib[6];
@@ -189,9 +190,6 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
 	}
 	if ((ifm->ifm_flags & IFF_UP) == 0)
 	    continue;		/* not up */
-	if (ifm->ifm_flags & IFF_LOOPBACK) {
-	    continue;		/* skip aliased loopbacks as well. */
-	}
 	while (addrcount > 0) {
 	    struct sockaddr_in *a;
 
@@ -207,7 +205,13 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
 	    if (count >= maxSize)	/* no more space */
 		dpf(("Too many interfaces..ignoring 0x%x\n",
 		       a->sin_addr.s_addr));
-	    else
+	    else if (!loopbacks && a->sin_addr.s_addr == htonl(0x7f000001)) {
+		addrcount--;
+		continue;	/* skip loopback address as well. */
+	    } else if (loopbacks && ifm->ifm_flags & IFF_LOOPBACK) {
+		addrcount--;
+		continue;	/* skip aliased loopbacks as well. */
+	    } else
 		buffer[count++] = a->sin_addr.s_addr;
 	    addrcount--;
 	    ifam = (struct ifa_msghdr *)((char *)ifam + ifam->ifam_msglen);
@@ -218,8 +222,8 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
 }
 
 int
-rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
-		      afs_int32 mtuBuffer[], int maxSize)
+rxi_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
+		      afs_uint32 mtuBuffer[], int maxSize)
 {
     int s;
 
@@ -298,7 +302,7 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
 			maskBuffer[count] = a->sin_addr.s_addr;
 		    else
 			maskBuffer[count] = htonl(0xffffffff);
-		    memset(&ifr, sizeof(ifr), 0);
+		    memset(&ifr, 0, sizeof(ifr));
 		    ifr.ifr_addr.sa_family = AF_INET;
 		    strncpy(ifr.ifr_name, sdl->sdl_data, sdl->sdl_nlen);
 		    if (ioctl(s, SIOCGIFMTU, (caddr_t) & ifr) < 0)
@@ -315,9 +319,19 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
     free(buf);
     return count;
 }
+
+
+int
+rx_getAllAddr(afs_uint32 buffer[], int maxSize)
+{
+    return rx_getAllAddr_internal(buffer, maxSize, 0);
+}
+/* this function returns the total number of interface addresses
+** the buffer has to be passed in by the caller
+*/
 #else
 static int
-rx_getAllAddr_internal(afs_int32 buffer[], int maxSize, int loopbacks)
+rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 {
     int s;
     int i, len, count = 0;
@@ -393,7 +407,7 @@ rx_getAllAddr_internal(afs_int32 buffer[], int maxSize, int loopbacks)
 }
 
 int
-rx_getAllAddr(afs_int32 buffer[], int maxSize)
+rx_getAllAddr(afs_uint32 buffer[], int maxSize)
 {
     return rx_getAllAddr_internal(buffer, maxSize, 0);
 }
@@ -405,8 +419,8 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
  * by afsi_SetServerIPRank().
  */
 int
-rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
-		      afs_int32 mtuBuffer[], int maxSize)
+rxi_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
+		      afs_uint32 mtuBuffer[], int maxSize)
 {
     int s;
     int i, len, count = 0;
@@ -418,7 +432,7 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
 #endif
 
 #if !defined(AFS_USERSPACE_IP_ADDR)
-    count = rx_getAllAddr_internal(addrBuffer, 1024, 1);
+    count = rx_getAllAddr_internal(addrBuffer, 1024, 0);
     for (i = 0; i < count; i++) {
 	maskBuffer[i] = htonl(0xffffffff);
 	mtuBuffer[i] = htonl(1500);
