@@ -37,10 +37,19 @@ extern int osi_utoa(char *buf, size_t len, unsigned long val);
 #define osi_Assert(e) (void)((e) || (osi_AssertFailK(#e, __FILE__, __LINE__), 0))
 
 #define	osi_Msg printf)(
+#define osi_VMsg vprintf)(
 
 #define	osi_YieldIfPossible()
 #define	osi_WakeupAndYieldIfPossible(x)	    rx_Wakeup(x)
 
+#ifndef AFS_DARWIN80_ENV
+#define ifnet_mtu(x) (x)->if_mtu
+#define ifnet_flags(x) (x?(x)->if_flags:0)
+#define AFS_IFNET_T struct ifnet *
+#else
+#define AFS_IFNET_T ifnet_t
+#endif
+
 #include "afs/longc_procs.h"
 
 #endif /* __RX_KERNEL_INCL_ */
