@@ -22,7 +22,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_lwp.c,v 1.19 2005/11/05 06:48:18 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_lwp.c,v 1.19.4.4 2008/09/24 21:36:53 shadow Exp $");
 
 # include <sys/types.h>		/* fd_set on older platforms */
 # include <errno.h>
@@ -50,9 +50,6 @@ RCSID
 
 #define MAXTHREADNAMELENGTH 64
 
-extern int (*registerProgram) ();
-extern int (*swapNameProgram) ();
-
 int debugSelectFailure;		/* # of times select failed */
 
 /*
@@ -75,7 +72,7 @@ rxi_Wakeup(void *addr)
 }
 
 PROCESS rx_listenerPid = 0;	/* LWP process id of socket listener process */
-static int rx_ListenerProc(void *dummy);
+static void* rx_ListenerProc(void *dummy);
 
 /*
  * Delay the current thread the specified number of seconds.
@@ -119,14 +116,14 @@ rxi_InitializeThreadSupport(void)
 }
 
 void
-rxi_StartServerProc(void (*proc) (void), int stacksize)
+rxi_StartServerProc(void *(*proc) (void *), int stacksize)
 {
     PROCESS scratchPid;
     static int number = 0;
     char name[32];
 
     sprintf(name, "srv_%d", ++number);
-    LWP_CreateProcess((int (*)(void *))proc, stacksize, RX_PROCESS_PRIORITY, (void *)0,
+    LWP_CreateProcess(proc, stacksize, RX_PROCESS_PRIORITY, NULL,
 		      "rx_ServerProc", &scratchPid);
     if (registerProgram)
 	(*registerProgram) (scratchPid, name);
@@ -138,7 +135,7 @@ rxi_StartListener(void)
     /* Priority of listener should be high, so it can keep conns alive */
 #define	RX_LIST_STACK	24000
     LWP_CreateProcess(rx_ListenerProc, RX_LIST_STACK, LWP_MAX_PRIORITY,
-		      (void *)0, "rx_Listener", &rx_listenerPid);
+		      NULL, "rx_Listener", &rx_listenerPid);
     if (registerProgram)
 	(*registerProgram) (rx_listenerPid, "listener");
 }
@@ -283,7 +280,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 			    rxi_FreePacket(p);
 			}
 			if (swapNameProgram) {
-			    (*swapNameProgram) (rx_listenerPid, &name, 0);
+			    (*swapNameProgram) (rx_listenerPid, name, 0);
 			    rx_listenerPid = 0;
 			}
 			return;
@@ -303,7 +300,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 			    rxi_FreePacket(p);
 			}
 			if (swapNameProgram) {
-			    (*swapNameProgram) (rx_listenerPid, &name, 0);
+			    (*swapNameProgram) (rx_listenerPid, name, 0);
 			    rx_listenerPid = 0;
 			}
 			return;
@@ -320,7 +317,7 @@ rxi_ListenerProc(fd_set * rfds, int *tnop, struct rx_call **newcallp)
 /* This is the listener process request loop. The listener process loop
  * becomes a server thread when rxi_ListenerProc returns, and stays
  * server thread until rxi_ServerProc returns. */
-static int
+static void *
 rx_ListenerProc(void *dummy)
 {
     int threadID;
@@ -343,13 +340,14 @@ rx_ListenerProc(void *dummy)
 	/* assert(sock != OSI_NULLSOCKET); */
     }
     /* not reached */
+    return NULL;
 }
 
 /* This is the server process request loop. The server process loop
  * becomes a listener thread when rxi_ServerProc returns, and stays
  * listener thread until rxi_ListenerProc returns. */
-void
-rx_ServerProc(void)
+void *
+rx_ServerProc(void * unused)
 {
     osi_socket sock;
     int threadID;
@@ -376,6 +374,7 @@ rx_ServerProc(void)
 	/* assert(newcall != NULL); */
     }
     /* not reached */
+    return NULL;
 }
 
 /*
@@ -426,6 +425,10 @@ rxi_Listen(osi_socket sock)
 int
 rxi_Recvmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 {
+#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(ADAPT_PMTU)
+    while((rxi_HandleSocketError(socket)) > 0)
+	;
+#endif
     return recvmsg(socket, msg_p, flags);
 }
 
@@ -445,12 +448,16 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 	    if (!(sfds = IOMGR_AllocFDSet())) {
 		(osi_Msg "rx failed to alloc fd_set: ");
 		perror("rx_sendmsg");
-		return 3;
+		return -1;
 	    }
 	    FD_SET(socket, sfds);
 	}
+#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(ADAPT_PMTU)
+	while((rxi_HandleSocketError(socket)) > 0)
+	  ;
+#endif
 #ifdef AFS_NT40_ENV
-	if (errno)
+	if (WSAGetLastError())
 #elif defined(AFS_LINUX22_ENV)
 	/* linux unfortunately returns ECONNREFUSED if the target port
 	 * is no longer in use */
@@ -463,7 +470,7 @@ rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 	{
 	    (osi_Msg "rx failed to send packet: ");
 	    perror("rx_sendmsg");
-	    return 3;
+	    return -1;
 	}
 	while ((err = select(socket + 1, 0, sfds, 0, 0)) != 1) {
 	    if (err >= 0 || errno != EINTR)
