@@ -36,18 +36,12 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxgen/rpc_parse.c,v 1.17 2003/07/15 23:16:41 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rxgen/rpc_parse.c,v 1.20.4.5 2008/10/27 23:41:46 shadow Exp $");
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include "rpc_scan.h"
 #include "rpc_parse.h"
 #include "rpc_util.h"
@@ -134,6 +128,10 @@ static void cs_ProcMarshallInParams_setup(definition * defp, int split_flag);
 static void cs_ProcSendPacket_setup(definition * defp, int split_flag);
 static void cs_ProcUnmarshallOutParams_setup(definition * defp);
 static void cs_ProcTail_setup(definition * defp, int split_flag);
+static void ucs_ProcName_setup(definition * defp, char *procheader,
+			      int split_flag);
+static void ucs_ProcParams_setup(definition * defp, int split_flag);
+static void ucs_ProcTail_setup(definition * defp, int split_flag);
 static void ss_Proc_CodeGeneration(definition * defp);
 static void ss_ProcName_setup(definition * defp);
 static void ss_ProcParams_setup(definition * defp, int *somefrees);
@@ -775,7 +773,10 @@ check_proc(definition * defp, token * tokp, int noname)
     scan4(TOK_SPLIT, TOK_MULTI, TOK_EQUAL, TOK_SEMICOLON, &tok);
     if (tok.kind == TOK_MULTI) {
 	proc_multi = 1;
+	defp->pc.multi_flag = 1;
 	scan2(TOK_EQUAL, TOK_SEMICOLON, &tok);
+    } else {
+	defp->pc.multi_flag = 0;
     }
     if (tok.kind == TOK_SPLIT) {
 	proc_split = 1;
@@ -848,7 +849,7 @@ InvalidConstant(char *name)
     int slen;
 
     map = LEGALNUMS;
-    slen = strlen(name);
+    slen = (int)strlen(name);
     return (slen != strspn(name, map));
 }
 
@@ -902,7 +903,8 @@ analyze_ProcParams(definition * defp, token * tokp)
 	Proc_listp = &Proc_list->next;
 	decls = ALLOC(decl_list);
 	memset((char *)decls, 0, sizeof(decl_list));
-	decls->decl = dec;
+    if (tokp->kind != TOK_RPAREN)
+        decls->decl = dec;
 	*tailp = decls;
 	tailp = &decls->next;
     } while (tokp->kind != TOK_RPAREN);
@@ -931,7 +933,7 @@ static void
 handle_split_proc(definition * defp, int multi_flag)
 {
     char *startname = SplitStart, *endname = SplitEnd;
-    int numofparams;
+    int numofparams = 0;
 
     if (!startname)
 	startname = "Start";
@@ -1098,22 +1100,39 @@ cs_Proc_CodeGeneration(definition * defp, int split_flag, char *procheader)
 	}
 	cs_ProcTail_setup(defp, split_flag);
     }
-}
 
+    if (!kflag && !split_flag && uflag) {
+	ucs_ProcName_setup(defp, "ubik_", split_flag);
+	if (!cflag) {
+	    ucs_ProcParams_setup(defp, split_flag);
+	    ucs_ProcTail_setup(defp, split_flag);
+	}
+    }
+}
 
 static void
 cs_ProcName_setup(definition * defp, char *procheader, int split_flag)
 {
     proc1_list *plist;
+    char *first_arg;
+
+    if (ansic_flag) {
+	if (split_flag) {
+	    first_arg = "register struct rx_call *z_call";
+	} else {
+	    first_arg = "register struct rx_connection *z_conn";
+	}
+    } else {
+	if (split_flag) {
+	    first_arg = "z_call";
+	} else {
+	    first_arg = "z_conn";
+	}
+    }
 
     if (!cflag) {
-	if (split_flag) {
-	    f_print(fout, "int %s%s%s%s(z_call", procheader, prefix,
-		    PackagePrefix[PackageIndex], defp->pc.proc_name);
-	} else {
-	    f_print(fout, "int %s%s%s%s(z_conn", procheader, prefix,
-		    PackagePrefix[PackageIndex], defp->pc.proc_name);
-	}
+	f_print(fout, "int %s%s%s%s(%s", procheader, prefix,
+		PackagePrefix[PackageIndex], defp->pc.proc_name, first_arg);
     }
     if ((strlen(procheader) + strlen(prefix) +
 	 strlen(PackagePrefix[PackageIndex]) + strlen(defp->pc.proc_name)) >=
@@ -1123,20 +1142,32 @@ cs_ProcName_setup(definition * defp, char *procheader, int split_flag)
     if (!cflag) {
 	for (plist = defp->pc.plists; plist; plist = plist->next) {
 	    if (plist->component_kind == DEF_PARAM) {
-		plist->pl.param_flag &= ~PROCESSED_PARAM;
-		f_print(fout, ", %s", plist->pl.param_name);
+		if (ansic_flag) {
+		    if (plist->pl.param_flag & OUT_STRING) {
+			f_print(fout, ",%s *%s", plist->pl.param_type,
+				plist->pl.param_name);
+		    } else {
+			f_print(fout, ",%s %s", plist->pl.param_type,
+				plist->pl.param_name);
+		    }
+		} else {
+		    f_print(fout, ", %s", plist->pl.param_name);    
+		    plist->pl.param_flag &= ~PROCESSED_PARAM;
+		}
 	    }
 	}
 	f_print(fout, ")\n");
     }
 }
 
-
 static void
 cs_ProcParams_setup(definition * defp, int split_flag)
 {
     proc1_list *plist, *plist1;
 
+    if (ansic_flag)
+	return;
+
     if (!split_flag)
 	f_print(fout, "\tregister struct rx_connection *z_conn;\n");
     if (split_flag) {
@@ -1726,6 +1757,223 @@ ss_ProcTail_setup(definition * defp, int somefrees)
 }
 
 
+static void
+ucs_ProcName_setup(definition * defp, char *procheader, int split_flag)
+{
+    proc1_list *plist;
+
+    if (!cflag) {
+     	if (ansic_flag) {
+	    f_print(fout, "int %s%s%s%s(register struct ubik_client *aclient, afs_int32 aflags",
+			  procheader, prefix, PackagePrefix[PackageIndex],
+			  defp->pc.proc_name);
+	} else {
+	    f_print(fout, "int %s%s%s%s(aclient, aflags", procheader, prefix,
+			  PackagePrefix[PackageIndex], defp->pc.proc_name);
+	}
+    }
+    if ((strlen(procheader) + strlen(prefix) +
+	 strlen(PackagePrefix[PackageIndex]) + strlen(defp->pc.proc_name)) >=
+	MAX_FUNCTION_NAME_LEN) {
+	error("function name is too long, increase MAX_FUNCTION_NAME_LEN");
+    }
+    if (!cflag) {
+	for (plist = defp->pc.plists; plist; plist = plist->next) {
+	    if (plist->component_kind == DEF_PARAM) {
+		if (ansic_flag) {
+		    if (plist->pl.param_flag & OUT_STRING) {
+			f_print(fout, ",%s *%s", plist->pl.param_type,
+				plist->pl.param_name);
+		    } else {
+			f_print(fout, ",%s %s", plist->pl.param_type,
+				plist->pl.param_name);
+		    }
+		} else {
+		    plist->pl.param_flag &= ~PROCESSED_PARAM;
+		    f_print(fout, ", %s", plist->pl.param_name);
+		}
+	    }
+	}
+	f_print(fout, ")\n");
+    }
+}
+
+
+static void
+ucs_ProcParams_setup(definition * defp, int split_flag)
+{
+    proc1_list *plist, *plist1;
+
+    if (ansic_flag)
+	return;
+
+    f_print(fout, "\tregister struct ubik_client *aclient;\n\tafs_int32 aflags;\n");
+    for (plist = defp->pc.plists; plist; plist = plist->next) {
+	if (plist->component_kind == DEF_PARAM
+	    && !(plist->pl.param_flag & PROCESSED_PARAM)) {
+	    if (plist->pl.param_flag & OUT_STRING) {
+		f_print(fout, "\t%s *%s", plist->pl.param_type,
+			plist->pl.param_name);
+	    } else {
+		f_print(fout, "\t%s %s", plist->pl.param_type,
+			plist->pl.param_name);
+	    }
+	    plist->pl.param_flag |= PROCESSED_PARAM;
+	    for (plist1 = defp->pc.plists; plist1; plist1 = plist1->next) {
+		if ((plist1->component_kind == DEF_PARAM)
+		    && streq(plist->pl.param_type, plist1->pl.param_type)
+		    && !(plist1->pl.param_flag & PROCESSED_PARAM)) {
+		    char *star = "";
+		    char *pntr = strchr(plist1->pl.param_type, '*');
+		    if (pntr)
+			star = "*";
+		    if (plist1->pl.param_flag & OUT_STRING) {
+			f_print(fout, ", *%s%s", star, plist1->pl.param_name);
+		    } else {
+			f_print(fout, ", %s%s", star, plist1->pl.param_name);
+		    }
+		    plist1->pl.param_flag |= PROCESSED_PARAM;
+		}
+	    }
+	    f_print(fout, ";\n");
+	}
+    }
+}
+
+static void
+ucs_ProcTail_setup(definition * defp, int split_flag)
+{
+    proc1_list *plist;
+
+    f_print(fout, "{\tafs_int32 rcode, code, newHost, thisHost, i, _ucount;\n");
+    f_print(fout, "\tint chaseCount, pass, needsync, inlist;\n");
+#if 0 /* goes with block below */
+    f_print(fout, "\tint j;\n");
+#endif
+    f_print(fout, "\tstruct rx_connection *tc;\n");
+    f_print(fout, "\tstruct rx_peer *rxp;\n");
+    f_print(fout, "\tshort origLevel;\n\n");
+    f_print(fout, "\tif (!aclient)\n");
+    f_print(fout, "\t\treturn UNOENT;\n");
+    f_print(fout, "\tLOCK_UBIK_CLIENT(aclient);\n\n");
+    f_print(fout, "\t restart:\n");
+    f_print(fout, "\torigLevel = aclient->initializationState;\n");
+    f_print(fout, "\trcode = UNOSERVERS;\n");
+    f_print(fout, "\tchaseCount = inlist = needsync = 0;\n\n");
+#if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
+    f_print(fout, "\tLOCK_UCLNT_CACHE;\n");
+    f_print(fout, "\tfor (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {\n");
+    f_print(fout, "\t\tif (calls_needsync[j] == (int *)%s%s%s) {\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
+    f_print(fout, "\t\t\tinlist = needsync = 1;\n");
+    f_print(fout, "\t\t\tbreak;\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t}\n");
+    f_print(fout, "\tUNLOCK_UCLNT_CACHE;\n");
+#endif
+    f_print(fout, "\t/* \n\t* First  pass, we try all servers that are up.\n\t* Second pass, we try all servers.\n\t*/\n");
+    f_print(fout, "\tfor (pass = 0; pass < 2; pass++) {  /*p */\n");
+    f_print(fout, "\t\t/* For each entry in our servers list */\n");
+    f_print(fout, "\t\tfor (_ucount = 0;; _ucount++) {     /*s */\n\n");
+    f_print(fout, "\t\tif (needsync) {\n");
+    f_print(fout, "\t\t\t/* Need a sync site. Lets try to quickly find it */\n");
+    f_print(fout, "\t\t\tif (aclient->syncSite) {\n");
+    f_print(fout, "\t\t\t\tnewHost = aclient->syncSite;        /* already in network order */\n");
+    f_print(fout, "\t\t\t\taclient->syncSite = 0;      /* Will reset if it works */\n");
+    f_print(fout, "\t\t\t} else if (aclient->conns[3]) {\n");
+    f_print(fout, "\t\t\t\t/* If there are fewer than four db servers in a cell,\n");
+    f_print(fout, "\t\t\t\t* there's no point in making the GetSyncSite call.\n");
+    f_print(fout, "\t\t\t\t* At best, it's a wash. At worst, it results in more\n");
+    f_print(fout, "\t\t\t\t* RPCs than you would otherwise make.\n");
+    f_print(fout, "\t\t\t\t*/\n");
+    f_print(fout, "\t\t\t\ttc = aclient->conns[_ucount];\n");
+    f_print(fout, "\t\t\t\tif (tc && rx_ConnError(tc)) {\n");
+    f_print(fout, "\t\t\t\t\taclient->conns[_ucount] = tc = ubik_RefreshConn(tc);\n");
+    f_print(fout, "\t\t\t\t}\n");
+    f_print(fout, "\t\t\t\tif (!tc)\n");
+    f_print(fout, "\t\t\t\t\tbreak;\n");
+    f_print(fout, "\t\t\t\tcode = VOTE_GetSyncSite(tc, &newHost);\n");
+    f_print(fout, "\t\t\t\tif (aclient->initializationState != origLevel)\n");
+    f_print(fout, "\t\t\t\t\tgoto restart;   /* somebody did a ubik_ClientInit */\n");
+    f_print(fout, "\t\t\t\tif (code)\n");
+    f_print(fout, "\t\t\t\t\tnewHost = 0;\n");
+    f_print(fout, "\t\t\t\tnewHost = htonl(newHost);   /* convert to network order */\n");
+    f_print(fout, "\t\t\t} else {\n");
+    f_print(fout, "\t\t\t\tnewHost = 0;\n");
+    f_print(fout, "\t\t\t}\n");
+    f_print(fout, "\t\t\tif (newHost) {\n");
+    f_print(fout, "\t\t\t\t/* position count at the appropriate slot in the client\n");
+    f_print(fout, "\t\t\t\t* structure and retry. If we can't find in slot, we'll\n");
+    f_print(fout, "\t\t\t\t* just continue through the whole list \n");
+    f_print(fout, "\t\t\t\t*/\n");
+    f_print(fout, "\t\t\t\tfor (i = 0; i < MAXSERVERS && aclient->conns[i]; i++) {\n");
+    f_print(fout, "\t\t\t\t\trxp = rx_PeerOf(aclient->conns[i]);\n");
+    f_print(fout, "\t\t\t\t\tthisHost = rx_HostOf(rxp);\n");
+    f_print(fout, "\t\t\t\t\tif (!thisHost)\n");
+    f_print(fout, "\t\t\t\t\t\tbreak;\n");
+    f_print(fout, "\t\t\t\t\tif (thisHost == newHost) {\n");
+    f_print(fout, "\t\t\t\t\t\tif (chaseCount++ > 2)\n");
+    f_print(fout, "\t\t\t\t\t\t\tbreak;  /* avoid loop asking */\n");
+    f_print(fout, "\t\t\t\t\t\t_ucount = i;  /* this index is the sync site */\n");
+    f_print(fout, "\t\t\t\t\t\tbreak;\n");
+    f_print(fout, "\t\t\t\t\t}\n\t\t\t\t}\n\t\t\t}\n\t\t}\n");
+    f_print(fout, "\t\t/*needsync */\n");
+    f_print(fout, "\t\ttc = aclient->conns[_ucount];\n");
+    f_print(fout, "\t\tif (tc && rx_ConnError(tc)) {\n");
+    f_print(fout, "\t\t\taclient->conns[_ucount] = tc = ubik_RefreshConn(tc);\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t\tif (!tc)\n");
+    f_print(fout, "\t\t\tbreak;\n\n");
+    f_print(fout, "\t\tif ((pass == 0) && (aclient->states[_ucount] & CFLastFailed)) {\n");
+    f_print(fout, "\t\t\tcontinue;       /* this guy's down */\n");
+    f_print(fout, "\t\t}\n");
+    
+    f_print(fout, "\t\trcode = %s%s%s(tc\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
+    for (plist = defp->pc.plists; plist; plist = plist->next) {
+	if (plist->component_kind == DEF_PARAM) {
+	    plist->pl.param_flag &= ~PROCESSED_PARAM;
+	    f_print(fout, ", %s", plist->pl.param_name);
+	}
+    }
+    f_print(fout, ");\n");
+    f_print(fout, "\t\tif (aclient->initializationState != origLevel) {\n");
+    f_print(fout, "\t\t\t/* somebody did a ubik_ClientInit */\n");
+    f_print(fout, "\t\t\tif (rcode)\n");
+    f_print(fout, "\t\t\t\tgoto restart;       /* call failed */\n");
+    f_print(fout, "\t\t\telse\n");
+    f_print(fout, "\t\t\t\tgoto done;  /* call suceeded */\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t\tif (rcode < 0) {    /* network errors */\n");
+    f_print(fout, "\t\t\taclient->states[_ucount] |= CFLastFailed; /* Mark server down */\n");
+    f_print(fout, "\t\t} else if (rcode == UNOTSYNC) {\n");
+    f_print(fout, "\t\t\tneedsync = 1;\n");
+    f_print(fout, "\t\t} else if (rcode != UNOQUORUM) {\n");
+    f_print(fout, "\t\t\t/* either misc ubik code, or misc appl code, or success. */\n");
+    f_print(fout, "\t\t\taclient->states[_ucount] &= ~CFLastFailed;        /* mark server up*/\n");
+    f_print(fout, "\t\t\tgoto done;      /* all done */\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t\t}                       /*s */\n");
+    f_print(fout, "\t}                           /*p */\n\n");
+    f_print(fout, "\tdone:\n");
+    f_print(fout, "\tif (needsync) {\n");
+    f_print(fout, "\t\tif (!inlist) {          /* Remember proc call that needs sync site */\n");
+#if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
+    f_print(fout, "\t\t\tLOCK_UCLNT_CACHE;\n");
+    f_print(fout, "\t\t\tcalls_needsync[synccount % SYNCCOUNT] = (int *)%s%s%s;\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
+    f_print(fout, "\t\t\tsynccount++;\n");
+    f_print(fout, "\t\t\tUNLOCK_UCLNT_CACHE;\n");
+#endif
+    f_print(fout, "\t\t\tinlist = 1;\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t\tif (!rcode) {           /* Remember the sync site - cmd successful */\n");
+    f_print(fout, "\t\t\trxp = rx_PeerOf(aclient->conns[_ucount]);\n");
+    f_print(fout, "\t\t\taclient->syncSite = rx_HostOf(rxp);\n");
+    f_print(fout, "\t\t}\n");
+    f_print(fout, "\t}\n");
+    f_print(fout, "\tUNLOCK_UBIK_CLIENT(aclient);\n");
+    f_print(fout, "\treturn rcode;\n}\n\n");
+}
+
+
 static int
 opcode_holes_exist(void)
 {
