@@ -55,21 +55,14 @@
 #else
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-#include <rx/xdr.h>
 #include <rx/rx.h>
 #include <des.h>
 #include "lifetimes.h"
 #include "rxkad.h"
 #endif /* defined(UKERNEL) */
 
-RCSID("$Id: md5.c,v 1.2.8.1 2006/08/02 19:01:22 shadow Exp $");
+RCSID("$Id: md5.c,v 1.2.8.2 2007/10/30 15:16:46 shadow Exp $");
 
 
 
