@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/beacon.c,v 1.21 2004/08/19 01:07:51 kolya Exp $");
+    ("$Header: /cvs/openafs/src/ubik/beacon.c,v 1.21.8.6 2008/07/19 06:15:02 rra Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -26,13 +26,7 @@ RCSID
 #endif
 #include <errno.h>
 #include <lock.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_multi.h>
@@ -83,6 +77,7 @@ static verifyInterfaceAddress();
  */
 
 /* procedure called from debug rpc call to get this module's state for debugging */
+void
 ubeacon_Debug(aparm)
      register struct ubik_debug *aparm;
 {
@@ -310,7 +305,8 @@ ubeacon_InitServerListCommon(ame, info, clones, aservers)
  * we're sync site or we want to be the sync site.  It runs in its very own light-weight
  * process.
  */
-ubeacon_Interact()
+void *
+ubeacon_Interact(void *dummy)
 {
     register afs_int32 code;
     struct timeval tt;
@@ -334,7 +330,11 @@ ubeacon_Interact()
 		temp = POLLTIME;
 	    tt.tv_sec = temp;
 	    tt.tv_usec = 0;
+#ifdef AFS_PTHREAD_ENV
+	    code = select(0, 0, 0, 0, &tt);
+#else
 	    code = IOMGR_Select(0, 0, 0, 0, &tt);
+#endif
 	} else
 	    code = 0;
 
@@ -450,7 +450,11 @@ ubeacon_Interact()
 		ubik_dprint("Ubik: I am the sync site\n");
 	    ubik_amSyncSite = 1;
 	    syncSiteUntil = oldestYesVote + SMALLTIME;
-	    LWP_NoYieldSignal(&ubik_amSyncSite);
+#ifndef AFS_PTHREAD_ENV
+		/* I did not find a corresponding LWP_WaitProcess(&ubik_amSyncSite) --
+		   this may be a spurious signal call -- sjenkins */
+		LWP_NoYieldSignal(&ubik_amSyncSite);
+#endif
 	} else {
 	    if (ubik_amSyncSite)
 		ubik_dprint("Ubik: I am no longer the sync site\n");
@@ -459,6 +463,7 @@ ubeacon_Interact()
 	}
 
     }				/* while loop */
+    return NULL;
 }
 
 /* 
