@@ -11,12 +11,13 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/recovery.c,v 1.11 2003/07/15 23:17:05 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/recovery.c,v 1.14.4.9 2008/10/18 15:24:45 jaltman Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <time.h>
+#include <fcntl.h>
 #else
 #include <sys/file.h>
 #include <netinet/in.h>
@@ -24,13 +25,7 @@ RCSID
 #endif
 #include <assert.h>
 #include <lock.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <errno.h>
@@ -77,7 +72,10 @@ int
 urecovery_ResetState(void)
 {
     urecovery_state = 0;
+#if !defined(AFS_PTHREAD_ENV)
+    /*  No corresponding LWP_WaitProcess found anywhere for this -- klm */
     LWP_NoYieldSignal(&urecovery_state);
+#endif
     return 0;
 }
 
@@ -88,8 +86,11 @@ urecovery_ResetState(void)
 int
 urecovery_LostServer(void)
 {
+#if !defined(AFS_PTHREAD_ENV)
+    /*  No corresponding LWP_WaitProcess found anywhere for this -- klm */
     LWP_NoYieldSignal(&urecovery_state);
     return 0;
+#endif
 }
 
 /* return true iff we have a current database (called by both sync
@@ -164,6 +165,7 @@ urecovery_CheckTid(register struct ubik_tid *atid)
 	    ubik_currentTrans = (struct ubik_trans *)0;
 	}
     }
+    return 0;
 }
 
 /* log format is defined here, and implicitly in disk.c
@@ -209,7 +211,7 @@ ReplayLog(register struct ubik_dbase *adbase)
     /* for now, assume that all ops in log pertain to one transaction; see if there's a commit */
     while (1) {
 	code =
-	    (*adbase->read) (adbase, LOGFILE, &opcode, tpos,
+	    (*adbase->read) (adbase, LOGFILE, (char *)&opcode, tpos,
 			     sizeof(afs_int32));
 	if (code != sizeof(afs_int32))
 	    break;
@@ -224,7 +226,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	} else if (opcode == LOGTRUNCATE) {
 	    tpos += 4;
 	    code =
-		(*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				 2 * sizeof(afs_int32));
 	    if (code != 2 * sizeof(afs_int32))
 		break;		/* premature eof or io error */
@@ -232,7 +234,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	} else if (opcode == LOGDATA) {
 	    tpos += 4;
 	    code =
-		(*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				 3 * sizeof(afs_int32));
 	    if (code != 3 * sizeof(afs_int32))
 		break;
@@ -252,7 +254,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	syncFile = -1;
 	while (1) {
 	    code =
-		(*adbase->read) (adbase, LOGFILE, &opcode, tpos,
+		(*adbase->read) (adbase, LOGFILE, (char *)&opcode, tpos,
 				 sizeof(afs_int32));
 	    if (code != sizeof(afs_int32))
 		break;
@@ -264,11 +266,11 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    else if (opcode == LOGEND) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     2 * sizeof(afs_int32));
 		if (code != 2 * sizeof(afs_int32))
 		    return UBADLOG;
-		code = (*adbase->setlabel) (adbase, 0, buffer);
+		code = (*adbase->setlabel) (adbase, 0, (ubik_version *)buffer);
 		if (code)
 		    return code;
 		logIsGood = 1;
@@ -276,7 +278,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    } else if (opcode == LOGTRUNCATE) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     2 * sizeof(afs_int32));
 		if (code != 2 * sizeof(afs_int32))
 		    break;	/* premature eof or io error */
@@ -289,7 +291,7 @@ ReplayLog(register struct ubik_dbase *adbase)
 	    } else if (opcode == LOGDATA) {
 		tpos += 4;
 		code =
-		    (*adbase->read) (adbase, LOGFILE, buffer, tpos,
+		    (*adbase->read) (adbase, LOGFILE, (char *)buffer, tpos,
 				     3 * sizeof(afs_int32));
 		if (code != 3 * sizeof(afs_int32))
 		    break;
@@ -312,12 +314,12 @@ ReplayLog(register struct ubik_dbase *adbase)
 		    thisSize = (len > sizeof(data) ? sizeof(data) : len);
 		    /* copy sizeof(data) buffer bytes at a time */
 		    code =
-			(*adbase->read) (adbase, LOGFILE, data, tpos,
+			(*adbase->read) (adbase, LOGFILE, (char *)data, tpos,
 					 thisSize);
 		    if (code != thisSize)
 			return UBADLOG;
 		    code =
-			(*adbase->write) (adbase, tfile, data, filePos,
+			(*adbase->write) (adbase, tfile, (char *)data, filePos,
 					  thisSize);
 		    if (code != thisSize)
 			return UBADLOG;
@@ -367,7 +369,11 @@ InitializeDB(register struct ubik_dbase *adbase)
 	    adbase->version.counter = 0;
 	    (*adbase->setlabel) (adbase, 0, &adbase->version);
 	}
+#ifdef AFS_PTHREAD_ENV
+	assert(pthread_cond_broadcast(&adbase->version_cond) == 0);
+#else
 	LWP_NoYieldSignal(&adbase->version);
+#endif
     }
     return 0;
 }
@@ -416,8 +422,8 @@ urecovery_Initialize(register struct ubik_dbase *adbase)
  * requests.  However, the recovery module still has one more task:
  * propagating the dbase out to everyone who is up in the network.
  */
-int
-urecovery_Interact(void)
+void *
+urecovery_Interact(void *dummy)
 {
     afs_int32 code, tcode;
     struct ubik_server *bestServer = NULL;
@@ -430,9 +436,14 @@ urecovery_Interact(void)
     struct timeval tv;
     int length, tlen, offset, file, nbytes;
     struct rx_call *rxcall;
-    char tbuffer[256];
+    char tbuffer[1024];
     struct ubik_stat ubikstat;
     struct in_addr inAddr;
+#ifndef OLD_URECOVERY
+    char pbuffer[1028];
+    int flen, fd = -1;
+    afs_int32 epoch, pass;
+#endif
 
     /* otherwise, begin interaction */
     urecovery_state = 0;
@@ -442,7 +453,11 @@ urecovery_Interact(void)
 	/* Run through this loop every 4 seconds */
 	tv.tv_sec = 4;
 	tv.tv_usec = 0;
+#ifdef AFS_PTHREAD_ENV
+	select(0, 0, 0, 0, &tv);
+#else
 	IOMGR_Select(0, 0, 0, 0, &tv);
+#endif
 
 	ubik_dprint("recovery running in state %x\n", urecovery_state);
 
@@ -535,11 +550,7 @@ urecovery_Interact(void)
 	    urecovery_state |= UBIK_RECHAVEDB;
 	} else {
 	    /* we don't have the best version; we should fetch it. */
-#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
-#else
-	    ObtainWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 	    urecovery_AbortAll(ubik_dbase);
 
 	    /* Rx code to do the Bulk fetch */
@@ -555,7 +566,7 @@ urecovery_Interact(void)
 		ubik_dprint("StartDiskGetFile failed=%d\n", code);
 		goto FetchEndCall;
 	    }
-	    nbytes = rx_Read(rxcall, &length, sizeof(afs_int32));
+	    nbytes = rx_Read(rxcall, (char *)&length, sizeof(afs_int32));
 	    length = ntohl(length);
 	    if (nbytes != sizeof(afs_int32)) {
 		ubik_dprint("Rx-read length error=%d\n", code = BULK_ERROR);
@@ -563,40 +574,72 @@ urecovery_Interact(void)
 		goto FetchEndCall;
 	    }
 
-	    /* Truncate the file firest */
+#ifdef OLD_URECOVERY
+	    /* Truncate the file first */
 	    code = (*ubik_dbase->truncate) (ubik_dbase, file, 0);
 	    if (code) {
 		ubik_dprint("truncate io error=%d\n", code);
 		goto FetchEndCall;
 	    }
-
+	    tversion.counter = 0;
+#endif
 	    /* give invalid label during file transit */
 	    tversion.epoch = 0;
-	    tversion.counter = 0;
 	    code = (*ubik_dbase->setlabel) (ubik_dbase, file, &tversion);
 	    if (code) {
 		ubik_dprint("setlabel io error=%d\n", code);
 		goto FetchEndCall;
 	    }
+#ifndef OLD_URECOVERY
+	    flen = length;
+	    afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.TMP", ubik_dbase->pathName);
+	    fd = open(pbuffer, O_CREAT | O_RDWR | O_TRUNC, 0600);
+	    if (fd < 0) {
+		code = errno;
+		goto FetchEndCall;
+	    }
+	    code = lseek(fd, HDRSIZE, 0);
+	    if (code != HDRSIZE) {
+		close(fd);
+		goto FetchEndCall;
+	    }
+#endif
 
+	    pass = 0;
 	    while (length > 0) {
 		tlen = (length > sizeof(tbuffer) ? sizeof(tbuffer) : length);
+#ifndef AFS_PTHREAD_ENV
+		if (pass % 4 == 0)
+		    IOMGR_Poll();
+#endif
 		nbytes = rx_Read(rxcall, tbuffer, tlen);
 		if (nbytes != tlen) {
 		    ubik_dprint("Rx-read bulk error=%d\n", code = BULK_ERROR);
 		    code = EIO;
+		    close(fd);
 		    goto FetchEndCall;
 		}
+#ifdef OLD_URECOVERY
 		nbytes =
 		    (*ubik_dbase->write) (ubik_dbase, file, tbuffer, offset,
 					  tlen);
+#else
+		nbytes = write(fd, tbuffer, tlen);
+		pass++;
+#endif
 		if (nbytes != tlen) {
 		    code = UIOERROR;
+		    close(fd);
 		    goto FetchEndCall;
 		}
 		offset += tlen;
 		length -= tlen;
 	    }
+#ifndef OLD_URECOVERY
+	    code = close(fd);
+	    if (code)
+		goto FetchEndCall;
+#endif	    
 	    code = EndDISK_GetFile(rxcall, &tversion);
 	  FetchEndCall:
 	    tcode = rx_EndCall(rxcall, code);
@@ -607,27 +650,57 @@ urecovery_Interact(void)
 		urecovery_state |= UBIK_RECHAVEDB;
 		memcpy(&ubik_dbase->version, &tversion,
 		       sizeof(struct ubik_version));
+#ifdef OLD_URECOVERY
 		(*ubik_dbase->sync) (ubik_dbase, 0);	/* get data out first */
+#else
+		afs_snprintf(tbuffer, sizeof(tbuffer), "%s.DB0", ubik_dbase->pathName);
+#ifdef AFS_NT40_ENV
+		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.OLD", ubik_dbase->pathName);
+		code = unlink(pbuffer);
+		if (!code)
+		    code = rename(tbuffer, pbuffer);
+		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.TMP", ubik_dbase->pathName);
+#endif
+		if (!code) 
+		    code = rename(pbuffer, tbuffer);
+		if (!code) 
+		    code = (*ubik_dbase->open) (ubik_dbase, 0);
+		if (!code)
+#endif
 		/* after data is good, sync disk with correct label */
 		code =
 		    (*ubik_dbase->setlabel) (ubik_dbase, 0,
 					     &ubik_dbase->version);
+#ifndef OLD_URECOVERY
+#ifdef AFS_NT40_ENV
+		afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0.OLD", ubik_dbase->pathName);
+		unlink(pbuffer);
+#endif
+#endif
 	    }
 	    if (code) {
+#ifndef OLD_URECOVERY
+		unlink(pbuffer);
+		/* 
+		 * We will effectively invalidate the old data forever now.
+		 * Unclear if we *should* but we do.
+		 */
+#endif
 		ubik_dbase->version.epoch = 0;
 		ubik_dbase->version.counter = 0;
 		ubik_print("Ubik: Synchronize database failed (error = %d)\n",
 			   code);
 	    } else {
 		ubik_print("Ubik: Synchronize database completed\n");
+		urecovery_state |= UBIK_RECHAVEDB;
 	    }
 	    udisk_Invalidate(ubik_dbase, 0);	/* data has changed */
-	    LWP_NoYieldSignal(&ubik_dbase->version);
-#if defined(UBIK_PAUSE)
-	    DBRELE(ubik_dbase);
+#ifdef AFS_PTHREAD_ENV
+	    assert(pthread_cond_broadcast(&ubik_dbase->version_cond) == 0);
 #else
-	    ReleaseWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
+	    LWP_NoYieldSignal(&ubik_dbase->version);
+#endif
+	    DBRELE(ubik_dbase);
 	}
 #if defined(UBIK_PAUSE)
 	if (!(urecovery_state & UBIK_RECSYNCSITE))
@@ -642,11 +715,7 @@ urecovery_Interact(void)
 	 * database and overwrite this one.
 	 */
 	if (ubik_dbase->version.epoch == 1) {
-#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
-#else
-	    ObtainWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 	    urecovery_AbortAll(ubik_dbase);
 	    ubik_epochTime = 2;
 	    ubik_dbase->version.epoch = ubik_epochTime;
@@ -654,12 +723,12 @@ urecovery_Interact(void)
 	    code =
 		(*ubik_dbase->setlabel) (ubik_dbase, 0, &ubik_dbase->version);
 	    udisk_Invalidate(ubik_dbase, 0);	/* data may have changed */
-	    LWP_NoYieldSignal(&ubik_dbase->version);
-#if defined(UBIK_PAUSE)
-	    DBRELE(ubik_dbase);
+#ifdef AFS_PTHREAD_ENV
+	    assert(pthread_cond_broadcast(&ubik_dbase->version_cond) == 0);
 #else
-	    ReleaseWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
+	    LWP_NoYieldSignal(&ubik_dbase->version);
+#endif
+	    DBRELE(ubik_dbase);
 	}
 
 	/* Check the other sites and send the database to them if they
@@ -669,11 +738,7 @@ urecovery_Interact(void)
 	    /* now propagate out new version to everyone else */
 	    dbok = 1;		/* start off assuming they all worked */
 
-#if defined(UBIK_PAUSE)
 	    DBHOLD(ubik_dbase);
-#else
-	    ObtainWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 	    /*
 	     * Check if a write transaction is in progress. We can't send the
 	     * db when a write is in progress here because the db would be
@@ -689,20 +754,16 @@ urecovery_Interact(void)
 		tv.tv_sec = 0;
 		tv.tv_usec = 50000;
 		while ((ubik_dbase->flags & DBWRITING) && (safety < 500)) {
-#if defined(UBIK_PAUSE)
 		    DBRELE(ubik_dbase);
-#else
-		    ReleaseWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 		    /* sleep for a little while */
+#ifdef AFS_PTHREAD_ENV
+		    select(0, 0, 0, 0, &tv);
+#else
 		    IOMGR_Select(0, 0, 0, 0, &tv);
+#endif
 		    tv.tv_usec += 10000;
 		    safety++;
-#if defined(UBIK_PAUSE)
 		    DBHOLD(ubik_dbase);
-#else
-		    ObtainWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 		}
 	    }
 
@@ -769,15 +830,12 @@ urecovery_Interact(void)
 		    ts->currentDB = 1;
 		}
 	    }
-#if defined(UBIK_PAUSE)
 	    DBRELE(ubik_dbase);
-#else
-	    ReleaseWriteLock(&ubik_dbase->versionLock);
-#endif /* UBIK_PAUSE */
 	    if (dbok)
 		urecovery_state |= UBIK_RECSENTDB;
 	}
     }
+    return NULL;
 }
 
 /*
