@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/remote.c,v 1.15.4.7 2008/07/19 06:15:02 rra Exp $");
+    ("$Header: /cvs/openafs/src/ubik/remote.c,v 1.15.4.9 2008/10/27 23:53:46 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -489,18 +489,19 @@ SDISK_SendFile(rxcall, file, length, avers)
      struct ubik_version *avers;
 {
     register afs_int32 code;
-    register struct ubik_dbase *dbase;
+    struct ubik_dbase *dbase = NULL;
     char tbuffer[1024];
     afs_int32 offset;
     struct ubik_version tversion;
     register int tlen;
     struct rx_peer *tpeer;
     struct rx_connection *tconn;
-    afs_uint32 otherHost;
+    afs_uint32 otherHost = 0;
 #ifndef OLD_URECOVERY
     char pbuffer[1028];
     int flen, fd = -1;
-    afs_int32 epoch, pass;
+    afs_int32 epoch = 0;
+    afs_int32 pass;
 #endif
 
     /* send the file back to the requester */
@@ -560,7 +561,6 @@ SDISK_SendFile(rxcall, file, length, avers)
 	close(fd);
 	goto failed;
     }
-#else
     pass = 0;
 #endif
     memcpy(&ubik_dbase->version, &tversion, sizeof(struct ubik_version));
@@ -639,8 +639,10 @@ SDISK_SendFile(rxcall, file, length, avers)
 #ifndef OLD_URECOVERY
 	unlink(pbuffer);
 	/* Failed to sync. Allow reads again for now. */
-	tversion.epoch = epoch;
-	(*dbase->setlabel) (dbase, file, &tversion);
+	if (dbase != NULL) {
+	    tversion.epoch = epoch;
+	    (*dbase->setlabel) (dbase, file, &tversion);
+	}
 #endif
 	ubik_print
 	    ("Ubik: Synchronize database with server %s failed (error = %d)\n",
