@@ -73,6 +73,8 @@
 #ifdef AFS_PTHREAD_ENV
 #include <pthread.h>
 #include <assert.h>
+#else
+#include <lwp.h>
 #endif
 
 /* per-client structure for ubik */
@@ -170,6 +172,12 @@ struct ubik_dbase {
     int (*getnfiles) (struct ubik_dbase * adbase);	/* find out number of files */
     short readers;		/* number of current read transactions */
     struct ubik_version cachedVersion;	/* version of caller's cached data */
+#ifdef AFS_PTHREAD_ENV
+    pthread_cond_t version_cond;    /* condition variable to manage changes to version */
+    pthread_cond_t flags_cond;      /* condition variable to manage changes to flags */
+  pthread_mutex_t version_mutex;
+  pthread_mutex_t flags_mutex;
+#endif
 };
 
 /* procedures for automatically authenticating ubik connections */
@@ -303,7 +311,8 @@ extern int uphys_getlabel(register struct ubik_dbase *adbase, afs_int32 afile,
 extern int uphys_setlabel(register struct ubik_dbase *adbase, afs_int32 afile,
 			  struct ubik_version *aversion);
 extern int uphys_sync(register struct ubik_dbase *adbase, afs_int32 afile);
-
+extern void uphys_invalidate(register struct ubik_dbase *adbase, 
+			     afs_int32 afid);
 
 /* recovery.c */
 extern int urecovery_ResetState(void);
@@ -313,11 +322,11 @@ extern int urecovery_AllBetter(register struct ubik_dbase *adbase,
 extern int urecovery_AbortAll(struct ubik_dbase *adbase);
 extern int urecovery_CheckTid(register struct ubik_tid *atid);
 extern int urecovery_Initialize(register struct ubik_dbase *adbase);
-extern int urecovery_Interact(void);
+extern void *urecovery_Interact(void *);
 extern int DoProbe(struct ubik_server *server);
 
 
-extern int ubeacon_Interact();
+extern void *ubeacon_Interact(void *);
 extern int sdisk_Interact();
 extern int uvote_Interact();
 extern int DISK_Abort();
@@ -354,9 +363,28 @@ extern int ubik_ClientInit(register struct rx_connection **serverconns,
 
 extern afs_int32 ubik_ClientDestroy(struct ubik_client *aclient);
 
+extern afs_int32 ubik_CallIter(int (*aproc) (), struct ubik_client *aclient,
+			       afs_int32 aflags, int *apos, long p1, long p2,
+			       long p3, long p4, long p5, long p6, long p7,
+			       long p8, long p9, long p10, long p11, long p12,
+			       long p13, long p14, long p15, long p16);
+
+extern struct rx_connection *ubik_RefreshConn(struct rx_connection *tc);
+
 /* ubik.c */
 extern int ubik_BeginTrans(register struct ubik_dbase *dbase,
 			   afs_int32 transMode, struct ubik_trans **transPtr);
 extern int ubik_EndTrans(register struct ubik_trans *transPtr);
 
+/* uinit.c */
+
+extern afs_int32 ugen_ClientInit(int noAuthFlag, char *confDir, char *cellName,
+				 afs_int32 sauth, 
+				 struct ubik_client **uclientp,
+				 int (*secproc) (), char *funcName, 
+				 afs_int32 gen_rxkad_level, 
+				 afs_int32 maxservers, char *serviceid, 
+				 afs_int32 deadtime, afs_uint32 server, 
+				 afs_uint32 port, afs_int32 usrvid);
+
 #endif /* UBIK_H */
