@@ -14,11 +14,11 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.16 2003/09/24 19:26:54 shadow Exp $");
+    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.23.2.4 2008/10/27 23:41:46 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
-#if !defined(AFS_SUN3_ENV) && !defined(sys_vax_ul43) && !defined(AFS_DARWIN_ENV)
+#if !defined(AFS_SUN3_ENV) && !defined(sys_vax_ul43) 
 #include <time.h>
 /*#ifdef	AFS_AIX_ENV*/
 #include <sys/time.h>
@@ -27,13 +27,7 @@ RCSID
 #undef abs
 #include <stdlib.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/stds.h>
 #include <afs/cmd.h>
@@ -41,7 +35,7 @@ RCSID
 #include <afs/icl.h>
 #include <afs/afsutil.h>
 
-#if defined(AFS_ALPHA_ENV) || defined(AFS_SGI61_ENV) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || defined(AFS_SGI61_ENV) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 /* For SGI 6.2, this is changed to 1 if it's a 32 bit kernel. */
 int afs_icl_sizeofLong = 2;
 #else
@@ -87,6 +81,7 @@ struct logInfo {
 } *allInfo = 0;
 
 char dumpFileName[256] = "";
+void
 RegisterIclDumpFileName(name)
      char *name;
 {
@@ -258,7 +253,7 @@ CheckTypes(bufferp, typesp, typeCount)
 #if defined(AFS_SGI61_ENV) && !defined(AFS_SGI62_ENV)
 #define uint64_t long long
 #endif
-static
+static void
 DisplayRecord(outFilep, alp, rsize)
      FILE *outFilep;
      register afs_int32 *alp;
@@ -285,6 +280,7 @@ DisplayRecord(outFilep, alp, rsize)
     int status;
     int printed;		/* did we print the string yet? */
     afs_int32 *tlp;
+    time_t tmv;
 
     /* decode parameters */
     temp = alp[0];		/* type encoded in low-order 24 bits, t0 high */
@@ -312,14 +308,12 @@ DisplayRecord(outFilep, alp, rsize)
 		printfParms[pfpix] <<= 32;
 		printfParms[pfpix] |= alp[pix + 1];
 	    }
-#else /* AFS_SGI61_ENV */
-#ifdef AFS_ALPHA_ENV
+#elif defined(AFS_OSF_ENV)
 	    printfParms[pfpix] = alp[pix + 1];
 	    printfParms[pfpix] |= (alp[pix] <<= 32);
-#else /* AFS_ALPHA_ENV */
+#else /* !AFS_OSF_ENV && !AFS_SGI61_ENV */
 	    printfParms[pfpix] = alp[pix];
-#endif /* AFS_ALPHA_ENV */
-#endif /* AFS_SGI61_ENV */
+#endif
 	    pfpix++;
 	    break;
 	case ICL_TYPE_INT32:
@@ -356,7 +350,8 @@ DisplayRecord(outFilep, alp, rsize)
 #endif /* AFS_SGI64_ENV */
 	    break;
 	case ICL_TYPE_UNIXDATE:
-	    printfParms[pfpix++] = (long)ctime((time_t *) & alp[pix]);
+	    tmv = alp[pix];
+	    printfParms[pfpix++] = (long)ctime(&tmv);
 	    break;
 	default:
 	    printf("DisplayRecord: Bad type %d in decode switch.\n", type);
@@ -452,8 +447,9 @@ DisplayRecord(outFilep, alp, rsize)
     }
     if (!printed) {
 	if (alp[1] == ICL_INFO_TIMESTAMP) {
+	    tmv = alp[4];
 	    fprintf(outFilep, "time %d.%06d, pid %u: %s\n", alp[3] / 1000000,
-		    alp[3] % 1000000, alp[2], ctime((time_t *) & alp[4]));
+		    alp[3] % 1000000, alp[2], ctime(&tmv));
 	} else {
 	    fprintf(outFilep, "raw op %d, time %d.%06d, pid %u\n", alp[1],
 		    alp[3] / 1000000, alp[3] % 1000000, alp[2]);
@@ -502,8 +498,9 @@ DisplayRecord(outFilep, alp, rsize)
 		    fprintf(outFilep, "p%d:%s ", i, (char *)&alp[pix]);
 		    break;
 		case ICL_TYPE_UNIXDATE:
+		    tmv = alp[pix];
 		    fprintf(outFilep, "p%d:%s ", i,
-			    ctime((time_t *) & alp[pix]));
+			    ctime(&tmv));
 		    break;
 		default:
 		    printf
@@ -1493,7 +1490,11 @@ dce1_error_inq_text(status_to_convert, error_text, status)
     char nls_filename[11];
     char alt_filename[80];
     char *message;
+#if defined(AFS_64BITPOINTER_ENV)
+    long J;
+#else
     int J;
+#endif
     static char *facility_names[] = {
 	"xxx",
 	"afs"
@@ -1559,8 +1560,12 @@ dce1_error_inq_text(status_to_convert, error_text, status)
 #else
     catd = (nl_catd) catopen1(nls_filename, 0);
 #endif
+#else
+#if defined(AFS_64BITPOINTER_ENV)
+    J = (long)catopen(nls_filename, 0);
 #else
     J = (int)catopen(nls_filename, 0);
+#endif
     catd = (nl_catd) J;
 #endif
     if (catd == (nl_catd) - 1) {
@@ -1582,8 +1587,12 @@ dce1_error_inq_text(status_to_convert, error_text, status)
 #else
 	catd = (nl_catd) catopen1(alt_filename, 0);
 #endif
+#else
+#if defined(AFS_64BITPOINTER_ENV)
+        J = (long)catopen(alt_filename, 0);
 #else
 	J = (int)catopen(alt_filename, 0);
+#endif
 	catd = (nl_catd) J;
 #endif
 	if (catd == (nl_catd) - 1) {
@@ -1653,7 +1662,7 @@ icl_DumpKernel(outFilep, setname)
 	for (i = 0; i < ICL_LOGSPERSET; i++) {
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_ENUMLOGSBYSET, (long)setname,
-			    i, (long)tname, sizeof(tname));
+			    i, (long)tname, sizeof(tname), 0, 0);
 	    if (code) {
 		if (errno == EBADF) {
 		    code = 0;
@@ -1663,7 +1672,7 @@ icl_DumpKernel(outFilep, setname)
 	    }
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)tname,
-			    (long)&dummy, (long)&dummy2, 0);
+			    (long)&dummy, (long)&dummy2, 0, 0, 0);
 	    if (code)
 		break;
 	    found++;
@@ -1682,7 +1691,7 @@ icl_DumpKernel(outFilep, setname)
 	for (i = 0; i < 1000; i++) {
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_ENUMLOGS, i, (long)tname,
-			    sizeof(tname), (long)&dummy);
+			    sizeof(tname), (long)&dummy, 0, 0);
 	    if (code)
 		break;
 	    if (dummy > bufferSize)	/* find biggest log */
@@ -1718,7 +1727,7 @@ icl_DumpKernel(outFilep, setname)
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_COPYOUT, (long)lip->name,
 			    (long)(bufferp + nwords), bufferSize - nwords,
-			    (long)&i);
+			    (long)&i, 0, 0);
 	    if (code < 0) {
 		/* otherwise we've got an error */
 		fprintf(outFilep, "Returned error %d dumping log.\n", errno);
@@ -1777,7 +1786,7 @@ icl_ClearLog(name)
 {
     afs_int32 code;
 
-    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRLOG, (long)name, 0, 0, 0);
+    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRLOG, (long)name, 0, 0, 0, 0, 0);
     return code;
 }
 
@@ -1787,7 +1796,7 @@ icl_ClearSet(name)
 {
     afs_int32 code;
 
-    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRSET, (long)name, 0, 0, 0);
+    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRSET, (long)name, 0, 0, 0, 0, 0);
     return code;
 }
 
@@ -1796,7 +1805,7 @@ icl_ClearAll()
 {
     afs_int32 code;
 
-    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRALL, 0, 0, 0, 0);
+    code = afs_syscall(AFSCALL_ICL, ICL_OP_CLRALL, 0, 0, 0, 0, 0, 0);
     return code;
 }
 
@@ -1813,7 +1822,7 @@ icl_ListSets(outFileP)
     for (i = 0; i < 1000; i++) {
 	code =
 	    afs_syscall(AFSCALL_ICL, ICL_OP_ENUMSETS, i, (long)tname,
-			sizeof(tname), (long)&states);
+			sizeof(tname), (long)&states, 0, 0);
 	if (code)
 	    break;
 	(void)fprintf(outFileP, "%s %s%s%s\n", tname,
@@ -1840,14 +1849,14 @@ icl_ListLogs(outFileP, int32flg)
     for (i = 0; i < 1000; i++) {
 	code =
 	    afs_syscall(AFSCALL_ICL, ICL_OP_ENUMLOGS, i, (long)tname,
-			sizeof(tname), (long)&logSize);
+			sizeof(tname), (long)&logSize, 0, 0);
 	if (code)
 	    break;
 	if (int32flg) {
 	    /* get more information on the log */
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)tname,
-			    (long)&logSize, (long)&allocated, 0);
+			    (long)&logSize, (long)&allocated, 0, 0, 0);
 	    if (code)
 		break;
 	    (void)fprintf(outFileP, "%s : %d kbytes (%s)\n", tname,
@@ -1876,7 +1885,7 @@ icl_ListLogsBySet(outFileP, setname, int32flg)
     for (i = 0; i < ICL_LOGSPERSET; i++) {
 	code =
 	    afs_syscall(AFSCALL_ICL, ICL_OP_ENUMLOGSBYSET, (long)setname, i,
-			(long)tname, sizeof(tname));
+			(long)tname, sizeof(tname), 0, 0);
 	if (code) {
 	    if (errno == EBADF) {
 		code = 0;
@@ -1888,7 +1897,7 @@ icl_ListLogsBySet(outFileP, setname, int32flg)
 	    /* get more information on the log */
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)tname,
-			    (long)&logSize, (long)&allocated, 0);
+			    (long)&logSize, (long)&allocated, 0, 0, 0);
 	    if (code)
 		break;
 	    (void)fprintf(outFileP, "%s : %d kbytes (%s)\n", tname,
@@ -1909,7 +1918,7 @@ icl_ChangeSetState(name, op)
 {
     afs_int32 code;
 
-    code = afs_syscall(AFSCALL_ICL, ICL_OP_SETSTAT, (long)name, op, 0, 0);
+    code = afs_syscall(AFSCALL_ICL, ICL_OP_SETSTAT, (long)name, op, 0, 0, 0, 0);
     return code;
 }
 
@@ -1920,7 +1929,7 @@ icl_ChangeAllSetState(op)
 {
     afs_int32 code;
 
-    code = afs_syscall(AFSCALL_ICL, ICL_OP_SETSTATALL, op, 0, 0, 0);
+    code = afs_syscall(AFSCALL_ICL, ICL_OP_SETSTATALL, op, 0, 0, 0, 0, 0);
     return code;
 }
 
@@ -1934,7 +1943,7 @@ icl_ChangeLogSize(name, logSize)
 
     code =
 	afs_syscall(AFSCALL_ICL, ICL_OP_SETLOGSIZE, (long)name, logSize, 0,
-		    0);
+		    0, 0, 0);
     return code;
 }
 
@@ -1948,7 +1957,7 @@ icl_GetLogsize(logname, logSizeP, allocatedP)
     afs_int32 code;
     code =
 	afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)logname,
-		    (long)logSizeP, (long)allocatedP, 0);
+		    (long)logSizeP, (long)allocatedP, 0, 0, 0);
     return code;
 }
 
@@ -1961,7 +1970,7 @@ icl_GetSetState(setname, stateP)
     afs_int32 code;
     code =
 	afs_syscall(AFSCALL_ICL, ICL_OP_GETSETINFO, (long)setname,
-		    (long)stateP, 0, 0);
+		    (long)stateP, 0, 0, 0, 0);
     return code;
 }
 
@@ -1984,7 +1993,7 @@ icl_TailKernel(outFilep, logname, waitTime)
     /* get information about the specified log */
     code =
 	afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)logname,
-		    (long)&bufferSize, (long)&allocated, 0);
+		    (long)&bufferSize, (long)&allocated, 0, 0, 0);
     if (code) {
 	if (errno == ENOENT)
 	    (void)fprintf(stderr, "'%s' not found\n", logname);
@@ -2021,7 +2030,7 @@ icl_TailKernel(outFilep, logname, waitTime)
 	    code =
 		afs_syscall(AFSCALL_ICL, ICL_OP_COPYOUTCLR, (long)logname,
 			    (long)(bufferp + nwords), bufferSize - nwords,
-			    (long)&i);
+			    (long)&i, 0, 0);
 	    if (code < 0) {
 		/* otherwise we've got an error */
 		fprintf(stderr, "returned error %d dumping log.\n", errno);
@@ -2069,7 +2078,7 @@ icl_TailKernel(outFilep, logname, waitTime)
 	/* see if things have changed */
 	code =
 	    afs_syscall(AFSCALL_ICL, ICL_OP_GETLOGINFO, (long)logname,
-			(long)&newBufferSize, (long)&allocated, 0);
+			(long)&newBufferSize, (long)&allocated, 0, 0, 0);
 	if (code) {
 	    if (errno == ENOENT)
 		(void)fprintf(stderr, "'%s' not found\n", logname);
@@ -2115,7 +2124,7 @@ icl_TailKernel(outFilep, logname, waitTime)
 afs_syscall(call, parm0, parm1, parm2, parm3, parm4, parm5, parm6)
      long call, parm0, parm1, parm2, parm3, parm4, parm5, parm6;
 {
-    int code;
+    int code, rval;
 #ifdef AFS_LINUX20_ENV
 #if defined AFS_LINUX_64BIT_KERNEL
     long long eparm[4];
@@ -2133,15 +2142,23 @@ afs_syscall(call, parm0, parm1, parm2, parm3, parm4, parm5, parm6)
 #endif
     /* Linux can only handle 5 arguments in the actual syscall. */
     if (call == AFSCALL_ICL) {
-	code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, eparm);
+	rval = proc_afs_syscall(call, parm0, parm1, parm2, eparm, &code);
+	if (rval)
+	    code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, eparm);
     } else {
-	code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, parm3);
+	rval = proc_afs_syscall(call, parm0, parm1, parm2, parm3, &code);
+	if (rval)
+	    code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, parm3);
     }
 #if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
     /* on sparc this function returns none value, so do it myself */
     __asm__ __volatile__("mov	%o0, %i0; ret; restore");
 #endif
 #else
+#ifdef AFS_DARWIN80_ENV
+    code = ioctl_afs_syscall(call, parm0, parm1, parm2, parm3, parm4, parm5, &rval);
+    if (!code) code = rval;
+#else
 #if !defined(AFS_SGI_ENV) && !defined(AFS_AIX32_ENV)
     code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, parm3, parm4);
 #else
@@ -2151,6 +2168,7 @@ afs_syscall(call, parm0, parm1, parm2, parm3, parm4, parm5, parm6)
     code = syscall(AFSCALL_ICL, parm0, parm1, parm2, parm3, parm4);
 #endif
 #endif
+#endif
 #endif /* AFS_LINUX20_ENV */
     return code;
 }
@@ -2690,10 +2708,8 @@ GetBulkLogInfo()
 }
 
 
-static
-DoDump(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoDump(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code = 0;
     afs_int32 tcode;
@@ -2769,7 +2785,7 @@ SetUpDump()
     struct cmd_syndesc *dumpSyntax;
 
     dumpSyntax =
-	cmd_CreateSyntax("dump", DoDump, (char *)NULL, "dump AFS trace logs");
+	cmd_CreateSyntax("dump", DoDump, NULL, "dump AFS trace logs");
     (void)cmd_AddParm(dumpSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(dumpSyntax, "-follow", CMD_SINGLE, CMD_OPTIONAL,
 		      "log_name");
@@ -2779,10 +2795,8 @@ SetUpDump()
 		      "seconds_between_reads");
 }
 
-static
-DoShowLog(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowLog(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2845,17 +2859,15 @@ SetUpShowLog()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lslog", DoShowLog, (char *)NULL,
+	cmd_CreateSyntax("lslog", DoShowLog, NULL,
 			 "list available logs");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(showSyntax, "-log", CMD_LIST, CMD_OPTIONAL, "log_name");
     (void)cmd_AddParm(showSyntax, "-long", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoShowSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2903,15 +2915,13 @@ SetUpShowSet()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lsset", DoShowSet, (char *)NULL,
+	cmd_CreateSyntax("lsset", DoShowSet, NULL,
 			 "list available event sets");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
 }
 
-static
-DoClear(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoClear(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2962,7 +2972,7 @@ SetUpClear()
     struct cmd_syndesc *clearSyntax;
 
     clearSyntax =
-	cmd_CreateSyntax("clear", DoClear, (char *)NULL,
+	cmd_CreateSyntax("clear", DoClear, NULL,
 			 "clear logs by logname or by event set");
     (void)cmd_AddParm(clearSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
 		      "set_name");
@@ -2970,10 +2980,8 @@ SetUpClear()
 		      "log_name");
 }
 
-static
-DoSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3051,7 +3059,7 @@ SetUpSet()
     struct cmd_syndesc *setSyntax;
 
     setSyntax =
-	cmd_CreateSyntax("setset", DoSet, (char *)NULL,
+	cmd_CreateSyntax("setset", DoSet, NULL,
 			 "set state of event sets");
     (void)cmd_AddParm(setSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(setSyntax, "-active", CMD_FLAG, CMD_OPTIONAL, "");
@@ -3059,10 +3067,8 @@ SetUpSet()
     (void)cmd_AddParm(setSyntax, "-dormant", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoResize(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoResize(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3110,7 +3116,7 @@ SetUpResize()
     struct cmd_syndesc *setsizeSyntax;
 
     setsizeSyntax =
-	cmd_CreateSyntax("setlog", DoResize, (char *)NULL,
+	cmd_CreateSyntax("setlog", DoResize, NULL,
 			 "set the size of a log");
     (void)cmd_AddParm(setsizeSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
 		      "log_name");
