@@ -10,7 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/livesys.c,v 1.1 2001/10/09 06:32:58 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/venus/livesys.c,v 1.4.14.3 2007/10/30 15:16:48 shadow Exp $");
 
 #include <afs/afs_args.h>
 #include <rx/xdr.h>
@@ -21,20 +22,22 @@ RCSID("$Header: /cvs/openafs/src/venus/livesys.c,v 1.1 2001/10/09 06:32:58 shado
 #include <stdio.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
+#include <string.h>
 #include <afs/stds.h>
 #include <afs/vice.h>
 #include <afs/venus.h>
+#include <afs/com_err.h>
 #ifdef	AFS_AIX32_ENV
 #include <signal.h>
 #endif
-#include <strings.h>
 
 #define	MAXSIZE	2048
 static char space[MAXSIZE];
 
-int main(argc, argv)
-    int argc;
-    char **argv;
+int
+main(argc, argv)
+     int argc;
+     char **argv;
 {
     afs_int32 code;
     struct ViceIoctl blob;
@@ -63,7 +66,7 @@ int main(argc, argv)
     memcpy(space, &setp, sizeof(afs_int32));
     code = pioctl(0, VIOC_AFS_SYSNAME, &blob, 1);
     if (code) {
-	fprintf(stderr, "livesys: %s\n", error_message(code));
+	fprintf(stderr, "livesys: %s\n", afs_error_message(code));
 	return 1;
     }
     input = space;
