@@ -11,10 +11,11 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.12 2003/11/12 14:23:30 rees Exp $");
+    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.15.4.4 2008/10/27 23:41:47 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <string.h>
 #include <lock.h>
 #include <afs/afsutil.h>
 #include <ubik.h>
@@ -25,13 +26,6 @@ RCSID
 #else
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <afs/keys.h>
 #include "vlserver.h"
 #include "afs/audit.h"
@@ -61,17 +55,17 @@ static char rxinfo_str[128];	/* Need rxinfo string to be non-local */
 static int put_attributeentry();
 static int put_nattributeentry();
 static int RemoveEntry();
-static ReleaseEntry();
+static void ReleaseEntry();
 static int check_vldbentry();
 static int check_nvldbentry();
 static int vldbentry_to_vlentry();
 static int nvldbentry_to_vlentry();
 static get_vldbupdateentry();
 static int repsite_exists();
-static repsite_compress();
-static vlentry_to_vldbentry();
-static vlentry_to_nvldbentry();
-static vlentry_to_uvldbentry();
+static void repsite_compress();
+static void vlentry_to_vldbentry();
+static void vlentry_to_nvldbentry();
+static void vlentry_to_uvldbentry();
 static int InvalidVolname();
 static int InvalidVoltype();
 static int InvalidOperation();
@@ -97,7 +91,8 @@ rxinfo(rxcall)
 	rxkad_GetServerInfo(rxcall->conn, NULL, &exp, tname, tinst, tcell,
 			    NULL);
     if (!code)
-	sprintf(rxinfo_str, "%s %s", inet_ntoa(hostAddr), tname);
+	sprintf(rxinfo_str, "%s %s%s%s%s%s", inet_ntoa(hostAddr), tname,
+		tinst?".":"", tinst?tinst:"", tcell?"@":"", tcell?tcell:"");
     else
 	sprintf(rxinfo_str, "%s noauth", inet_ntoa(hostAddr));
     return (rxinfo_str);
@@ -1136,11 +1131,13 @@ SVL_ListAttributes(rxcall, attributes, nentries, vldbentries)
 	    return VL_SIZEEXCEEDED;
 	}
     } else {
-	afs_int32 nextblockindex = 0, count = 0, k, match = 0;
+	afs_int32 nextblockindex = 0, count = 0, k = 0, match = 0;
 	while (nextblockindex =
 	       NextEntry(trans, nextblockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 	    match = 0;
@@ -1274,11 +1271,13 @@ SVL_ListAttributesN(rxcall, attributes, nentries, vldbentries)
 	    return VL_SIZEEXCEEDED;
 	}
     } else {
-	afs_int32 nextblockindex = 0, count = 0, k, match = 0;
+	afs_int32 nextblockindex = 0, count = 0, k = 0, match = 0;
 	while (nextblockindex =
 	       NextEntry(trans, nextblockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1372,12 +1371,14 @@ SVL_ListAttributesN2(rxcall, attributes, name, startindex, nentries,
     struct ubik_trans *trans;
     struct nvlentry tentry;
     struct nvldbentry *Vldbentry = 0, *VldbentryFirst = 0, *VldbentryLast = 0;
-    afs_int32 blockindex = 0, count = 0, k, match, matchindex;
+    afs_int32 blockindex = 0, count = 0, k, match;
+    afs_int32 matchindex = 0;
     int serverindex = -1;	/* no server found */
     int findserver = 0, findpartition = 0, findflag = 0, findname = 0;
     char *t;
     int pollcount = 0;
-    int namematchRWBK, namematchRO, thismatch, matchtype;
+    int namematchRWBK, namematchRO, thismatch;
+    int matchtype = 0;
     char volumename[VL_MAXNAMELEN];
 #ifdef HAVE_POSIX_REGEX
     regex_t re;
@@ -1460,7 +1461,9 @@ SVL_ListAttributesN2(rxcall, attributes, name, startindex, nentries,
 	blockindex = startindex;
 	while (blockindex = NextEntry(trans, blockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1639,7 +1642,8 @@ SVL_LinkedList(rxcall, attributes, nentries, vldbentries)
     struct ubik_trans *trans;
     struct nvlentry tentry;
     vldblist vllist, *vllistptr;
-    afs_int32 blockindex, count, k, match;
+    afs_int32 blockindex, count, match;
+    afs_int32 k = 0;
     int serverindex;
     int pollcount = 0;
 
@@ -1682,7 +1686,9 @@ SVL_LinkedList(rxcall, attributes, nentries, vldbentries)
 	    match = 0;
 
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1765,7 +1771,8 @@ SVL_LinkedListN(rxcall, attributes, nentries, vldbentries)
     struct ubik_trans *trans;
     struct nvlentry tentry;
     nvldblist vllist, *vllistptr;
-    afs_int32 blockindex, count, k, match;
+    afs_int32 blockindex, count, match;
+    afs_int32 k = 0;
     int serverindex;
     int pollcount = 0;
 
@@ -1808,7 +1815,9 @@ SVL_LinkedListN(rxcall, attributes, nentries, vldbentries)
 	    match = 0;
 
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1898,7 +1907,7 @@ SVL_GetStats(rxcall, stats, vital_header)
 #endif
     if (errorcode = Init_VLdbase(&trans, LOCKREAD, this_op))
 	return errorcode;
-    VLog(5, ("GetStats %\n", rxinfo(rxcall)));
+    VLog(5, ("GetStats %s\n", rxinfo(rxcall)));
     memcpy((char *)vital_header, (char *)&cheader.vital_header,
 	   sizeof(vital_vlheader));
     memcpy((char *)stats, (char *)&dynamic_statistics, sizeof(vldstats));
@@ -1952,7 +1961,7 @@ SVL_GetAddrs(rxcall, Handle, spare2, spare3, nentries, addrsp)
     return (ubik_EndTrans(trans));
 }
 
-#define PADDR(addr) printf("%d.%d.%d.%d", (addr>>24)&0xff, (addr>>16)&0xff, (addr>>8) &0xff, addr&0xff);
+#define PADDR(addr) VLog(0,("%d.%d.%d.%d", (addr>>24)&0xff, (addr>>16)&0xff, (addr>>8) &0xff, addr&0xff));
 
 afs_int32
 SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
@@ -1969,8 +1978,9 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
     afs_uint32 addrs[VL_MAXIPADDRS_PERMH];
     afs_int32 fbase;
     int count, willChangeEntry, foundUuidEntry, willReplaceCnt;
-    int WillReplaceEntry, WillChange[MAXSERVERID + 1], FoundUuid,
-	ReplaceEntry;
+    int WillReplaceEntry, WillChange[MAXSERVERID + 1];
+    int FoundUuid = 0;
+    int ReplaceEntry = 0;
     int srvidx, mhidx;
 
     COUNT_REQ(VLREGADDR);
@@ -2087,18 +2097,18 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 	|| (!foundUuidEntry && (count > 1))) {
 	VLog(0,
 	     ("The following fileserver is being registered in the VLDB:\n"));
-	printf("      [");
+	VLog(0, ("      ["));
 	for (k = 0; k < cnt; k++) {
 	    if (k > 0)
-		printf(" ");
+		VLog(0,(" "));
 	    PADDR(addrs[k]);
 	}
-	printf("]\n");
+	VLog(0,("]\n"));
 
 	if (foundUuidEntry) {
-	    printf
-		("   It would have replaced the existing VLDB server entry:\n");
-	    printf("      entry %d: [", FoundUuid);
+	    VLog(0,
+		 ("   It would have replaced the existing VLDB server entry:\n"));
+	    VLog(0, ("      entry %d: [", FoundUuid));
 	    base = (HostAddress[FoundUuid] >> 16) & 0xff;
 	    index = HostAddress[FoundUuid] & 0x0000ffff;
 	    exp = &ex_addr[base][index];
@@ -2106,21 +2116,21 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 		if (!exp->ex_addrs[mhidx])
 		    continue;
 		if (mhidx > 0)
-		    printf(" ");
+		    VLog(0,(" "));
 		PADDR(ntohl(exp->ex_addrs[mhidx]));
 	    }
-	    printf("]\n");
+	    VLog(0, ("]\n"));
 	}
 
 	if (count == 1)
-	    printf("   Yet another VLDB server entry exists:\n");
+	    VLog(0, ("   Yet another VLDB server entry exists:\n"));
 	else
-	    printf("   Yet other VLDB server entries exist:\n");
+	    VLog(0, ("   Yet other VLDB server entries exist:\n"));
 	for (j = 0; j < count; j++) {
 	    srvidx = WillChange[j];
-	    printf("      entry %d: ", srvidx);
+	    VLog(0, ("      entry %d: ", srvidx));
 	    if ((HostAddress[srvidx] & 0xff000000) == 0xff000000) {
-		printf("[");
+		VLog(0, ("["));
 		base = (HostAddress[srvidx] >> 16) & 0xff;
 		index = HostAddress[srvidx] & 0x0000ffff;
 		exp = &ex_addr[base][index];
@@ -2128,25 +2138,25 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 		    if (!exp->ex_addrs[mhidx])
 			continue;
 		    if (mhidx > 0)
-			printf(" ");
+			VLog(0, (" "));
 		    PADDR(ntohl(exp->ex_addrs[mhidx]));
 		}
-		printf("]");
+		VLog(0, ("]"));
 	    } else {
 		PADDR(HostAddress[srvidx]);
 	    }
-	    printf("\n");
+	    VLog(0, ("\n"));
 	}
 
 	if (count == 1)
-	    printf("   You must 'vos changeaddr' this other server entry\n");
+	    VLog(0, ("   You must 'vos changeaddr' this other server entry\n"));
 	else
-	    printf
-		("   You must 'vos changeaddr' these other server entries\n");
+	    VLog(0, 
+		("   You must 'vos changeaddr' these other server entries\n"));
 	if (foundUuidEntry)
-	    printf
-		("   and/or remove the sysid file from the registering fileserver\n");
-	printf("   before the fileserver can be registered in the VLDB.\n");
+	    VLog(0, 
+		("   and/or remove the sysid file from the registering fileserver\n"));
+	VLog(0, ("   before the fileserver can be registered in the VLDB.\n"));
 
 	ubik_AbortTrans(trans);
 	return VL_MULTIPADDR;
@@ -2178,26 +2188,26 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
     }
 
     VLog(0, ("The following fileserver is being registered in the VLDB:\n"));
-    printf("      [");
+    VLog(0, ("      ["));
     for (k = 0; k < cnt; k++) {
 	if (k > 0)
-	    printf(" ");
+	    VLog(0, (" "));
 	PADDR(addrs[k]);
     }
-    printf("]\n");
+    VLog(0, ("]\n"));
 
     if (foundUuidEntry) {
-	printf
-	    ("   It will replace the following existing entry in the VLDB (same uuid):\n");
-	printf("      entry %d: [", FoundUuid);
+	VLog(0, 
+	    ("   It will replace the following existing entry in the VLDB (same uuid):\n"));
+	VLog(0, ("      entry %d: [", FoundUuid));
 	for (k = 0; k < VL_MAXIPADDRS_PERMH; k++) {
 	    if (exp->ex_addrs[k] == 0)
 		continue;
 	    if (k > 0)
-		printf(" ");
+		VLog(0, (" "));
 	    PADDR(ntohl(exp->ex_addrs[k]));
 	}
-	printf("]\n");
+	VLog(0, ("]\n"));
     } else if (willReplaceCnt || (count == 1)) {
 	/* If we are not replacing an entry and there is only one entry to change,
 	 * then we will replace that entry.
@@ -2213,24 +2223,24 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 	    index = HostAddress[ReplaceEntry] & 0x0000ffff;
 	    exp = &ex_addr[fbase][index];
 
-	    printf
-		("   It will replace the following existing entry in the VLDB (new uuid):\n");
-	    printf("      entry %d: [", ReplaceEntry);
+	    VLog(0, 
+		("   It will replace the following existing entry in the VLDB (new uuid):\n"));
+	    VLog(0, ("      entry %d: [", ReplaceEntry));
 	    for (k = 0; k < VL_MAXIPADDRS_PERMH; k++) {
 		if (exp->ex_addrs[k] == 0)
 		    continue;
 		if (k > 0)
-		    printf(" ");
+		    VLog(0, (" "));
 		PADDR(ntohl(exp->ex_addrs[k]));
 	    }
-	    printf("]\n");
+	    VLog(0, ("]\n"));
 	} else {
 	    /* Not a mh entry. So we have to create a new mh entry and 
 	     * put it on the ReplaceEntry slot of the HostAddress array.
 	     */
-	    printf("   It will replace existing entry %d, ", ReplaceEntry);
+	    VLog(0, ("   It will replace existing entry %d, ", ReplaceEntry));
 	    PADDR(HostAddress[ReplaceEntry]);
-	    printf(", in the VLDB (new uuid):\n");
+	    VLog(0,(", in the VLDB (new uuid):\n"));
 
 	    code =
 		FindExtentBlock(trans, uuidp, 1, ReplaceEntry, &exp, &fbase);
@@ -2243,7 +2253,7 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 	/* There is no entry for this server, must create a new mh entry as
 	 * well as use a new slot of the HostAddress array.
 	 */
-	printf("   It will create a new entry in the VLDB.\n");
+	VLog(0, ("   It will create a new entry in the VLDB.\n"));
 	code = FindExtentBlock(trans, uuidp, 1, -1, &exp, &fbase);
 	if (code || !exp) {
 	    ubik_AbortTrans(trans);
@@ -2291,10 +2301,10 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 	tex = &ex_addr[fbase][index];
 
 	if (++m == 1)
-	    printf
-		("   The following existing entries in the VLDB will be updated:\n");
+	    VLog(0, 
+		("   The following existing entries in the VLDB will be updated:\n"));
 
-	printf("      entry %d: [", WillChange[i]);
+	VLog(0, ("      entry %d: [", WillChange[i]));
 	for (h = j = 0; j < VL_MAXIPADDRS_PERMH; j++) {
 	    if (tex->ex_addrs[j]) {
 		if (j > 0)
@@ -2315,7 +2325,7 @@ SVL_RegisterAddrs(rxcall, uuidp, spare1, addrsp)
 	for (j = h; j < VL_MAXIPADDRS_PERMH; j++) {
 	    tex->ex_addrs[j] = 0;	/* zero rest of mh entry */
 	}
-	printf("]\n");
+	VLog(0, ("]\n"));
 
 	/* Write out the modified mh entry */
 	tex->ex_uniquifier = htonl(ntohl(tex->ex_uniquifier) + 1);
@@ -2561,7 +2571,7 @@ RemoveEntry(trans, entryptr, tentry)
     return 0;
 }
 
-static
+static void
 ReleaseEntry(tentry, releasetype)
      struct nvlentry *tentry;
      afs_int32 releasetype;
@@ -2866,7 +2876,7 @@ repsite_exists(VlEntry, server, partition)
 
 
 /* Repsite table compression: used when deleting a repsite entry so that all active repsite entries are on the top of the table. */
-static
+static void
 repsite_compress(VlEntry, offset)
      struct nvlentry *VlEntry;
      int offset;
@@ -2887,7 +2897,7 @@ repsite_compress(VlEntry, offset)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_vldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct vldbentry *VldbEntry;
@@ -2929,7 +2939,7 @@ vlentry_to_vldbentry(VlEntry, VldbEntry)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_nvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct nvldbentry *VldbEntry;
@@ -2969,7 +2979,7 @@ vlentry_to_nvldbentry(VlEntry, VldbEntry)
     VldbEntry->flags = VlEntry->flags;
 }
 
-static
+static void
 vlentry_to_uvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct uvldbentry *VldbEntry;
@@ -3131,8 +3141,9 @@ ChangeIPAddr(ipaddr1, ipaddr2, atrans)
 {
     int i, j;
     afs_int32 code;
-    struct extentaddr *exp;
-    int base, index, mhidx;
+    struct extentaddr *exp = NULL;
+    int base = 0;
+    int index, mhidx;
     afsUUID tuuid;
     afs_int32 blockindex, count;
     int pollcount = 0;
@@ -3195,7 +3206,9 @@ ChangeIPAddr(ipaddr1, ipaddr2, atrans)
 	for (blockindex = NextEntry(atrans, 0, &tentry, &count); blockindex;
 	     blockindex = NextEntry(atrans, blockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 	    for (j = 0; j < NMAXNSERVERS; j++) {
@@ -3212,25 +3225,25 @@ ChangeIPAddr(ipaddr1, ipaddr2, atrans)
     VLog(0,
 	 ("The following IP address is being %s:\n",
 	  (ipaddr2 ? "changed" : "removed")));
-    printf("      entry %d: ", i);
+    VLog(0, ("      entry %d: ", i));
     if (exp) {
-	printf("[");
+	VLog(0, ("["));
 	for (mhidx = 0; mhidx < VL_MAXIPADDRS_PERMH; mhidx++) {
 	    if (!exp->ex_addrs[mhidx])
 		continue;
 	    if (mhidx > 0)
-		printf(" ");
+		VLog(0, (" "));
 	    PADDR(ntohl(exp->ex_addrs[mhidx]));
 	}
-	printf("]");
+	VLog(0, ("]"));
     } else {
 	PADDR(ipaddr1);
     }
     if (ipaddr2) {
-	printf(" -> ");
+	VLog(0, (" -> "));
 	PADDR(ipaddr2);
     }
-    printf("\n");
+    VLog(0, ("\n"));
 
     /* Change the registered uuuid addresses */
     if (exp) {
