@@ -19,7 +19,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/clone.c,v 1.19.2.6 2007/05/18 05:53:56 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/clone.c,v 1.19.2.9 2008/10/27 23:41:47 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -38,13 +38,7 @@ RCSID
 #include <sys/file.h>
 #include <sys/time.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <errno.h>
 #include <sys/stat.h>
 
@@ -174,14 +168,15 @@ DoCloneIndex(Volume * rwvp, Volume * clvp, VnodeClass class, int reclone)
     char buf[SIZEOF_LARGEDISKVNODE], dbuf[SIZEOF_LARGEDISKVNODE];
     struct VnodeDiskObject *rwvnode = (struct VnodeDiskObject *)buf;
     struct VnodeDiskObject *clvnode = (struct VnodeDiskObject *)dbuf;
-    Inode rwinode, clinode;
+    Inode rwinode = 0;
+    Inode clinode;
     struct clone_head decHead;
     struct clone_rock decRock;
-    afs_int32 offset, dircloned, inodeinced;
+    afs_int32 offset = 0;
+    afs_int32 dircloned, inodeinced;
 
     struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
     int ReadWriteOriginal = VolumeWriteable(rwvp);
-    struct DiskPartition *partition = rwvp->partition;
     Device device = rwvp->device;
 
     /* Open the RW volume's index file and seek to beginning */
