@@ -20,7 +20,7 @@
 #define MAXINT     (~(1<<((sizeof(int)*8)-1)))
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/vnode.c,v 1.27.2.4 2008/02/04 18:51:39 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/vnode.c,v 1.27.2.5 2008/10/27 23:41:47 shadow Exp $");
 
 #include <errno.h>
 #include <stdio.h>
@@ -109,9 +109,8 @@ extern int LogLevel;
 static afs_int32 theLog[THELOGSIZE];
 static afs_int32 vnLogPtr = 0;
 void
-VNLog(aop, anparms, av1, av2, av3, av4)
-     afs_int32 aop, anparms;
-     afs_int32 av1, av2, av3, av4;
+VNLog(afs_int32 aop, afs_int32 anparms, afs_int32 av1, afs_int32 av2, 
+      afs_int32 av3, afs_int32 av4)
 {
     register afs_int32 temp;
     afs_int32 data[4];
@@ -487,7 +486,7 @@ VGetFreeVnode_r(struct VnodeClassInfo * vcp)
     if (Vn_refcount(vnp) != 0 || CheckLock(&vnp->lock))
 	Abort("VGetFreeVnode_r: locked vnode in lruq");
 #endif
-    VNLog(1, 2, Vn_id(vnp), (afs_int32) vnp);
+    VNLog(1, 2, Vn_id(vnp), (afs_int32) vnp, 0, 0);
 
     /* 
      * it's going to be overwritten soon enough.
@@ -654,13 +653,13 @@ VAllocVnode_r(Error * ec, Volume * vp, VnodeType type)
      */
 
  vnrehash:
-    VNLog(2, 1, vnodeNumber);
+    VNLog(2, 1, vnodeNumber, 0, 0, 0);
     /* Prepare to move it to the new hash chain */
     vnp = VLookupVnode(vp, vnodeNumber);
     if (vnp) {
 	/* slot already exists.  May even not be in lruq (consider store file locking a file being deleted)
 	 * so we may have to wait for it below */
-	VNLog(3, 2, vnodeNumber, (afs_int32) vnp);
+	VNLog(3, 2, vnodeNumber, (afs_int32) vnp, 0, 0);
 
 	VnCreateReservation_r(vnp);
 	if (Vn_refcount(vnp) == 1) {
@@ -817,13 +816,13 @@ VAllocVnode_r(Error * ec, Volume * vp, VnodeType type)
 
 	}
     sane:
-	VNLog(4, 2, vnodeNumber, (afs_int32) vnp);
+	VNLog(4, 2, vnodeNumber, (afs_int32) vnp, 0, 0);
 #ifndef AFS_DEMAND_ATTACH_FS
 	AddToVnHash(vnp);
 #endif
     }
 
-    VNLog(5, 1, (afs_int32) vnp);
+    VNLog(5, 1, (afs_int32) vnp, 0, 0, 0);
     memset(&vnp->disk, 0, sizeof(vnp->disk));
     vnp->changed_newTime = 0;	/* set this bit when vnode is updated */
     vnp->changed_oldTime = 0;	/* set this on CopyOnWrite. */
@@ -1119,7 +1118,7 @@ VGetVnode_r(Error * ec, Volume * vp, VnodeId vnodeNumber, int locktype)
 	return NULL;
     }
 
-    VNLog(100, 1, vnodeNumber);
+    VNLog(100, 1, vnodeNumber, 0, 0, 0);
 
 #ifdef AFS_DEMAND_ATTACH_FS
     /*
@@ -1167,7 +1166,7 @@ VGetVnode_r(Error * ec, Volume * vp, VnodeId vnodeNumber, int locktype)
     if (vnp) {
 	/* vnode is in cache */
 
-	VNLog(101, 2, vnodeNumber, (afs_int32) vnp);
+	VNLog(101, 2, vnodeNumber, (afs_int32) vnp, 0, 0);
 	VnCreateReservation_r(vnp);
 
 #ifdef AFS_DEMAND_ATTACH_FS
@@ -1254,7 +1253,7 @@ VGetVnode_r(Error * ec, Volume * vp, VnodeId vnodeNumber, int locktype)
 
     /* Check that the vnode hasn't been removed while we were obtaining
      * the lock */
-    VNLog(102, 2, vnodeNumber, (afs_int32) vnp);
+    VNLog(102, 2, vnodeNumber, (afs_int32) vnp, 0, 0);
     if ((vnp->disk.type == vNull) || (Vn_cacheCheck(vnp) == 0)) {
 	VnUnlock(vnp, locktype);
 	VnCancelReservation_r(vnp);
@@ -1317,7 +1316,7 @@ VPutVnode_r(Error * ec, register Vnode * vnp)
     class = vnodeIdToClass(Vn_id(vnp));
     vcp = &VnodeClassInfo[class];
     assert(vnp->disk.vnodeMagic == vcp->magic);
-    VNLog(200, 2, Vn_id(vnp), (afs_int32) vnp);
+    VNLog(200, 2, Vn_id(vnp), (afs_int32) vnp, 0, 0);
 
 #ifdef AFS_DEMAND_ATTACH_FS
     writeLocked = (Vn_state(vnp) == VN_STATE_EXCLUSIVE);
@@ -1336,7 +1335,7 @@ VPutVnode_r(Error * ec, register Vnode * vnp)
 	VNLog(201, 2, (afs_int32) vnp,
 	      ((vnp->changed_newTime) << 1) | ((vnp->
 						changed_oldTime) << 1) | vnp->
-	      delete);
+	      delete, 0, 0);
 	if (thisProcess != vnp->writer)
 	    Abort("VPutVnode: Vnode at 0x%x locked by another process!\n",
 		  vnp);
@@ -1351,7 +1350,7 @@ VPutVnode_r(Error * ec, register Vnode * vnp)
 		/* No longer any directory entries for this vnode. Free the Vnode */
 		memset(&vnp->disk, 0, sizeof(vnp->disk));
 		/* delete flag turned off further down */
-		VNLog(202, 2, Vn_id(vnp), (afs_int32) vnp);
+		VNLog(202, 2, Vn_id(vnp), (afs_int32) vnp, 0, 0);
 	    } else if (vnp->changed_newTime) {
 		vnp->disk.serverModifyTime = now;
 	    }
@@ -1458,7 +1457,7 @@ VVnodeWriteToRead_r(Error * ec, register Vnode * vnp)
     class = vnodeIdToClass(Vn_id(vnp));
     vcp = &VnodeClassInfo[class];
     assert(vnp->disk.vnodeMagic == vcp->magic);
-    VNLog(300, 2, Vn_id(vnp), (afs_int32) vnp);
+    VNLog(300, 2, Vn_id(vnp), (afs_int32) vnp, 0, 0);
 
 #ifdef AFS_DEMAND_ATTACH_FS
     writeLocked = (Vn_state(vnp) == VN_STATE_EXCLUSIVE);
@@ -1473,7 +1472,7 @@ VVnodeWriteToRead_r(Error * ec, register Vnode * vnp)
     VNLog(301, 2, (afs_int32) vnp,
 	  ((vnp->changed_newTime) << 1) | ((vnp->
 					    changed_oldTime) << 1) | vnp->
-	  delete);
+	  delete, 0, 0);
 
     /* sanity checks */
 #ifdef AFS_PTHREAD_ENV
