@@ -11,12 +11,13 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29.2.3 2006/10/13 13:09:07 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29.2.13 2008/10/27 23:41:47 shadow Exp $");
 
 #include <sys/types.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
+#include <string.h>
 #ifdef AFS_NT40_ENV
 #include <fcntl.h>
 #else
@@ -26,13 +27,6 @@ RCSID
 #include <netinet/in.h>
 #include <unistd.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <sys/stat.h>
 #ifdef AFS_PTHREAD_ENV
 #include <assert.h>
@@ -83,7 +77,7 @@ struct iod {
     struct rx_call *call;	/* call to which to write, might be an array */
     int device;			/* dump device ID for volume */
     int parentId;		/* dump parent ID for volume */
-    struct DiskPartition *dumpPartition;	/* Dump partition. */
+    struct DiskPartition64 *dumpPartition;	/* Dump partition. */
     struct rx_call **calls;	/* array of pointers to calls */
     int ncalls;			/* how many calls/codes in array */
     int *codes;			/* one return code for each call */
@@ -525,9 +519,10 @@ static int
 DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 {
     int code = 0, error = 0;
-    afs_int32 pad = 0, offset;
+    afs_int32 pad = 0;
+    afs_int32 offset = 0;
     afs_sfsize_t n, nbytes, howMany, howBig;
-    afs_foff_t lcode;
+    afs_foff_t lcode = 0;
     byte *p;
 #ifndef AFS_NT40_ENV
     struct afs_stat status;
@@ -994,11 +989,15 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 	    (size <=
 	     vcp->diskSize ? 0 : size - vcp->diskSize) >> vcp->logSize;
 	if (nVnodes > 0) {
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: Set up %d inodes for volume %d\n",
+		    nVnodes, V_id(vp));
+	    }
 	    Buf = (afs_int32 *) malloc(nVnodes * sizeof(afs_int32));
 	    if (Buf == NULL) {
 		STREAM_CLOSE(afile);
 		FDH_CLOSE(fdP);
-		return 1;
+		return -1;
 	    }
 	    memset((char *)Buf, 0, nVnodes * sizeof(afs_int32));
 	    STREAM_SEEK(afile, offset = vcp->diskSize, 0);
@@ -1013,6 +1012,9 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 		}
 		offset += vcp->diskSize;
 	    }
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: found %d inodes\n", cnt);
+	    }
 	    *Bufp = Buf;
 	    *sizep = nVnodes;
 	}
@@ -1035,12 +1037,13 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     struct iod iod;
     register struct iod *iodp = &iod;
     afs_int32 *b1 = NULL, *b2 = NULL;
-    int s1 = 0, s2 = 0, delo = incremental, tdelo;
+    int s1 = 0, s2 = 0, delo = 0, tdelo;
     int tag;
 
     iod_Init(iodp, call);
 
     vp = avp;
+
     if (!ReadDumpHeader(iodp, &header)) {
 	Log("1 Volser: RestoreVolume: Error reading header file for dump; aborted\n");
 	return VOLSERREAD_DUMPERROR;
@@ -1056,13 +1059,10 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
 	delo = ProcessIndex(vp, vLarge, &b1, &s1, 0);
     if (!delo)
 	delo = ProcessIndex(vp, vSmall, &b2, &s2, 0);
-    if (delo) {
-	if (b1)
-	    free((char *)b1);
-	if (b2)
-	    free((char *)b2);
-	b1 = b2 = NULL;
-	s1 = s2 = 0;
+    if (delo < 0) {
+	Log("1 Volser: RestoreVolume: ProcessIndex failed; not restored\n");
+	error = VOLSERREAD_DUMPERROR;
+	goto out;
     }
 
     strncpy(vol.name, cookie->name, VOLSER_OLDMAXVOLNAME);
@@ -1070,16 +1070,9 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     vol.cloneId = cookie->clone;
     vol.parentId = cookie->parent;
 
-
     tdelo = delo;
     while (1) {
-	int temprc;
-
-	V_linkHandle(avp)->ih_flags |= IH_DELAY_SYNC;	/* Avoid repetitive fdsync()s on linkfile */
-	temprc = ReadVnodes(iodp, vp, 0, b1, s1, b2, s2, tdelo);
-	V_linkHandle(avp)->ih_flags &= ~IH_DELAY_SYNC;	/* normal sync behaviour again */
-	IH_CONDSYNC(V_linkHandle(avp));			/* sync link file */
-	if (temprc) {
+	if (ReadVnodes(iodp, vp, 0, b1, s1, b2, s2, tdelo)) {
 	    error = VOLSERREAD_DUMPERROR;
 	    goto clean;
 	}
@@ -1109,8 +1102,8 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     if (!delo) {
 	delo = ProcessIndex(vp, vLarge, &b1, &s1, 1);
 	if (!delo)
-	    ProcessIndex(vp, vSmall, &b2, &s2, 1);
-	if (delo) {
+	    delo = ProcessIndex(vp, vSmall, &b2, &s2, 1);
+	if (delo < 0) {
 	    error = VOLSERREAD_DUMPERROR;
 	    goto clean;
 	}
@@ -1162,6 +1155,10 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 
 	if (!ReadInt32(iodp, &vnode->uniquifier))
 	    return VOLSERREAD_DUMPERROR;
+
+	if (DoLogging) {
+	    Log("ReadVnodes: setup %d/%d\n", vnodeNumber, vnode->uniquifier);
+	}
 	while ((tag = iod_getc(iodp)) > D_MAX && tag != EOF) {
 	    haveStuff = 1;
 	    switch (tag) {
@@ -1230,8 +1227,8 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 				  V_parentId(vp), vnodeNumber,
 				  vnode->uniquifier, vnode->dataVersion);
 		    if (!VALID_INO(ino)) {
-			perror("unable to allocate inode");
-			Log("1 Volser: ReadVnodes: Restore aborted\n");
+			Log("1 Volser: ReadVnodes: IH_CREATE: %s - restore aborted\n",
+                            afs_error_message(errno));
 			return VOLSERREAD_DUMPERROR;
 		    }
 		    nearInode = ino;
@@ -1239,6 +1236,8 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 		    IH_INIT(tmpH, vp->device, V_parentId(vp), ino);
 		    fdP = IH_OPEN(tmpH);
 		    if (fdP == NULL) {
+			Log("1 Volser: ReadVnodes: IH_OPEN: %s - restore aborted\n",
+                            afs_error_message(errno));
 			IH_RELEASE(tmpH);
 			return VOLSERREAD_DUMPERROR;
 		    }
