@@ -13,7 +13,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.42.2.14 2008/03/17 17:14:11 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.42.2.18 2008/09/25 08:58:22 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -230,9 +230,9 @@ ViceCreateRoot(Volume *vp)
     did.Vnode = (VnodeId) 1;
     did.Unique = 1;
 
-    assert(!(MakeDir(&dir, &did, &did)));
+    assert(!(MakeDir(&dir, (afs_int32 *)&did, (afs_int32 *)&did)));
     DFlush();			/* flush all modified dir buffers out */
-    DZap(&dir);			/* Remove all buffers for this dir */
+    DZap((afs_int32 *)&dir);			/* Remove all buffers for this dir */
     length = Length(&dir);	/* Remember size of this directory */
 
     FidZap(&dir);		/* Done with the dir handle obtained via SetSalvageDirHandle() */
@@ -352,7 +352,6 @@ SAFSVolNukeVolume(struct rx_call *acid, afs_int32 apartID, afs_int32 avolID)
 afs_int32
 VolNukeVolume(struct rx_call *acid, afs_int32 apartID, afs_int32 avolID)
 {
-    register char *tp;
     char partName[50];
     afs_int32 error;
     register afs_int32 code;
@@ -365,10 +364,8 @@ VolNukeVolume(struct rx_call *acid, afs_int32 apartID, afs_int32 avolID)
     if (DoLogging)
 	Log("%s is executing VolNukeVolume %u\n", caller, avolID);
 
-    tp = volutil_PartitionName(apartID);
-    if (!tp)
+    if (volutil_PartitionName2_r(apartID, partName, sizeof(partName)) != 0)
 	return VOLSERNOVOL;
-    strcpy(partName, tp);	/* remember it for later */
     /* we first try to attach the volume in update mode, so that the file
      * server doesn't try to use it (and abort) while (or after) we delete it.
      * If we don't get the volume, that's fine, too.  We just won't put it back.
@@ -449,7 +446,7 @@ VolCreateVolume(struct rx_call *acid, afs_int32 apart, char *aname,
 	return EIO;
     }
     V_uniquifier(vp) = 1;
-    V_creationDate(vp) = V_copyDate(vp);
+    V_updateDate(vp) = V_creationDate(vp) = V_copyDate(vp);
     V_inService(vp) = V_blessed(vp) = 1;
     V_type(vp) = atype;
     AssignVolumeName(&V_disk(vp), aname, 0);
@@ -846,6 +843,11 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	LogError(error);
 	goto fail;
     }
+    /* VUpdateVolume succeeded. Mark it in service so there's no window 
+     * between FSYNC_VOL_ON and VolSetFlags where it's offline with no  
+     * specialStatus; this is a reclone and this volume started online  
+     */
+    V_inService(clonevp) = 1;
     VDetachVolume(&error, clonevp);	/* allow file server to get it's hands on it */
     clonevp = NULL;
     VUpdateVolume(&error, originalvp);
@@ -1448,6 +1450,7 @@ VolSetForwarding(struct rx_call *acid, afs_int32 atid, afs_int32 anewsite)
 {
     register struct volser_trans *tt;
     char caller[MAXKTCNAMELEN];
+    char partName[16];
 
     if (!afsconf_SuperUser(tdir, acid, caller))
 	return VOLSERBAD_ACCESS;	/*not a super user */
@@ -1462,7 +1465,10 @@ VolSetForwarding(struct rx_call *acid, afs_int32 atid, afs_int32 anewsite)
     }
     strcpy(tt->lastProcName, "SetForwarding");
     tt->rxCallPtr = acid;
-    FSYNC_VolOp(tt->volid, NULL, FSYNC_VOL_MOVE, anewsite, NULL);
+    if (volutil_PartitionName2_r(tt->partition, partName, sizeof(partName)) != 0) {
+	partName[0] = '\0';
+    }
+    FSYNC_VolOp(tt->volid, partName, FSYNC_VOL_MOVE, anewsite, NULL);
     tt->rxCallPtr = (struct rx_call *)0;
     if (TRELE(tt))
 	return VOLSERTRELE_ERROR;
@@ -1825,22 +1831,26 @@ typedef struct {
  * fill in appropriate type of on-wire volume metadata structure.
  *
  * @param vp      pointer to volume object
- * @param hdr     pointer to volume disk data object
  * @param handle  pointer to wire format handle object
  *
+ * @pre vp object must contain header & pending_vol_op structurs (populate if from RPC)
  * @pre handle object must have a valid pointer and enumeration value
  *
+ * @note passing a NULL value for vp means that the fileserver doesn't
+ *       know about this particular volume, thus implying it is offline.
+ *
  * @return operation status
  *   @retval 0 success
  *   @retval 1 failure
  */
 static int
-FillVolInfo(Volume * vp, VolumeDiskData * hdr, volint_info_handle_t * handle)
+FillVolInfo(Volume * vp, volint_info_handle_t * handle)
 {
     unsigned int numStatBytes, now;
+    register struct VolumeDiskData *hdr = &vp->header->diskstuff;
 
     /*read in the relevant info */
-    strcpy(VOLINT_INFO_PTR(handle, name), hdr->name);
+    strcpy((char *)VOLINT_INFO_PTR(handle, name), hdr->name);
     VOLINT_INFO_STORE(handle, status, VOK);	/*its ok */
     VOLINT_INFO_STORE(handle, volid, hdr->id);
     VOLINT_INFO_STORE(handle, type, hdr->type);	/*if ro volume */
@@ -1871,15 +1881,35 @@ FillVolInfo(Volume * vp, VolumeDiskData * hdr, volint_info_handle_t * handle)
      * along with the blessed and inService flags from the header.
      *   -- tkeiser 11/27/2007
      */
-    if ((V_attachState(vp) == VOL_STATE_UNATTACHED) ||
+
+    /* Conditions that offline status is based on: 
+		volume is unattached state
+		volume state is in (one of several error states)
+		volume not in service
+		volume is not marked as blessed (not on hold)
+		volume in salvage req. state
+		volume needsSalvaged 
+		next op would set volume offline
+		next op would not leave volume online (based on several conditions)
+    */
+    if (!vp ||
+	(V_attachState(vp) == VOL_STATE_UNATTACHED) ||
 	VIsErrorState(V_attachState(vp)) ||
 	!hdr->inService ||
-	!hdr->blessed) {
+	!hdr->blessed || 
+	(V_attachState(vp) == VOL_STATE_SALVSYNC_REQ) ||
+	hdr->needsSalvaged ||
+	(vp->pending_vol_op && 
+		(vp->pending_vol_op->com.command == FSYNC_VOL_OFF || 
+		!VVolOpLeaveOnline_r(vp, vp->pending_vol_op) )
+	)
+	) {
 	VOLINT_INFO_STORE(handle, inUse, 0);
     } else {
 	VOLINT_INFO_STORE(handle, inUse, 1);
     }
 #else
+    /* offline status based on program type, where != fileServer enum (1) is offline */
     if (hdr->inUse == fileServer) {
 	VOLINT_INFO_STORE(handle, inUse, 1);
     } else {
@@ -1889,11 +1919,13 @@ FillVolInfo(Volume * vp, VolumeDiskData * hdr, volint_info_handle_t * handle)
 
 
     switch(handle->volinfo_type) {
+	/* NOTE: VOLINT_INFO_STORE not used in this section because values are specific to one volinfo_type */
     case VOLINT_INFO_TYPE_BASE:
 
 #ifdef AFS_DEMAND_ATTACH_FS
 	/* see comment above where we set inUse bit */
-	if (hdr->needsSalvaged || VIsErrorState(V_attachState(vp))) {
+	if (hdr->needsSalvaged || 
+	    (vp && VIsErrorState(V_attachState(vp)))) {
 	    handle->volinfo_ptr.base->needsSalvaged = 1;
 	} else {
 	    handle->volinfo_ptr.base->needsSalvaged = 0;
@@ -1943,28 +1975,44 @@ FillVolInfo(Volume * vp, VolumeDiskData * hdr, volint_info_handle_t * handle)
  * get struct Volume out of the fileserver.
  *
  * @param[in] volumeId  volumeId for which we want state information
- * @param[out] vp       pointer to Volume object
+ * @param[in] pname     partition name string
+ * @param[inout] vp     pointer to pointer to Volume object which 
+ *                      will be populated (see note)
  *
  * @return operation status
- *   @retval 0 success
- *   @retval nonzero failure
+ *   @retval 0         success
+ *   @retval non-zero  failure
+ *
+ * @note if FSYNC_VolOp fails in certain ways, *vp will be set to NULL
+ *
+ * @internal
  */
 static int
-GetVolObject(afs_uint32 volumeId, Volume * vp)
+GetVolObject(afs_uint32 volumeId, char * pname, Volume ** vp)
 {
     int code;
     SYNC_response res;
 
     res.hdr.response_len = sizeof(res.hdr);
-    res.payload.buf = vp;
-    res.payload.len = sizeof(*vp);
+    res.payload.buf = *vp;
+    res.payload.len = sizeof(Volume);
 
     code = FSYNC_VolOp(volumeId,
-		       "",
+		       pname,
 		       FSYNC_VOL_QUERY,
 		       0,
 		       &res);
 
+    if (code != SYNC_OK) {
+	switch (res.hdr.reason) {
+	case FSYNC_WRONG_PART:
+	case FSYNC_UNKNOWN_VOLID:
+	    *vp = NULL;
+	    code = SYNC_OK;
+	    break;
+	}
+    }
+
     return code;
 }
 
@@ -2001,9 +2049,21 @@ GetVolInfo(afs_uint32 partId,
 	   vol_info_list_mode_t mode)
 {
     int code = -1;
+    int reason;
     afs_int32 error;
     struct volser_trans *ttc = NULL;
-    struct Volume fs_tv, *tv = NULL;
+    struct Volume *fill_tv, *tv = NULL;
+#ifdef AFS_DEMAND_ATTACH_FS
+    struct Volume fs_tv_buf, *fs_tv = &fs_tv_buf; /* Create a structure, and a pointer to that structure */
+    SYNC_PROTO_BUF_DECL(fs_res_buf); /* Buffer for the pending_vol_op */
+    SYNC_response fs_res; /* Response handle for the pending_vol_op */
+    FSSYNC_VolOp_info pending_vol_op_res; /* Pending vol ops to full in volume */
+
+    /* Set up response handle for pending_vol_op */
+    fs_res.hdr.response_len = sizeof(fs_res.hdr);
+    fs_res.payload.buf = fs_res_buf;
+    fs_res.payload.len = SYNC_PROTO_MAX_LEN;
+#endif
 
     ttc = NewTrans(volumeId, partId);
     if (!ttc) {
@@ -2013,6 +2073,7 @@ GetVolInfo(afs_uint32 partId,
 	goto drop;
     }
 
+    /* Get volume from volserver */
     tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
     if (error) {
 	Log("1 Volser: GetVolInfo: Could not attach volume %u (%s:%s) error=%d\n", 
@@ -2046,23 +2107,40 @@ GetVolInfo(afs_uint32 partId,
 	/*this volume will be salvaged */
 	Log("1 Volser: GetVolInfo: Volume %u (%s:%s) needs to be salvaged\n", 
 	    volumeId, pname, volname);
-	goto drop;
     }
 
 #ifdef AFS_DEMAND_ATTACH_FS
-    if (GetVolObject(volumeId, &fs_tv)) {
+    /* If using DAFS, get volume from fsserver */
+    if (GetVolObject(volumeId, pname, &fs_tv) != SYNC_OK) {
 	goto drop;
     }
+
+    /* fs_tv is a shallow copy, must populate certain structures before passing along */
+    if (FSYNC_VolOp(volumeId, pname, FSYNC_VOL_QUERY_VOP, 0, &fs_res) == SYNC_OK) { 
+	/* If we if the pending vol op */
+	memcpy(&pending_vol_op_res, fs_res.payload.buf, sizeof(FSSYNC_VolOp_info));
+	fs_tv->pending_vol_op=&pending_vol_op_res;
+    } else {
+	fs_tv->pending_vol_op=NULL;
+    }
+
+    /* populate the header from the volserver copy */
+    fs_tv->header=tv->header;
+
+    /* When using DAFS, use the fs volume info, populated with required structures */
+    fill_tv = fs_tv;
+#else 
+    /* When not using DAFS, just use the local volume info */
+    fill_tv = tv;
 #endif
 
     /* ok, we have all the data we need; fill in the on-wire struct */
-    code = FillVolInfo(&fs_tv, &tv->header->diskstuff, handle);
-
+    code = FillVolInfo(fill_tv, handle);
 
  drop:
     if (code == -1) {
 	VOLINT_INFO_STORE(handle, status, 0);
-	strcpy(VOLINT_INFO_PTR(handle, name), volname);
+	strcpy((char *)VOLINT_INFO_PTR(handle, name), volname);
 	VOLINT_INFO_STORE(handle, volid, volumeId);
     }
     if (tv) {
@@ -2070,7 +2148,7 @@ GetVolInfo(afs_uint32 partId,
 	tv = NULL;
 	if (error) {
 	    VOLINT_INFO_STORE(handle, status, 0);
-	    strcpy(VOLINT_INFO_PTR(handle, name), volname);
+	    strcpy((char *)VOLINT_INFO_PTR(handle, name), volname);
 	    Log("1 Volser: GetVolInfo: Could not detach volume %u (%s:%s)\n",
 		volumeId, pname, volname);
 	}
@@ -2113,6 +2191,8 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid, afs_int32
     volumeInfo->volEntries_val = (volintInfo *) malloc(sizeof(volintInfo));
     if (!volumeInfo->volEntries_val)
 	return ENOMEM;
+    memset(volumeInfo->volEntries_val, 0, sizeof(volintInfo)); /* Clear structure */
+
     pntr = volumeInfo->volEntries_val;
     volumeInfo->volEntries_len = 1;
     if (GetPartName(partid, pname))
@@ -2218,6 +2298,8 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	(volintXInfo *) malloc(sizeof(volintXInfo));
     if (!a_volumeXInfoP->volXEntries_val)
 	return ENOMEM;
+    memset(a_volumeXInfoP->volXEntries_val, 0, sizeof(volintXInfo)); /* Clear structure */
+
     xInfoP = a_volumeXInfoP->volXEntries_val;
     a_volumeXInfoP->volXEntries_len = 1;
     code = ENODEV;
@@ -2325,6 +2407,8 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 	(volintInfo *) malloc(allocSize * sizeof(volintInfo));
     if (!volumeInfo->volEntries_val)
 	return ENOMEM;
+    memset(volumeInfo->volEntries_val, 0, sizeof(volintInfo)); /* Clear structure */
+
     pntr = volumeInfo->volEntries_val;
     volumeInfo->volEntries_len = 0;
     if (GetPartName(partid, pname))
@@ -2456,6 +2540,8 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	(volintXInfo *) malloc(allocSize * sizeof(volintXInfo));
     if (!a_volumeXInfoP->volXEntries_val)
 	return ENOMEM;
+    memset(a_volumeXInfoP->volXEntries_val, 0, sizeof(volintXInfo)); /* Clear structure */
+
     xInfoP = a_volumeXInfoP->volXEntries_val;
     a_volumeXInfoP->volXEntries_len = 0;
 
