@@ -18,20 +18,13 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/xstat/xstat_fs.c,v 1.10.2.1 2006/06/26 16:20:50 rra Exp $");
+    ("$Header: /cvs/openafs/src/xstat/xstat_fs.c,v 1.10.2.3 2008/03/10 22:32:37 shadow Exp $");
 
 #include "xstat_fs.h"		/*Interface for this module */
 #include <lwp.h>		/*Lightweight process package */
 
 #include <afs/afsutil.h>
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #define LWP_STACK_SIZE	(16 * 1024)
 
@@ -245,8 +238,8 @@ xstat_fs_Cleanup(int a_releaseMem)
  *	Nothing interesting.
  *------------------------------------------------------------------------*/
 
-static void
-xstat_fs_LWP()
+static void *
+xstat_fs_LWP(void *unused)
 {
     static char rn[] = "xstat_fs_LWP";	/*Routine name */
     register afs_int32 code;	/*Results of calls */
@@ -395,6 +388,7 @@ xstat_fs_LWP()
 			code);
 	}			/*Continuous execution */
     }				/*Service loop */
+    return NULL;
 }
 
 /*------------------------------------------------------------------------
